# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# Action navigation_filters
#
# Edite les differentes regles de filtrage
#
###########################################################################

""" Action de gestion des filtres web sur un amon """

from twisted.python import log
from ead2.backend.lib.action import * # import des actions et des types
from ead2.backend.actions import tools
import ead2.backend.actions.amon.amon_tools as t
from ead2.backend.actions.amon import forms
from ead2.backend.actions.amon import whitesitelist_forms
from ead2.backend.actions.amon import menus
from ead2.backend.config.config import SCRIPT_DIR, CONFIG_DIR
from ead2.backend.config.filenames import navigation_filters
from ead2.lib.libead import uni
import commands, os
from os.path import isfile, join
from ead2.lib.error import MissingKey, MissingValue

# message
page_msgs = {'filtres':"Activation des filtres facultatifs",
             'filtrage':"Règles de filtrage pour la navigation",
             'blacklist':"Gestion des sites interdits",
             'whitelist':"Gestion des sites autorisés",
             "extensions":"Interdictions d'extension",
             "mime":"Interdiction de type mime",
             "signalement": "Signalement de sites et d'URL"}

# templates utilisés
used_templates = ['main', 'form', 'formselect', 'accordion']

signalement_msg = navigation_filters['signalement_msg']

class FiltresWebPedago(Action):
    """ gestion des filtres optionnels de dansguardian pour la zone pedago"""
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'opt_filters_pedago'
    libelle = "Sites"
    description = "Activation des filtres facultatifs sur la zone de configuration 2"
    category = "Configuration 2"
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend",
                    keys=['server', 'action'])
    form_result = Dict(default={}, doc="retour formulaire de la requete en cours coté frontend",
                        keys = ['opt_filters_form'])
    def execute(self):
        """ renvoie les données pour la mise en forme
        """
        params, self.server_nb = tools.get_request(self.request)
        if self.form_result.has_key('opt_filters_form'):
            try:
                datas = forms.valid_opt_filters_form(self.form_result['opt_filters_form'],
                                                     'pedago')
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name,
                                                     confirm=False)
            except Exception, msg:
                import traceback
                traceback.print_exc()
                datas = dict(message=msg.message)
            self.form_result = {}
            return self.send_frag(datas, template='amon_opt_filters', templates=used_templates)
        result = {'titre': self.description, 'sstitre':''}
        result['menus'] = menus.get_filters_menus(self.server_nb, 'filtres', 'pedago')
        result.update(forms.get_opt_filters_form('pedago'))
        result['validate'] = self.valid_btn()
        return self.send_all(result, template='amon_opt_filters', templates=used_templates)

    def valid_btn(self):
        """ renvoie les données pour la mise en forme du bouton valider """
        href = tools.make_form_link(self.server_nb, self.name,
                                    True,
                                    ['opt_filters_form'],
                                    'msg_container')
        title = 'Modifier la configuration des filtres'
        libelle = 'Valider'
        icone = '/image/ok.gif'
        return dict(href=href,
                    title=title,
                    libelle=libelle,
                    icone=icone)

class FiltragePedago(Action):
    """ réglage du mode de filtrage """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'filtrage_pedago'
    libelle = "Règles de filtrage pour la navigation sur la zone de configuration 2"
    description = "Règles de filtrage pour la navigation sur la zone de configuration 2"
    category = None
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend",
                    keys=['server', 'action'])
    form_result = Dict(default={}, doc="retour formulaire de la requete en cours coté frontend",
                        keys = ['filtrage_form'])
    def execute(self):
        """ renvoie les données pour la mise en forme
        """
        params, self.server_nb = tools.get_request(self.request)
        if self.form_result.has_key('filtrage_form'):
            try:
                datas = forms.valid_filtrage_form(self.form_result['filtrage_form'], 'pedago')
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name,
                                                     confirm=False)
            except Exception, msg:
                import traceback
                traceback.print_exc()
                datas = dict(message=msg.message)
            self.form_result = {}
            return self.send_frag(datas, template='amon_filtrage', templates=used_templates)
        result = {'titre': self.description, 'sstitre':''}
        result['menus'] = menus.get_filters_menus(self.server_nb, 'filtrage', 'pedago')
        result.update(forms.get_filtrage_form('pedago'))
        result['validate'] = self.valid_btn()
        return self.send_all(result, template='amon_filtrage', templates=used_templates)

    def valid_btn(self):
        """ renvoie les données pour la mise en forme du bouton valider """
        href = tools.make_form_link(self.server_nb, self.name,
                                    True,
                                    ['filtrage_form'],
                                    'msg_container')
        title = 'Modifier les règles de filtrage'
        libelle = 'Valider'
        icone = '/image/ok.gif'
        return dict(href=href,
                    title=title,
                    libelle=libelle,
                    icone=icone)


class SitesInterditsPedago(Action):
    """ ajout de sites interdits à la liste """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'sites_interdits_pedago'
    libelle = "Gestion des sites interdits sur la zone de configuration 2"
    description = 'Ajouter des sites à interdire sur la zone de configuration 2'
    category = None
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend",
                    keys=['server', 'action'])
    form_result = Dict(default={}, doc="retour formulaire de la requete en cours coté frontend",
                        keys = ['sitelist_form', 'site_names', 'site_table'])

    def execute(self):
        """ renvoie les données pour la mise en forme
        """
        params, self.server_nb = tools.get_request(self.request)

        if self.form_result.has_key('sitelist_form'):
            try:
                datas = forms.valid_sitelist_form(self.form_result['sitelist_form'],
                                                   'pedago',
                                                   'blacklist'
                                                   )
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name,
                                                     confirm=False)
            except Exception, msg:
                import traceback
                traceback.print_exc()
                datas = dict(message=msg.message)
            self.form_result = {}
            return self.send_frag(datas, template='amon_sitelist', templates=used_templates)

        elif self.form_result.has_key('site_table') and self.form_result.has_key('site_names'):
            try:
                datas = forms.valid_mod_list(self.form_result['site_names'],
                                                  self.form_result['site_table'],
                                                  'pedago',
                                                  'blacklist')
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name,
                                                     confirm=False)
            except Exception, msg:
                import traceback
                traceback.print_exc()
                datas = dict(message=msg.message)
            self.form_result = {}
            return self.send_frag(datas, template='amon_sitelist', templates=used_templates)

        result = {'titre': self.description, 'sstitre':''}
        result['menus'] = menus.get_filters_menus(self.server_nb, 'blacklist', 'pedago')
        result.update(forms.get_blacklist_form('pedago'))
        result['validate'] = self.valid_btn()
        result.update(forms.get_listed('pedago', 'blacklist'))
        result['validate_mod'] = self.valid_mod_btn()
        return self.send_all(result, template='amon_sitelist', templates=used_templates)

    def valid_btn(self):
        """ renvoie les données pour la mise en forme du bouton valider """
        href = tools.make_form_link(self.server_nb, self.name,
                                    True,
                                    ['sitelist_form'],
                                    'msg_container')
        title = 'Ajouter des sites à la liste de sites interdits'
        libelle = 'Valider'
        icone = '/image/ok.gif'
        return dict(href=href,
                    title=title,
                    libelle=libelle,
                    icone=icone)

    def valid_mod_btn(self):
        """ renvoie les données pour la mise en forme du bouton valider
            (de la section de modification de blacklist locale)
        """
        href = tools.make_form_link(self.server_nb, self.name,
                                    True,
                                    ['site_names',
                                        'site_table'
                                        ],
                                    'msg_container')
        title = 'Modifier la liste de sites interdits'
        libelle = 'Valider'
        icone = '/image/ok.gif'
        return dict(href=href,
                    title=title,
                    libelle=libelle,
                    icone=icone)

class SitesAutorisesPedago(Action):
    """ ajout de sites autorisés à la liste """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'sites_autorises_pedago'
    libelle = "Gestion des sites autorisés sur la zone de configuration 2"
    description = 'Ajouter des sites à autoriser sur la zone de configuration 2'
    category = None
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend",
                    keys=['server', 'action'])
    form_result = Dict(default={}, doc="retour formulaire de la requete en cours coté frontend",
                       keys = ['sitelist_form', 'site_table', 'site_names'])

    def execute(self):
        """ renvoie les données pour la mise en forme
        """
        params, self.server_nb = tools.get_request(self.request)

        if self.form_result.has_key('sitelist_form'):
            try:
                datas = forms.valid_sitelist_form(self.form_result['sitelist_form'],
                                                    'pedago',
                                                    'whitelist'
                                                    )
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name,
                                                     confirm=False)
            except Exception, msg:
                import traceback
                traceback.print_exc()
                datas = dict(message=msg.message)
            self.form_result = {}
            return self.send_frag(datas, template='amon_sitelist', templates=used_templates)

        elif self.form_result.has_key('site_table') and self.form_result.has_key('site_names'):
            try:
                datas = forms.valid_mod_list(self.form_result['site_names'],
                                                  self.form_result['site_table'],
                                                  'pedago',
                                                  'whitelist')
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name,
                                                     confirm=False)
            except Exception, msg:
                import traceback
                traceback.print_exc()
                datas = dict(message=msg.message)
            self.form_result = {}
            return self.send_frag(datas, template='amon_sitelist', templates=used_templates)

        result = {'titre': self.description, 'sstitre':''}
        result['menus'] = menus.get_filters_menus(self.server_nb, 'whitelist', 'pedago')
        result.update(forms.get_whitelist_form('pedago'))
        result['validate'] = self.valid_btn()
        result.update(forms.get_listed('pedago', 'whitelist'))
        result['validate_mod'] = self.valid_mod_btn()
        return self.send_all(result, template='amon_sitelist', templates=used_templates)

    def valid_btn(self):
        """ renvoie les données pour la mise en forme du bouton valider """
        href = tools.make_form_link(self.server_nb, self.name,
                                    True,
                                    ['sitelist_form'],
                                    'msg_container')
        title = 'Ajouter des sites à la liste de sites autorisés'
        libelle = 'Valider'
        icone = '/image/ok.gif'
        return dict(href=href,
                    title=title,
                    libelle=libelle,
                    icone=icone)

    def valid_mod_btn(self):
        """ renvoie les données pour la mise en forme du bouton valider
            (de la section de modification de blacklist locale)
        """
        href = tools.make_form_link(self.server_nb, self.name,
                                    True,
                                    ['site_names',
                                     'site_table'],
                                    'msg_container')
        title = 'Modifier la liste de sites interdits'
        libelle = 'Valider'
        icone = '/image/ok.gif'
        return dict(href=href,
                    title=title,
                    libelle=libelle,
                    icone=icone)

class ExtensionsPedago(Action):
    """ gestion des extensions interdites sur la zone de configuration 2 """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'extensions_pedago'
    libelle = 'Extensions'
    description = 'Ajouter des extensions à interdire sur la zone de configuration 2'
    category = None
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend",
                    keys=['server', 'action'])
    form_result = Dict(default={}, doc="retour formulaire de la requete en cours coté frontend",
                        keys = ['extension_form', 'extlist_names', 'ext_table'])

    def execute(self):
        """ renvoie les données pour la mise en forme
        """
        params, self.server_nb = tools.get_request(self.request)

        if self.form_result.has_key('extension_form'):
            try:
                datas = forms.valid_extension_form(self.form_result['extension_form'], 'pedago')
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name,
                                                     confirm=False)
            except Exception, msg:
                import traceback
                traceback.print_exc()
                datas = dict(message=msg.message)
            self.form_result = {}
            return self.send_frag(datas, template='amon_extension', templates=used_templates)

        elif self.form_result.has_key('extlist_names') and self.form_result.has_key('ext_table'):
            try:
                datas = forms.valid_mod_list(self.form_result['extlist_names'],
                                                  self.form_result['ext_table'],
                                                  'pedago',
                                                  'extension')
                datas['toexec'] = tools.make_js_link(self.server_nb,
                                                     self.name,
                                                     confirm=False)
            except Exception, msg:
                import traceback
                traceback.print_exc()
                datas = dict(message=msg.message)
            self.form_result = {}
            return self.send_frag(datas, template='amon_extension', templates=used_templates)

        result = {'titre': self.description, 'sstitre':''}
        result['menus'] = menus.get_filters_menus(self.server_nb, 'extension', 'pedago')
        result.update(forms.get_extension_form('pedago'))
        result['validate'] = self.valid_btn()
        result.update(forms.get_forbidden_extension('pedago'))
        result['validate_mod'] = self.valid_mod_btn()
        return self.send_all(result, template='amon_extension', templates=used_templates)

    def valid_btn(self):
        """ renvoie les données pour la mise en forme du bouton valider """
        href = tools.make_form_link(self.server_nb, self.name,
                                    True,
                                    ['extension_form'],
                                    'msg_container')
        title = 'Ajouter des sites à la liste des extensions à interdire'
        libelle = 'Valider'
        icone = '/image/ok.gif'
        return dict(href=href,
                    title=title,
                    libelle=libelle,
                    icone=icone)

    def valid_mod_btn(self):
        """ renvoie les données pour la mise en forme du bouton valider
        """
        href = tools.make_form_link(self.server_nb, self.name,
                                    True,
                                    ['extlist_names',
                                     'ext_table'],
                                    'msg_container')
        title = 'Modifier la liste de sites interdits'
        libelle = 'Valider'
        icone = '/image/ok.gif'
        return dict(href=href,
                    title=title,
                    libelle=libelle,
                    icone=icone)

class MimePedago(Action):
    """ gestion des mime interdites sur la zone de configuration 2 """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'mime_pedago'
    libelle = 'Types Mime'
    description = 'Ajouter des types mimes à interdire sur la zone de configuration 2'
    category = None
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend",
                    keys=['server', 'action'])
    form_result = Dict(default={}, doc="retour formulaire de la requete en cours coté frontend",
                        keys = ['mime_form', 'mime_names', 'mime_table'])

    def execute(self):
        """ renvoie les données pour la mise en forme
        """
        params, self.server_nb = tools.get_request(self.request)

        if self.form_result.has_key('mime_form'):
            try:
                datas = forms.valid_mime_form(self.form_result['mime_form'], 'pedago')
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name,
                                                     confirm=False)
            except Exception, msg:
                import traceback
                traceback.print_exc()
                datas = dict(message=msg.message)
            self.form_result = {}
            return self.send_frag(datas, template='amon_mime', templates=used_templates)

        elif self.form_result.has_key('mime_names') and self.form_result.has_key('mime_table'):
            try:
                datas = forms.valid_mod_list(self.form_result['mime_names'],
                                                  self.form_result['mime_table'],
                                                  'pedago',
                                                  'mime')
                datas['toexec'] = tools.make_js_link(self.server_nb,
                                                     self.name,
                                                     confirm=False)
            except Exception, msg:
                import traceback
                traceback.print_exc()
                datas = dict(message=msg.message)
            self.form_result = {}
            return self.send_frag(datas, template='amon_mime', templates=used_templates)

        result = {'titre': self.description, 'sstitre':''}
        result['menus'] = menus.get_filters_menus(self.server_nb, 'mime', 'pedago')
        result.update(forms.get_mime_form('pedago'))
        result['validate'] = self.valid_btn()
        result.update(forms.get_forbidden_mimes('pedago'))
        result['validate_mod'] = self.valid_mod_btn()
        return self.send_all(result, template='amon_mime', templates=used_templates)

    def valid_btn(self):
        """ renvoie les données pour la mise en forme du bouton valider """
        href = tools.make_form_link(self.server_nb, self.name,
                                    True,
                                    ['mime_form'],
                                    'msg_container')
        title = 'Ajouter des types mimes à la liste des types mimes à interdire'
        libelle = 'Valider'
        icone = '/image/ok.gif'
        return dict(href=href,
                    title=title,
                    libelle=libelle,
                    icone=icone)

    def valid_mod_btn(self):
        """ renvoie les données pour la mise en forme du bouton valider
        """
        href = tools.make_form_link(self.server_nb, self.name,
                                    True,
                                    ['mime_names',
                                        'mime_table'],
                                    'msg_container')
        title = 'Modifier la liste de types mimes à interdire.'
        libelle = 'Valider'
        icone = '/image/ok.gif'
        return dict(href=href,
                    title=title,
                    libelle=libelle,
                    icone=icone)

class Signalement(Action):
    """ page de signalement """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'signalement'
    libelle = 'Signalement'
    description = 'Signalement'
    category = 'Outils'
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend",
                    keys=['server', 'action'])

    def execute(self):
        """ renvoie les données pour la mise en forme
        """
        params, self.server_nb = tools.get_request(self.request)
        result = {'titre': self.description, 'sstitre':''}
        result.update(self.get_signalement_form())
        return self.send_all(result, template='amon_signalement', templates=used_templates)

    def get_signalement_form(self):
        """ renvoie les données pour la page de signalement """
        guide = {'href':"javascript:OuvreFenetre('http://www.educnet.education.fr/aiedu/guide1.htm', 'Educnet')",
                         'libelle':"Plus d'information",
                         'icone':"/image/back.png", 'title':"Plus d'information"}
        form_link = {'href':"javascript:OuvreFenetre('http://aiedu.education.fr/', 'Educnet');",
                     "libelle":"Accès direct au formulaire de déclaration de site",
                     "icone":"/image/back.png",
                     'title':'formulaire de déclaration de site'}
        msg = t.get_lines(signalement_msg)
        return dict(signalement_text=msg,
                    signalement_form_link=form_link,
                    signalement_guide=guide)

class NavigationWhiteSiteListpedago(Action):
    """ interface de gestion des utilisateurs(ips) en liste blanche
        sur l'instance 0 de dansguardian
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'navigation_whitesitelist_pedago'
    libelle = 'Sites liste blanche'
    description = "Placer des sites en liste blanche sur la zone de configuration 2"
    category = None
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend",
                    keys=['server', 'action'])
    form_result = Dict(default={}, doc="retour formulaire de la requete en cours coté frontend",
                        keys = ['list_suppr', 'list_add'])

    def execute(self):
        """ renvoit les données pour la mise en forme
        """
        params, self.server_nb = tools.get_request(self.request)

        if self.form_result.has_key('list_add'):
            try:
                datas = whitesitelist_forms.valid_add_greysite(
                                            self.form_result['list_add'],
                                            zone='pedago')
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name,
                                                                  confirm=True)
            except Exception, msg:
                traceback.print_exc()
                datas = dict(message=msg.message)
            self.form_result = {}
            return self.send_frag(datas, template='amon_whitesitelist',
                                         templates=used_templates)

        elif self.form_result.has_key('list_suppr'):
            try:
                datas = whitesitelist_forms.valid_mod_greysitelist(self.form_result['list_suppr'],
                                                       zone='pedago')
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name, confirm=True)
            except Exception, msg:
                traceback.print_exc()
                datas = dict(message=msg.message)
            self.form_result = {}
            return self.send_frag(datas, template='amon_whitesitelist',
                                         templates=used_templates)

        result = {'titre': self.description}
        result.update(whitesitelist_forms.get_add_site_form(zone='pedago'))
        result.update(whitesitelist_forms.get_greysitelist(zone='pedago'))
        result['sstitre'] =  ''
        result['menus'] = menus.get_filters_menus(self.server_nb,
                                          'whitesitelist',
                                          'pedago')
        result['validate'] = whitesitelist_forms.get_validate(self.server_nb, self.name)
        result['validate_mod'] = whitesitelist_forms.get_validate_mod(self.server_nb, self.name)
        return self.send_all(result, template='amon_whitesitelist',
                                     templates=used_templates)

