# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# Action securite
#
# Edite les regles de pare-feu, interdiction de poste, destinations,
# peer to peer
#
###########################################################################

""" Actions de gestion des paramètres de sécurité """
from twisted.python import log
from ead2.backend.lib.action import *
from ead2.backend.actions import tools
from ead2.backend.config.config import SCRIPT_DIR
import os, traceback
from os.path import isfile, join
from ead2.backend.config.filenames import securite
from ead2.backend.actions.amon import amon_tools as t
from ead2.backend.actions.amon import menus


# templates additionnels
used_templates = ['main', 'form', 'formselect']

#id html des formulaires à valider
form_name = {'regles':['mod_regles'], 'interdits':['list_add', 'current_page'],
            'horaire':['list_add', 'current_page']}

# nom des fichiers a editer
regles_filename = securite['regles']
#interdits_filename = securite['interdits']
p2p_filename = securite['p2p']
horaire_filename = securite['horaire']

def get_default_horaires():
    """ renvoie les horaires par defaut """
    return ["1 7 19","2 7 19", "3 7 19", "4 7 19", "5 7 19", "6 7 19","0 - -"]

def get_horaires():
    """ renvoie les horaires du pare-feu
        sous la forme ["numjour heuredeb heurefin", ...]
    """
    result = []
    if isfile(horaire_filename):
        horaires = file(horaire_filename, 'r').read().splitlines()
        if len(horaires) <7:
            horaires = get_default_horaires()
    else:
        horaires = get_default_horaires()
    return parse_horaires(horaires)

def parse_horaires(horaires):
    """ parse les horaires du pare-feu """
    result = []
    for i in horaires:
        horaire = i.split()
        if len(horaire) == 3:
            result.append((horaire[0], horaire[1], horaire[2],))
        else:
            # si il y a une erreur dans le fichier
            return parse_horaires(get_default_horaires())
    return result

def write_horaire(jours):
    """ ecrit les horaires du pare-feu """
    for jour in jours:
        try:
            deb = int(jour[1])
            end = int(jour[2])
        except:
            deb = 0
            end = 1
        if deb > end:
            raise Exception, "Erreur: l'horaire d'ouverture du firewall doit précéder l'heure de fermeture"
    a = file(horaire_filename, 'w')
    for jour in jours:
        a.write('%s %s %s \n'%(jour[0], jour[1], jour[2]))
    a.close()
    return True

def get_hours(act_hour='-'):
    """ recupere les heures """
    hours = [{'libelle':"--------", "name":"-"},
             {'libelle':"0h 01", "name":"0"}]
    for i in range(1,24):
        libelle = '%dh 00'%i
        value = '%d'%i
        hours.append({'libelle':libelle, "name":value})
    hours.append({'libelle':'23h 59', 'name':'24'})
    for i in hours:
        if i['name'] == act_hour:
            i['default'] = 'ok'
    return hours

class P2P(Action):
    """ interface de gestion du peer to peer """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = "peertopeer"
    libelle = "Règles du pare-feu"
    description = "Activation du peer to peer et horaires"
    category = "Configuration générale"
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend",
                    keys=['server', 'action', 'p2p'])

    def execute(self):
        """ renvoie les données pour la mise en forme
        """
        params, self.server_nb = tools.get_request(self.request)
        self.page = 'p2p'
        if params.has_key('p2p'):
            p2p_status = params['p2p'][0]
            self._valid_p2p(p2p_status)

        result =  {'titre': self.libelle}
        result['sstitre'] = ''
        result['menus'] = menus._get_menu(self.server_nb, self.page)
        result.update(self._get_p2p())
        return self.send_all(result, template='amon_p2p', templates=used_templates)

    def _get_p2p(self):
        """ renvoie la description du formulaire de gestion p2p """
        if isfile(p2p_filename):
            msg = "Les échanges de fichiers (peer to peer) sont","interdits"
            color = "#b22"
            validate = {"href":tools.make_js_link(self.server_nb, self.name,
                                                    p2p="autoriser"),
                       'libelle':"Autoriser",
                       'icone':"/image/ok.gif",
                       'title':"Autoriser le peer to peer"}
        else:
            msg = "Les échanges de fichiers (peer to peer) sont","autorisés"
            color = "#2b2"
            validate = {"href":tools.make_js_link(self.server_nb, self.name,
                                                    p2p="interdire"),
                        'libelle':"Interdire",
                        'icone':"/image/amon/refused.png",
                        'title':"Interdire le peer to peer"}
        return dict(ptpmsg=msg,
                    forbid_p2p=validate,
                    entete="Echange de fichier (peer to peer)",
                    color=color)

    def _valid_p2p(self, p2p_status):
        """ valide l'activation/désactivation du peer to peer {'p2p':autoriser/interdire}"""
        if p2p_status == 'autoriser':
            if isfile(p2p_filename):
                os.remove(p2p_filename)
            t.kill_p2p_stop()
        else:
            t.add_lines(p2p_filename, '  ')
            t.kill_p2p_start()
        return {}

class Destination(Action):
    """ interface de gestion des horaires de pare-feu """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = "destination_interdite"
    libelle = 'Postes'
    description = "Interdire l'accès à des destinations (niveau pare-feu)"
    category = 'Configuration'
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend",
                    keys=['server', 'action'])
    form_result = Dict(default={}, doc="retour formulaire de la requete en cours coté frontend",
                        keys = ['list_add', 'list_suppr'])

    def execute(self):
        """ renvoie les données pour la mise en forme
        """
        params, self.server_nb = tools.get_request(self.request)
        self.page = 'interdits'
        if self.form_result != {}:
             try:
                 datas = self._valid_interdits()
                 datas['toexec'] = tools.make_js_link(self.server_nb, self.name)
             except Exception, mess:
                 traceback.print_exc()
                 log.err("Erreur: securite.py, %s"%mess)
                 datas = dict(message=mess.message)
             self.form_result = {}
             return self.send_frag(datas, template='amon_destination_interdite', templates=used_templates)
        result =  {'titre': self.libelle}
        result['sstitre'] = ''
        result['menus'] = menus._get_menu(self.server_nb, self.page)
        result.update(self._get_interdits())
        if form_name.has_key(self.page):
            result['validate'] = {'href':tools.make_long_form_link(self.server_nb,
                                                                    self.name,
                                                                    True,
                                                                    form_name[self.page],
                                                                    balise='return_msg_container'),
                                  'icone':'/image/amon/ok.gif',
                                  'libelle':'Valider',
                                  'title':'Valider les changements'}
            result['validation'] = tools.make_key_form_link(self.server_nb,
                                                                self.name,
                                                                False,
                                                                form_name[self.page],
                                                                balise='return_msg_container')
        return self.send_all(result, template='amon_destination_interdite', templates=used_templates)

    def _get_interdits(self):
        """ renvoie la description du formulaire pour les interdictions de destination """
        forbid_ip = {'name':'ip', 'onblur':"testVoid('ip', 'error_ip')",
                'libelle':"Ip  ou nom de la machine de destination à interdire"}
        ips = []
        if isfile(interdits_filename):
            for ip in file(interdits_filename).read().splitlines():
                tag = {'name':ip, 'libelle':ip}
                ips.append(tag)
        forbidden_title = "Réautoriser une destination"
        if ips != []:
            forbidden_ips = {
                    'options':[{'name':'destinations interdites', 'options':ips}],
                    'libelle':"Veuillez sélectionner la ou les destinations à réautoriser",
                    'name':'suppr',
                    'multi':'ok'
                            }
            forbidden_validate= {'href':tools.make_long_form_link(self.server_nb,
                                                                  self.name,
                                                                  True,
                                                                  ['list_suppr'],
                                                                  balise='return_msg_container'),
                                 'icone':'/image/amon/supprimer_big.gif',
                                 'libelle':'Réautoriser',
                                 'title':'Réautoriser les adresses ou postes selectionnés'}
            return dict(forbid_ip=forbid_ip,
                        forbidden_ips=forbidden_ips,
                        forbidden_validate=forbidden_validate,
                        focus='ip',
                        forbidden_title=forbidden_title)
        else:
            forbidden_ips = {"msg": "Aucune interdiction d'accès n'est appliquée pour le moment"}
            return dict(forbid_ip=forbid_ip, forbidden_ips=forbidden_ips, focus='ip', forbidden_title=forbidden_title)

    def _valid_interdits(self):
        """ valide l'interdiction de poste ou sous réseau """
        if not self.form_result.has_key('list_add') and not self.form_result.has_key('list_suppr'):
            log.err("Erreur: securite.py, il manque des données pour l'interdiction de poste/sous-réseau.")
            raise Exception, "Erreur: il manque des données pour l'interdiction de poste/sous-réseau."

        # recuperation des postes deja interdits
        if isfile(interdits_filename):
            lines = file(interdits_filename).read().splitlines()
        else:
            lines = []

        if self.form_result.has_key('list_add'):
            val = tools.format_form_result(self.form_result['list_add'])['ip']
            t.add_lines(interdits_filename, val)
            message = "Les postes (ou sous-réseau) ont été interdits, le pare-feu est redémarré."
        elif self.form_result.has_key('list_suppr'):
            todel = tools.format_form_result(self.form_result['list_suppr'])['suppr']
            t.del_lines(interdits_filename, todel)
            message = "Les postes (ou sous-réseau) ont été réautorisés, le pare-feu est redémarré."
        if t.bastion_restart():
            return dict(message=message)
        else:
            log.err("Erreur: securite.py, une erreur s'est produite au lancement de 'service bastion restart'(2).")
            raise Exception, "Erreur: une erreur s'est produite au redémarrage du pare-feu."

class Horaire(Action):
    """ interface de gestion des horaires de pare-feu """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = "horaire"
    libelle = 'Horaires du pare-feu'
    description = "horaires d'activation du pare-feu"
    category = None
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend",
                    keys=['server', 'action'])
    form_result = Dict(default={}, doc="retour formulaire de la requete en cours coté frontend",
                        keys = ['list_add', 'list_suppr'])

    def execute(self):
        """ renvoie les données pour la mise en forme
        """
        params, self.server_nb = tools.get_request(self.request)
        self.page = 'horaire'
        if self.form_result != {}:
             try:
                 datas = self._valid_horaire()
                 datas['toexec'] = tools.make_js_link(self.server_nb, self.name)
             except Exception, mess:
                 traceback.print_exc()
                 log.err("Erreur: securite.py, %s"%mess)
                 datas = dict(message=mess.message)
             self.form_result = {}
             return self.send_frag(datas, template='amon_horaire', templates=used_templates)
        result =  {'titre': self.libelle}
        result['sstitre'] = ''
        result['menus'] = menus._get_menu(self.server_nb, self.page)
        result.update(self._get_horaire())
        if form_name.has_key(self.page):
            result['validate'] = {'href':tools.make_long_form_link(self.server_nb, self.name,
                                                                    True,
                                                                    form_name[self.page],
                                                                    balise='return_msg_container'),
                                  'icone':'/image/amon/ok.gif',
                                  'libelle':'Valider',
                                  'title':'Valider les changements'}
            result['validation'] = tools.make_key_form_link(self.server_nb, self.name,
                                                                False,
                                                                form_name[self.page],
                                                                balise='return_msg_container')
        return self.send_all(result, template='amon_horaire', templates=used_templates)

    def _get_horaire(self):
        """ renvoie la description du formulaire de gestion des horaires de pare-feu """
        semaine = ['dimanche','lundi', 'mardi', 'mercredi', 'jeudi', 'vendredi', 'samedi']
        horaires = get_horaires()
        days = []
        for horaire in horaires:
            num_jour, houvert, hferme = horaire
            indice = int(num_jour)
            ouvert_select = {'name':'ouvert[%d]'%(indice), 'options':[{'name':'', 'options':get_hours(houvert)}]}
            ferme_select = {'name':'ferme[%d]'%(indice), 'options':[{'name':'', 'options':get_hours(hferme)}]}
            days.append({'libelle':semaine[indice], 'ouvert':ouvert_select, 'ferme':ferme_select})
        return dict(horaire=days)

    def _valid_horaire(self):
        """ valide la configuration des horaires du pare-feu """
        if not self.form_result.has_key('list_add'):
            raise Exception, "Erreur: il manque des données pour la gestion des horaires du pare-feu."
        config = tools.format_form_result(self.form_result['list_add'])
        jours = []
        for i in range(7)[1:]:
            keyopen = 'ouvert[%d]'%i
            keyclosed = 'ferme[%d]'%i
            indice = i
            jours.append(('%d'%indice, config[keyopen], config[keyclosed]))
        keyopen, keyclosed = 'ouvert[0]', 'ferme[0]'
        jours.append(('0', config[keyopen], config[keyclosed]))
        write_horaire(jours)
        if t.bastion_restart():
            return dict(message="Le pare-feu a bien été redémarré, les nouveaux horaires sont pris en compte.")
        else:
            log.err("Erreur: securite.py, une erreur s'est produite au lancement de 'service bastion restart'(3).")
            raise Exception, "Erreur: une erreur est survenue au redémarrage du pare-feu."

