# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# Action Bacula et BaculaConfig
#
# Programme et configure des sauvegardes bacula
#
###########################################################################

""" Actions permettant de gérer les sauvegardes bacula """
from twisted.python import log
from ead2.backend.config.config import BACULA_DIR, BACULA_TEMP_DIR, SCRIPT_DIR
from ead2.backend.config.filenames import bacula as baculafiles
from ead2.backend.lib.action import * # import des actions et des types
from ead2.backend.actions import tools

from creole.fonctionseole import get_module_name

# ajout du chemin pour bacula dans le pythonpath
import sys
sys.path.append(BACULA_DIR)
import baculaeole as bacula

from os.path import isfile, join

# def des noms de fichiers de configuration de sauvegarde:
distant_filename = baculafiles['distant']                       # conf de sauvegarde pour les supports distants
usb_filename = baculafiles['usb']                               # pour les supports usb
rapport_filename = baculafiles['rapport']                       # log des sauvegardes effectuees

filenames = {'usb':usb_filename, 'distant':distant_filename}
used_templates = ['main', 'formselect', 'form']

def get_actual_support():
    """ renvoit le type de support actuel """
    code, retour = tools.command_statusoutput('%sbaculasupport.py -l'%BACULA_DIR)
    if retour in ['distant' , 'usb', 'bande']:
        return retour
    else:
        return ''

def test_mail(mail_list_str):
    """ teste une liste de mail pour la conf bacula
        mail,mail,... sans espace avec un arrobas par mail
    """
    if ' ' in mail_list_str:
        return False, "Erreur à la configuration de messagerie: ne pas mettre d'espace dans la liste des mails."
    mail_list = mail_list_str.split(',')
    for mail in mail_list:
        if not tools.test_mail(mail):
            return False, "Erreur à la configuration de messagerie: une adresse mail doit contenir un @ (et un seul)."
    return True, ''

class Bacula(Action):
    """
    interface de planification de sauvegarde
    trois formulaires sont proposes pour les sauvegardes (hebdomadaire,quotidienne,  immediate)
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'bacula'
    libelle = 'Programmation'
    category = 'Sauvegarde'
    description = 'gestionnaire de sauvegarde bacula'
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend",
                            keys=['server', 'action', 'job_suppr', 'page'])
    form_result = Dict(default={}, doc="retour de formulaire en JSON", keys=['sauvegarde', 'page'])

    def execute(self):
        """ renvoie les donnees pour la mise en forme de l'action et gere les retours
            1 - Récupèration d'information:
                a - Le le type de sauvegarde à programmer
                b - Bacula est-il configuré ?
            2 - Renvoie la description du formulaire pour lancer la sauvegarde
            3 - Traite le retour formulaire et lance la sauvegarde
            4 - Annule une sauvegarde

        """
       # on recup le numero du serveur en cours de traitement et la requete
        params, self.server_nb = tools.get_request(self.request)

        ## 1 - a
        self.page = 'weekly'
        if params.has_key('page'):
            self.page = params['page'][0]
        result = {'titre':self.description}
        if self.form_result !={} :
            ## 1 -
            if self.form_result.has_key('page'):
                self.page = tools.format_form_result(self.form_result['page'])['page']
                self.form_result.pop('page')
            ## 3 -
            try:
                datas = self.save()
                datas['redirect'] = tools.make_js_link(self.server_nb, self.name, confirm=True, page=self.page)
            except Exception, mess:
                datas = dict(message=mess.message)
            self.form_result = {}
            return self.send_frag(datas, template='bacula', templates=used_templates)
        ## 1 - b
        sup = get_actual_support()
        if sup in filenames.keys():
            if not isfile(filenames[sup]):
                result['message'] = "Pensez à configurer Bacula avant toutes sauvegardes"
                result['redirect'] = tools.make_js_link(self.server_nb, 'bacula_config', confirm=True)
        ## 4 -
        if params.has_key('job_suppr'): # annulation d'une sauvegarde programmee
            self.cancel_save(params['job_suppr'][0])
        ## 2 -
        sstitre_dict = {'weekly':"Programmation de sauvegarde hebdomadaire",
                        'daily':'Programmation de sauvegarde quotidienne',
                        'direct':"Programmation de sauvegarde immédiate"}
        result['sstitre'] = sstitre_dict[self.page]
        result['current_page'] = {'default_value':self.page, 'name':'page'}
        result['menus'] = self._get_menu()
        result.update(self._get_form())
        result['valid'] = self._get_valid_btn()
        result['jobs'] = self._get_actual_saves()
        return self.send_all(result, template='bacula', templates=used_templates)

    def _get_form(self):
        """ renvoit le formulaire adequat """
        if self.page == 'weekly':
            return self._get_form1()
        elif self.page == 'daily':
            return self._get_form2()
        else:
            return self._get_form3()

    def _get_form1(self):
        """premier formulaire pour la programmation de sauvegarde (sauvegarde hebdomadaire)"""
        type_opts = [{'name':'Complete', 'libelle':'Complete', 'default':'ok'}]
        if get_module_name() == 'horus':
            type_opts.append({'name':'Restreinte', 'libelle':'Restreinte'})
        typesav = {'name':'typesav', 'options':[{'name':'type de sauvegarde', 'options':type_opts}],
                'libelle':"Programmer une sauvegarde", 'inline':'ok'}
        niveau_opts = [{'name':'Full', 'libelle':'Totale'}, {'name':'Incremental', 'libelle':'Incrémentale'}]
        level = {'name':'niveau','options':[{'name':'niveau de sauvegarde', 'options':niveau_opts}],
                'libelle':'de type', 'inline':'ok'}
        days_opts = self._get_days_for_select(1)
        begin = {'name':'jourdebut', 'options':[{'name':'jour de la semaine', 'options':days_opts}], 'libelle':'le',
                'inline':'ok'}
        hour_opts = self._get_hours(0)
        hour = {'name':'heure', 'options':[{'name':'heure de la journée', 'options':hour_opts}],
                'libelle':'à','inline':'ok'}
        return dict(typesav=typesav, level=level, begin=begin, hour=hour)

    def _get_form2(self):
        """ deuxieme formulaire pour la programmation de sauvegarde (sauvegarde quotidienne)"""
        type_opts = [{'name':'Complete', 'libelle':'Complete', 'default':'ok'}]
        if get_module_name() == 'horus':
            type_opts.append({'name':'Restreinte', 'libelle':'Restreinte'})
        typesav = {'name':'typesav', 'options':[{'name':'type de sauvegarde', 'options':type_opts}],
                'libelle':"Programmer une sauvegarde",'inline':'ok'}
        niveau_opts = [{'name':'Full', 'libelle':'Totale'}, {'name':'Incremental', 'libelle':'Incrémentale'}]
        level = {'name':'niveau','options':[{'name':'niveau de sauvegarde', 'options':niveau_opts}],
                'libelle':'de type','inline':'ok'}
        begin_opts = self._get_days_for_select(1)
        begin  = {'name':'jourdebut', 'options':[{'name':'jour de la semaine', 'options':begin_opts}],
                'libelle':'du','inline':'ok'}
        end_opts = self._get_days_for_select(5)
        end = {'name':'jourfin', 'options':[{'name':'jour de la semaine', 'options':end_opts}],
                'libelle':'au','inline':'ok'}
        hour_opts = self._get_hours(0)
        hour = {'name':'heure', 'options':[{'name':'heure de la journée', 'options':hour_opts}],
                'libelle':'à','inline':'ok'}
        return dict(typesav=typesav, level=level, begin=begin, end=end, hour=hour)

    def _get_form3(self):
        """ troisieme formulaire de sauvegarde (sauvegarde immediate)"""
        type_opts = [{'name':'Complete', 'libelle':'Complete', 'default':'ok'}]
        if get_module_name() == 'horus':
            type_opts.append({'name':'Restreinte', 'libelle':'Restreinte'})
        typesav = {'name':'typesav', 'options':[{'name':'type de sauvegarde', 'options':type_opts}],
                'libelle':"Programmer une sauvegarde", 'inline':'ok'}
        niveau_opts = [{'name':'Full', 'libelle':'Totale'}, {'name':'Incremental', 'libelle':'Incrémentale'}]
        level = {'name':'niveau','options':[{'name':'niveau de sauvegarde', 'options':niveau_opts}],
                'libelle':'de type','inline':'ok'}
        return dict(typesav=typesav, level=level)

    def _get_menu(self):
        """ renvoit la description du menu """
        weekly = {'href':tools.make_js_link(self.server_nb, self.name, page='weekly'),
                'libelle':'Hebdomadaire', 'icone':'/image/mail.gif', 'title':'Sauvegarde hebdomadaire'}
        daily = {'href':tools.make_js_link(self.server_nb, self.name, page='daily'),
                'libelle':'Quotidienne', 'icone':'/image/mail.gif', 'title':'Sauvegarde quotidienne'}
        direct = {'href':tools.make_js_link(self.server_nb, self.name, page='direct'),
                'libelle':'Immédiate', 'icone':'/image/mail.gif', 'title':'Sauvegarde immédiate'}
        if self.page == 'weekly':
            weekly['href'] = ''
        elif self.page == 'daily':
            daily['href'] = ''
        else:
            direct['href'] = ''
        return weekly, daily, direct

    def _get_valid_btn(self):
        """ renvoit la description du bouton valider pour le formulaire """
        validate = {}
        ref= tools.make_form_link(self.server_nb, self.name, False, ['sauvegarde', 'page'], balise='return_msg_container')
        if self.page == 'direct':
            message = "Etes-vous sûr de vouloir lancer cette sauvegarde ?"
        else:
            message = "Etes-vous sûr de vouloir programmer cette sauvegarde ?"
        validate['href']= tools.make_confirm_link(message, ref)
        validate['icone'] = "/image/ok.gif"
        validate['libelle'] = "OK"
        validate['title'] = "Programmer la sauvegarde"
        return validate

    def _get_actual_saves(self):
        """ renvoit la liste des sauvegardes planifiees """
        saves = bacula.chargedico()
        if saves == {}:
            return [{'sauvegarde':"Aucune sauvegarde n'est programmée"}]
        montemplate1 = """Sauvegarde %(niveau)s, %(typesav)s du %(jourdebut)s au %(jourfin)s à %(heure)s heure"""
        montemplate2 = """Sauvegarde %(niveau)s, %(typesav)s le %(jourdebut)s à %(heure)s heure"""
        jobs = []
        for job in saves.keys():
            href = tools.make_js_link(self.server_nb, self.name, False, job_suppr=job, page=self.page)
            saves[job]['jourdebut'] = self._get_days(saves[job]['jourdebut'])
            saves[job]['niveau'] = self._get_niveau(saves[job]['niveau'])
            if saves[job]['jourfin']:
                saves[job]['jourfin'] = self._get_days(saves[job]['jourfin'])
                sauvegarde = str(job)+" : "+montemplate1 % saves[job]
            else:
                sauvegarde = str(job)+" : "+montemplate2 % saves[job]
            btn = {'href':href, 'title':"Annulation de la sauvegarde", "libelle":"Annuler",
                    "icone":'/image/supprimer_big.gif'}
            jobs.append({'btn':btn, 'sauvegarde': sauvegarde})
        return jobs

    def _get_sauvegarde(self):
        """ renvoit la liste des sauvegardes realisées"""
        if isfile(rapport_filename):
            a = file(rapport_filename, 'r')
            lines = a.readlines()
            if len(lines) == 0 :
                a.close()
                return ''
            a.close()
            return lines
        return ''

    def _get_typesav(self, typesav=None):
        """ renvoie un dico décrivant les type de sauvegarde ou la traduction de l'appelation système"""
        dico = {'Complete': 'Complète', 'Restreinte': 'Restreinte'}
        if typesav != None:
            return dico[typesav]
        return dico

    def _get_niveau(self, niveau=None):
        """ renvoie un dico : {'niveau': 'libelle'} ou la traduction de l'appelation système"""
        dico = {'Level=Incremental':'Incrémentale', 'Level=Full':'Totale' }
        dico2 = {'Incremental':'Incrémentale', 'Full':'Totale'}
        if niveau in dico.keys():
            return dico[niveau]
        elif niveau in dico2.keys():
            return dico2[niveau]
        else:
            return dico

    def _get_days(self, day=None):
        """ renvoie un dico jouranglais:jourfrancais accompagné par un tuple décrivant l'ordre
            :day: jour en anglais (dans ce cas ne renvoie que le jour en français)
        """
        ordre = ('mon', 'thu', 'wed', 'tue', 'fri', 'sat', 'sun')
        days = {'mon':'lundi', 'thu':'mardi', 'wed': 'mercredi',
                'tue':'jeudi', 'fri':'vendredi', 'sat':'samedi',
                'sun':'dimanche'}
        if day != None:
            return days[day]
        return days, ordre

    def _get_days_for_select(self, default):
        """ renvoie les jours de la semaine pour une balise select avec default: valeur par defaut """
        days = self._get_days()[0]
        ordre = self._get_days()[1]
        days = [{'name':day, 'libelle':days[day]} for day in ordre]
        days[default]['default'] = 'ok'
        return days

    def _get_hours(self, default):
        """ renvoit les heures (depuis 0->23) pour les formulaires """
        hours = []
        for i in range(24):
            hour = {'name':'%s'%i, 'libelle':'%s heure'%i}
            hours.append(hour)
        hours[default]['default'] = 'ok'
        return hours

    def save(self):
        """ gère la distribution des sauvegardes """
        if not self.form_result.has_key('sauvegarde'):
            log.err("Il manque des données pour la programmation de sauvegarde.")
            raise Exception, "Erreur: il manque des données pour la programmation de sauvegarde."
        if self.page == 'weekly':return self.sauvegarde_hebdo()
        elif self.page == 'daily':return self.sauvegarde_quotidienne()
        elif self.page == 'direct':return self.sauvegarde_immediate()
        else:return 1, ''

    def sauvegarde_hebdo(self):
        """ programme une sauvegarde hebdomadaire """
        result = self.form_result['sauvegarde']
        options = tools.format_form_result(result)

        try:
            typesav ,niveau, jourdebut, heure = [options['typesav'],options['niveau'], options['jourdebut'],options['heure']]
        except KeyError:
            log.err("Il manque des données pour la programmation d'une sauvegarde hebdomadaire.")
            raise Exception, "Erreur: Il manque des données pour la programmation d'une sauvegarde hebdomadaire"
        heure = "%s:00"%heure
        if '' in [typesav ,niveau, jourdebut, heure]:
            log.err("Il manque des valeurs pour la programmation d'une sauvegarde hebdomadaire")
            raise Exception, "Erreur: Il manque des valeurs pour la programmation d'une sauvegarde hebdomadaire"
        else:
            code, toto = tools.command_statusoutput('%sbaculaeole.py -t %s -n %s -d %s -h %s'%(
                                                    BACULA_DIR,
                                                    typesav,
                                                    niveau,
                                                    jourdebut,
                                                    heure)
                                                )
            if code:
                log.err("Erreur lors du lancement de baculaeole.py avec les options '-t %s -n %s -d %s -h %s'"%(typesav, niveau, jourdebut, heure))
                raise Exception, "Erreur: erreur lors de la programmation de la sauvegarde hebdomadaire."
            return dict(message="Une sauvegarde %s de type %s a bien été programmée. \\n \\n La sauvegarde s'effectuera le %s à %s."%(
                                                             self._get_typesav(typesav),
                                                             self._get_niveau(niveau),
                                                             self._get_days(jourdebut),
                                                             heure)
                       )

    def sauvegarde_quotidienne(self):
        """ programme une sauvegarde quotidienne """
        result = self.form_result['sauvegarde']
        options = tools.format_form_result(result)
        try:
            heure = "%s:00"%options['heure']
            values = [options['typesav'], options['niveau'], options['jourdebut'], options['jourfin'], heure]
        except KeyError:
            log.err("Il manque des données pour la programmation d'une sauvegarde quotidienne.")
            raise Exception, "Erreur: Il manque des données pour la programmation d'une sauvegarde quotidienne."

        if '' in values:
            log.err("Il manque des valeurs pour la programmation d'une sauvegarde quotidienne.")
            raise Exception, "Erreur: Il manque des valeurs pour la programmation d'une sauvegarde quotidienne."
        else:
            code, retour = tools.command_statusoutput('%sbaculaeole.py -t %s -n %s -d %s -f %s -h %s'%(
                                                                            BACULA_DIR,
                                                                            values[0],
                                                                            values[1],
                                                                            values[2],
                                                                            values[3],
                                                                            values[4])
                                                    )
            if code:
                log.err("Erreur lors du lancement de baculaeole.py avec les options '-t %s -n %s -d %s -f %s -h %s'"%(values[0], values[1], values[2], values[3], values[4]))
                raise Exception, "Erreur: erreur lors de la programmation de la sauvegarde quotidienne."

            return dict(message="Une sauvegarde quotidienne %s de type %s a bien été programmée.\\n\\nLa sauvegarde s'effectuera du %s au %s à %s."%(self._get_typesav(values[0]),
                     self._get_niveau(values[1]),
                     self._get_days(values[2]),
                     self._get_days(values[3]),
                     values[4]))

    def sauvegarde_immediate(self):
        """ Lance une sauvegarde immédiate """
        options = self.form_result['sauvegarde']
        result = tools.format_form_result(options)
        if result.has_key('typesav'):
            if result['typesav'] == 'Restreinte': typesav='-R'
            else: typesav = ''
        if result.has_key('niveau'):
            if result['niveau'] == 'Incremental': niveau = '-I'
            else: niveau = ''
        commande = ''
        if typesav: commande = typesav
        if niveau: commande += ' %s'%niveau
        code, retour = tools.command_statusoutput('%s/bacula-sauv.sh %s'%(SCRIPT_DIR, commande))
        if code:
            log.err("Erreur au lancement de bacula-sauv.sh avec l'option %s"%commande)
            raise Exception, "Erreur: une erreur est survenue lors du lancement d'une sauvegarde immédiate."
        return dict(message="Une sauvegarde est lancée.")

    def cancel_save(self, num):
        """ Annule une sauvegarde programmee """
        code, retour  = tools.command_statusoutput('%sbaculaeole.py -s %s'%(BACULA_DIR, num))

class BaculaConfig(Action):
    """
    interface de configuration de l'outil de sauvegarde
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'bacula_config'
    libelle = 'Configuration'
    category = 'Sauvegarde'
    description = "configuration de l'outil de sauvegarde bacula"
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend",
                         keys=['server', 'action', 'save_type'])
    form_result = Dict(default={}, doc="retour de formulaire", keys=['config1', 'config2', 'config3'])

    def execute(self):
        """ renvoie les donnees pour la mise en forme de l'action
            1 - Récupère le type de sauvegarde à configurer
            2 - Renvoie le formulaire de configuration bacula
            3 - Traite les retours formulaires et configure la sauvegarde
        """
        params, self.server_nb = tools.get_request(self.request)
        result = {'titre':self.description}
        ## 1 -
        if params.has_key('save_type'):
            self.save_type = params['save_type'][0]
        else:
            self.save_type = get_actual_support()
        ## 3 -
        if self.form_result !={} :
            try:
                datas = self.modify_conf()
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name, confirm=True)
            except Exception, mess:
                import traceback
                traceback.print_exc()
                datas = dict(message=mess.message)
            self.form_result = {}
            return self.send_frag(datas, template='bacula_config', templates=used_templates)
        ## 2 -
        if self.save_type == '':
            result['formulaires'] = [ {'titre': 'Support de sauvegarde',
                                        'formulaire':self._get_main_form(),
                                        'name':'config1'}]
        else:
            result['formulaires'] = [ {'titre': 'Support de sauvegarde',
                                        'formulaire':self._get_main_form(),
                                        'name':'config1'},
                                      {'titre':'Paramètres de sauvegarde pour : %s'%self.save_type,
                                          'formulaire':self._get_form(),
                                          'name':'config2'},
                                      {'titre':"Paramètres d'envoi des logs (Facultatif)",
                                          'formulaire':self._get_log_form(),
                                          'name':'config3', 'validate':self._get_validate_btn()}]

            if self.save_type == 'bande':
                result['entete_bande'] = "<font color=red>La configuration est <b>manuelle</b>. Voir le fichier '/etc/bacula/bacula-sd.conf'</font>"


        return self.send_all(result, template='bacula_config', templates=used_templates)

    def _get_main_form(self):
        """ renvoit le formulaire pour le choix du type de sauvegarde a configurer """
        possibilities = {'distant':'Distant', 'usb':'Disque usb local','bande':'Bande: Configuration manuelle'}
        values = []
        for name, libelle in possibilities.items():
            # type de support configurer
            if name == self.save_type:
                values.append({'name':name, 'libelle':libelle, 'default':'ok'})
            else:
                values.append({'name':name, 'libelle':libelle})
        tag = {'_type':'choice', 'name':'support', 'options':[{'name':'', 'options':values}],
                'libelle':'Support de sauvegarde',
                'onchange':tools.make_js_link(self.server_nb, self.name, code=True,
                                                save_type='this.options[this.selectedIndex].value'),
                'inline':'ok'}
        return [tag]

    def _get_form(self):
        """ renvoit le formulaire pour configurer les sauvegardes du type self.save_type """
        # configuration usb
        if self.save_type == 'usb':
            tag = {'_type':'text_input', 'name':'usb_periph', 'libelle':"Chemin d'accès" ,
                    'onblur':"testVoid('usb_periph', 'error_usb_periph');", 'inline':'ok'}
            if len(self._get_usb_conf()) > 0:
                tag['default_value'] = self._get_usb_conf()[0]
            return [tag]
        # configuration distante
        elif self.save_type == 'distant':
            tag1 = {'_type':'text_input', 'name':'machine', 'libelle':"Nom machine distante",
                    'onblur':"testVoid('machine', 'error_machine')", 'inline':'ok'}
            tag2 = {'_type':'text_input', 'name':'ipmachine', 'libelle':"IP machine distante",
                    'onblur':"testIp('ipmachine', 'error_ipmachine')", 'inline':'ok'}
            tag3 = {'_type':'text_input', 'name':'partage', 'libelle':"Partage",
                    'onblur':"testVoid('partage', 'error_partage')", 'inline':'ok'}
            tag4 = {'_type':'text_input', 'name':'login', 'libelle':"Login (facultatif)", 'inline':'ok'}
            tag5 = {'_type':'password', 'name':'password', 'libelle':"Mot de passe (facultatif)", 'inline':'ok'}
            a = self._get_distant_conf()
            if len(a)>0:
                tag1['default_value'] = a[0]
                tag2['default_value'] = a[1]
                tag3['default_value'] = a[2]
                if len(a) == 5:
                    tag4['default_value'] = a[3]
                    tag5['default_value'] = a[4]
            return [tag1, tag2, tag3, tag4, tag5]
        else:
            return []

    def _get_log_form(self):
        """ renvoit le formulairee pour la configuration des logs """
        mail, smtp = self._get_actual_log_conf()
        tag1 = {'_type':'text_input', 'name':'mail', 'libelle':'Mail admin sauvegarde', 'default_value':mail, 'inline':'ok'}
        tag2 = {'_type':'text_input', 'name':'smtp', 'libelle':'Smtp', 'default_value':smtp, 'inline':'ok'}
        return [tag1, tag2]

    def _get_distant_conf(self):
        """ renvoit la dernière config enregistree pour les sauvegardes sur les
            supports distants
        """
        if isfile(distant_filename):
            lines = file(distant_filename, 'r').readlines()
            if len(lines)>0:
                datas = lines[0]
            else:
                return []
            return self.parse_distant_conf(datas)
        return []

    def _get_usb_conf(self):
        """ renvoit la dernière config enregistree pour les sauvegardes sur les
            supports usb
        """
        if isfile(usb_filename):
            lines = file(usb_filename, 'r').readlines()
            if len(lines)>0:
                datas = lines[0]
            else:
                return[]
            return self.parse_usb_conf(datas)
        return []

    def _get_actual_log_conf(self):
        """ renvoit  les paramètres de messagerie de bacula """
        code, retour = tools.command_statusoutput('%sbaculamessage.py -l'%BACULA_DIR)
        if ';' in retour:
            retour = retour.strip()
            datas = retour.split(';')
            smtp = datas[0]
            mail = datas[1]
            return mail, smtp
        return '', ''

    def _get_validate_btn(self):
        """ renvoit la description du bouton valider pour le formulaire form_name """
        validate = {}
        validate['href'] = tools.make_form_link(self.server_nb, self.name, True,['config1', 'config2', 'config3'], balise='return_msg_container')
        validate['icone'] = "/image/ok.gif"
        validate['libelle'] = "OK"
        validate['title'] = "Valider vos changements"
        return validate

    def parse_distant_conf(self, datas):
        """ parse un fichier de conf distant """
        log.msg("Distant %s"%datas)
        firstparse = datas.split()
        machine = firstparse[0].split('/')[2]
        partage = '/'.join(firstparse[0].split('/')[3:])
        ipmachine = firstparse[1]
        if len(firstparse)==4:
            login = firstparse[2]
            password = firstparse[3]
        else:
            login = ''
            password = ''
        return [machine, ipmachine, partage, login, password]

    def parse_usb_conf(self, datas):
        """ parse un fichier de conf usb """
        path = datas.strip()
        return [path]

    def modify_conf(self):
        """ valide les modifications de configuration de bacula """
        sup = get_actual_support()
        message = ''
        if not self.form_result.has_key('config1'):
            raise Exception, "Erreur: il manque des données pour la configuration de bacula."
        self.save_type = self.form_result['config1'][0]['value'] # recuperation du type de support
        if sup != self.save_type:
            self._set_support(self.save_type)
            message += "La sauvegarde a été définie en %s.\\n"%self.save_type

        if self.save_type in ('distant', 'usb'):
            if not self.form_result.has_key('config2'):
                raise Exception, "Erreur: il manque des données pour la configuration de bacula."

        if self.save_type == 'distant': # modification apportees pour 'distant'
            datas = self.form_result['config2']
            changes = tools.format_form_result(datas)
            for key, value in changes.items():
                if value == '': changes.pop(key)
            self.write_distant_conf(changes)
            message += "La configuration de sauvegarde de type distante a été effectuée."

        elif self.save_type == 'usb': # modification apportees pour 'usb'
            datas = tools.format_form_result(self.form_result['config2'])
            path = datas['usb_periph']
            self.write_usb_conf(path)
            message += "La configuration de sauvegarde de type usb a été effectuée."

        log_datas = tools.format_form_result(self.form_result['config3']) # modification des parametres de log
        smtp, mail = (log_datas['smtp'], log_datas['mail'])

        act_mail, act_smtp = self._get_actual_log_conf()

        status, response = test_mail(mail)
        if not status:
            message += response
        elif mail and smtp not in ['', '']:
            code, retour = tools.command_statusoutput('%sbaculamessage.py -s %s -m %s'%(BACULA_DIR, smtp, mail))
            if code: message+= "\\nErreur lors de la configuration de la messagerie."
            else: message+= "\\nLa messagerie a bien été configurée."
        elif mail != act_mail or smtp != act_smtp:
            code, retour = tools.command_statusoutput('%sbaculamessage.py -d'%(BACULA_DIR))
            if code :
                message += "\\nErreur à la suppression du mail"
            else:
                message += "\\nL'adresse mail a été supprimée."
        return dict(message=message)

    def write_distant_conf(self, changes):
        """ modifie le fichier de configuration de sauvegarde pour les supports distants"""
        if len(changes.keys()) == 3:
            conf = "//%s/%s %s"%(changes['machine'].strip(), changes['partage'].strip(), changes['ipmachine'].strip())
        elif len(changes.keys()) == 5:
            conf = "//%s/%s %s %s %s"%(changes['machine'].strip(), changes['partage'].strip(),
                                    changes['ipmachine'].strip(), changes['login'].strip(), changes['password'].strip())
        else:
            raise Exception, "La configuration de la sauvegarde a échouée (manque de paramètres)."
        a = file(distant_filename, 'w')
        a.write(conf)
        a.close()

    def write_usb_conf(self, changes):
        """ modifie le fichier de configuration pour les supports usb """
        a = file(usb_filename, 'w')
        a.write(changes)
        a.close()

    def _set_support(self, sup):
        """ definit le support de sauvegarde au niveau de bacula """
        if sup in ['distant' , 'usb', 'bande']:
            code, retour = tools.command_statusoutput('%sbaculasupport.py -s %s'%(BACULA_DIR, sup))
