# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# Action gestion_group* 
#  
# Cree modifie et supprime des groupes
#       
###########################################################################

"""Actions permettant de gérer les utilisateurs, compte, partage, groupe (application Gestion)"""
from twisted.python import log
from ead2.backend.lib.action import * # import des actions et des types
from ead2.backend.actions import tools
from ead2.backend.config.config import SCRIPT_DIR
from ead2.lib.libead import uni
import commands
from os.path import join
from horus import backend as horus # import des actions (get_groups, mod_user, inscription ... )
from ead2.backend.actions.horus import gestion_tools as G
from ead2.lib.error import MissingKey, MissingValue, BadGroupName, BadShareName

titre='Gestion des groupes'
used_templates = ['main', 'alphas', 'listlist', 'accordion', 'form', 'formselect', 'checklist' ]
spec_groups = ['DomainAdmins', 'DomainUsers', 'applidos', 'minedu', 'DomainComputers', 'PrintOperators']

def _get_groups():
    """ recupere la liste des groupes existants"""
    try:
        groups= horus.get_groups()
        for num, gr in enumerate(groups):
            if gr in spec_groups: 
                groups[num] = "- " + gr + "<br />  (Réservé eole)"
            else:
                groups[num] = "- " + gr
        groups.insert(0, '<b>Groupes existants:</b>')
    except:
        groups = ["Il n'y a aucun groupe"]
    return groups

class GestionGroupCreate(Action):
    """ creation d'un groupe """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'gestion_group_create'
    libelle = "Création"
    category = "Gestion/Groupes"
    description = 'créer un groupe'
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend", 
                        keys=['server', 'action', 'user_alpha'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", 
                        keys=['group', 'active', 'unactive', 'share_form', 'alpha_key'])
    
    def execute(self):
        """ renvoit les données pour l'affichage 
            1 - envoie la description du formulaire de création de groupe
            2 - renvoie les utilisateurs dont le nom commence par la lettre 'user_alpha'
            3- valide la création et renvoie un message
        """
        params, self.server_nb = tools.get_request(self.request)
        menu = G.get_group_menu(self.server_nb, self.name)
        sstitre = self.description
        result = {'sstitre':sstitre, 'titre':titre, 'menus':menu}
        result['retour']  = G.get_return_btn(self.server_nb)
        retour = ''
        ## 2 -
        self.user_alpha = ''
        if params.has_key('user_alpha'): 
            self.user_alpha = params['user_alpha'][0]
            return self.user_frag()
        ## 3 -
        if self.form_result!={}:
            try:
                retour = self.create_group(self.form_result)
                retour['toexec'] = tools.make_js_link(self.server_nb, self.name, confirm=True)
            except Exception, mess:
                retour = {'message':mess.message}
            self.form_result = {}
            return self.send_frag(retour, template='horus_gestion_group', templates=used_templates)
        ## 1 -
        result['leftpart'] = _get_groups()
        result['message'] = retour
        result.update(self._get_form())
        result.update(self._get_valid_btn())
        return self.send_all(result, template='horus_gestion_group', templates=used_templates)

    def user_frag(self):
        """ renvoit le fragment listant les utilisateurs filtré par la première lettre du login"""
        result = {}
        result['datas'] = self._get_users()
        d = {'template':'checklist', 'data':{'content':result, 'templates':['checklist']}}
        return 0, uni(str(d))

    def _get_form(self):
        """ renvoit les données pour le formulaire """
        #input pour le nom d'utilisateur
        group_name = {'name':'group_name', 'libelle':"Nom du groupe", 'onblur':"testVoid('group_name', 'error_group_name')"}
        # section ajout d'utilisateur
        left = {'title':"Utilisateurs disponibles", 'form':"active", 'datas':self._get_users()}
        if self.user_alpha != 'tous' and self.user_alpha:
            left['title'] = "Utilisateurs disponibles <br /> Lettre %s"%self.user_alpha
        right = {'title':"Utilisateurs du groupe", 'form':"unactive", 'datas':[]}
        title = ''
        name = 'group_users'
        group_users = {'name':name, 'btns':self._get_transf_btns1(), 'left':left, 'right':right, 'title':title}
        alphas=self._get_alpha_links()
        # section ajout de partage
        group_shares = {}
        group_shares['btns'] = self._get_transf_btns2()
        group_shares['input'] = {'libelle': 'Nom du partage (sans accent)', 'name':'share'}
        group_shares['liste'] = {'title':'Liste des partages associés au groupe', 'values':[], 'formid':'share_form',
                            'id':'share_table'}
        group_shares['title'] = "Ajouter des partages et les lier au groupe"
        return dict(group_name=group_name, group_users=group_users, group_shares=group_shares, 
                    user_alphas=alphas, focus='group_name')

    def _get_users(self):
        """ renvoit les utilisateurs disponibles dont le nom commence par user_alpha """
        try:
            usrs = horus.get_users()
        except:
            usrs = []
        result = []
        if self.user_alpha:
            alpha = self.user_alpha.lower()
        else:
            alpha = 'tous'
        if alpha == 'tous': return [{'name':usr, 'libelle':usr}for usr in usrs]
        if alpha:
            result = [{'name':usr, 'libelle':usr}for usr in tools.filter_by_alpha(usrs, alpha)]
        else:
            result = [{'name':usr, 'libelle':usr}for usr in usrs]
        return result

    def _get_transf_btns1(self):
        """ renvoit les btns de transfert de donnees liste a liste pour les utilisateurs"""
        href2 = "javascript:formTransferListElement('active','unactive_table');"
        href1 = "javascript:formTransferListElement('unactive','active_table');"
        icone1 = "/image/rightleft_red.png"
        icone2 = "/image/leftright.png"
        msg1 = "Retirer"
        msg2 = "Ajouter"
        btns = [{'href':href2, 'libelle':msg2, 'icone':icone2, 'title':msg2},
                {'href':href1, 'libelle':msg1, 'icone':icone1, 'title':msg1}]
        return btns
   
    def _get_alpha_links(self):
        """ renvoit la description des liens alphabetique """
        links = []
        alpha = self.user_alpha
        for i in range(97,123):
            if chr(i) != alpha:href = tools.make_js_link(self.server_nb, self.name, balise='active_table', user_alpha=chr(i))
            else: href = ''
            libelle = chr(i).upper()
            links.append({'href':href, 'libelle':libelle})
        if 'tous' != alpha:
            links.append({'href':tools.make_js_link(self.server_nb, self.name, balise='active_table', user_alpha='tous'), 'libelle':'Tous'})
        else:
            links.append({'href':'', 'libelle':'Tous'})
        return links

    def _get_transf_btns2(self):
        """ renvoit les btns de transfert de donnees input a liste pour les partages"""
        href1 = "javascript:formRemoveFromList('share_form');"
        href2 = "javascript:formAddNameToList1('share','share_table');"
        icone1 = "/image/rightleft_red.png"
        icone2 = "/image/leftright.png"
        msg1 = "Retirer"
        msg2 = "Ajouter"
        btns = [{'href':href2, 'libelle':msg2, 'icone':icone2, 'title':msg2},
                {'href':href1, 'libelle':msg1, 'icone':icone1, 'title':msg1}]
        return btns
    
    def _get_valid_btn(self):
        """ renvoit la description du bouton valider """
        validate = {}
        validate['href'] = tools.make_form_link(self.server_nb, self.name, True,  ['group', 'unactive', 'share_form', 'alpha_key'], 'group_msg_div_container')
        validate['icone'] = "/image/ok.gif"
        validate['libelle'] = "Valider"
        validate['title'] = "Créer ce groupe"
        validation = tools.make_key_form_link(self.server_nb, self.name, False, ['group', 'unactive', 'share_form', 'alpha_key'], 'group_msg_div_container')
        return dict(validate=validate, validation=validation)

    def create_group(self, dico):
        """ cree un groupe depuis le retour formulaire du frontend
            group: le nom du groupe
            share_form: les partages associés au groupe
            unactive: les utilisateurs du groupe
        """
        group = ''
        ## récupération des valeurs venant du client
        if dico.has_key('alpha_key'):
            self.user_alpha = dico['alpha_key'][0]['value']
        if dico.has_key('group'):
            result = tools.format_form_result(dico['group']) 
            if result.has_key('group_name'):
                group = result['group_name'] # nom du groupe cree
                if not tools.test_login(group):
                    raise BadGroupName
            else:
                raise Exception, "Erreur lors de la validation du formulaire"
        
        shares = {}
        if dico.has_key('share_form'):
            share_form = dico['share_form']
            resultat = tools.format_form_result(share_form, True)
            keys = resultat.keys()
            for key in keys:
                if key.startswith('sticky'):
                    share_name = key[6:]
                    if not tools.test_login(share_name):
                        raise BadShareName
                    if share_name in keys:
                        shares[share_name] = resultat[key]

        # creation du groupe
        if group:
            if horus.is_group(group):
                raise Exception, "Erreur : un groupe de ce nom existe déjà."
            a = horus.add_group(group)
            if not a:
                raise Exception, "Erreur : une erreur est survenue à la création du groupe."
            else:
                message = "Le groupe %s a bien été créé.\\n\\n"%group
            # inscription des utilisateurs
            if dico.has_key('unactive'):
                users = tools.format_form_result(dico['unactive'], True).keys()
                if users != []:
                    message += "Inscription d'utilisateur : \\n"
                    for nom in users:
                        if horus.inscription(nom, group):
                            message += "   - %s a bien été inscrit au groupe.\\n"%nom
                        else:
                            message += "   - erreur lors de l'inscription de %s. \\n"%nom
           # gestion des partages attaches au groupe on recupere le nom, le sticky_bit et on cree
            if shares != {}:
                message += "Attachement de partage : \\n"
                for share, sticky in shares.items():
                    if not horus.add_share(share, group, sticky=sticky):
                        message += " - une erreur est survenue lors de l'attachement du partage %s au groupe.\\n"%share
                    else:
                        message += "   - Le partage %s a bien été créé et attaché au groupe.\\n"%share
            return dict(message=message)
        raise Exception,  "Erreur: une erreur est survenue lors de la création du groupe."

class GestionGroupModify(Action):
    """ modification d'un groupe """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'gestion_group_modify'
    libelle = "Modification"
    category = "Gestion/Groupes"
    description = 'Modifier un groupe'
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend", 
                    keys=['server', 'action', 'current_group', 'user_alpha'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['group', 'active', 'unactive', 'share_form', 'alpha_key'])
    
    def execute(self):
        """ renvoie les données pour l'affichage
            1 - récupère le groupe en cours d'édition
            2 - renvoie le formulaire de modification de groupe
            3 - renvoie la liste des utilisateurs filtrés sur la première lettre de login
            4 - valide la modification
        """
        params, self.server_nb = tools.get_request(self.request)
        menu = G.get_group_menu(self.server_nb, self.name)
        sstitre = self.description
        result = {'sstitre':sstitre, 'titre':titre, 'menus':menu}
        result['retour']  = G.get_return_btn(self.server_nb)
        self.user_alpha = ''
        ## 4 -
        if self.form_result!={}:
            try:
                datas = self.modify_group(self.form_result)
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name, confirm=True, 
                                        current_group=self.current_group)
            except (Exception), mess:
                datas = dict(message=mess.message)
            self.form_result = {}
            return self.send_frag(datas, template='horus_gestion_group', templates=used_templates)
        ## 1 -
        if 'current_group' in params.keys():
            self.current_group = params['current_group'][0]
        else:
            self.current_group = ''
        ## 3 -
        if params.has_key('user_alpha'): 
            self.user_alpha = params['user_alpha'][0]
            return self.user_frag()
        ## 2 -
        result.update(self._get_form())
        result.update(self._get_valid_btn())
        result['leftpart'] = _get_groups()
        return self.send_all(result, template='horus_gestion_group', templates=used_templates)

    def user_frag(self):
        """ renvoit le fragment listant les utilisateurs"""
        result = {}
        try:
            members = horus.get_members(self.current_group)
        except:
            members = []
        result['datas'] = [{'name':user, 'libelle':user}for user in self._get_users()if user not in members]
        d = {'template':'checklist', 'data':{'content':result}}
        return 0, uni(str(d))

    def _get_form(self):
        """ renvoit les données pour le formulaire """
        try:
            groups = horus.get_groups()
        except:
            groups = []
        for groupname in spec_groups:
            if groupname in groups:
                groups.remove(groupname)
        # si on a pas encore choisit de serveur on ne renvoit que le choix du nom
        if not self.current_group:
            options = [{'name':group, 'libelle':group}for group in groups]
            options.append({'name':'', 'libelle':'', 'default':''})
            group_name = {'name':'group_name', 'options':[{'name':'groupe disponible', 'options':options}], 
                    'libelle':"Nom du groupe",
                    'onchange':tools.make_js_link(self.server_nb, self.name, code=True, current_group='this.options[this.selectedIndex].value')            }
            return dict(group_select=group_name)
        else:
            if self.current_group in groups: groups.remove(self.current_group)
            options = [{'name':group, 'libelle':group}for group in groups]
            options.append({'name':self.current_group, 'libelle':self.current_group,'default':'te'})
            group_name = {'name':'group_name', 'options':[{'name':'groupe disponible', 'options':options}], 
                    'libelle':"Nom du groupe",
                    'onchange':tools.make_js_link(self.server_nb, self.name, code=True, current_group='this.options[this.selectedIndex].value')            }
        # section ajout d'utilisateur
        users = self._get_users()
        members = []
        if self.current_group:
            try: 
                members = horus.get_members(self.current_group)
            except:
                members = []            
            for a in members:
                if a in users:
                    users.remove(a)
        left_opts = [{'name':user}for user in users]
        right_opts = [{'name':member}for member in members]
        left = {'title':"Utilisateurs disponibles", 'form':"active", 'datas':left_opts}
        if self.user_alpha != 'tous' and self.user_alpha:
            left['title'] = "Utilisateurs disponibles <br /> Lettre %s"%self.user_alpha
        right = {'title':"Utilisateurs du groupe", 'form':"unactive", 'datas':right_opts}
        title = ''
        name = 'group_users'
        group_users = {'name':name, 'btns':self._get_transf_btns1(), 'left':left, 'right':right, 'title':title}
        alphas = self._get_alpha_links()
        # section ajout de partage
        group_shares = {}
        group_shares['btns'] = self._get_transf_btns2()
        group_shares['input'] = {'libelle': 'Nom du partage (sans accent)', 'name':'share'}
        try:
            shares = horus.get_group_shares(self.current_group)
        except:
            shares = []
        group_shares['liste'] = {'title':'Liste des partages associés au groupe', 'values':shares, 'formid':'share_form',
                            'id':'share_table'}
        group_shares['title'] = "Ajouter des partages et les lier au groupe"
        return dict(group_select=group_name, group_users=group_users, group_shares=group_shares,
                         user_alphas=alphas)
    
    
    def _get_users(self):
        """ renvoit les utilisateurs disponibles dont le nom commence par user_alpha """
        try:
            usrs = horus.get_users()
        except:
            usrs = []
        result = []
        if self.user_alpha:
            alpha = self.user_alpha.lower()
        else:
            alpha = 'tous'
        if alpha == 'tous': return usrs
        if alpha:
            return tools.filter_by_alpha(usrs, alpha)
        else:
            return usrs

    def _get_alpha_links(self):
        """ renvoit la description des liens alphabetique """
        links = []
        alpha = self.user_alpha
        for i in range(97,123):
            if chr(i) != alpha:href = tools.make_js_link(self.server_nb, self.name, balise='active_table', user_alpha=chr(i), current_group=self.current_group)
            else: href = ''
            libelle = chr(i).upper()
            links.append({'href':href, 'libelle':libelle})
        if 'tous' != alpha:
            links.append({'href':tools.make_js_link(self.server_nb, self.name, balise='active_table', user_alpha='tous',current_group=self.current_group), 'libelle':'Tous'})
        else:
            links.append({'href':'', 'libelle':'Tous'})
        return links

    def _get_transf_btns1(self):
        """ renvoit les btns de transfert de donnees liste a liste pour les utilisateurs"""
        href2 = "javascript:formTransferListElement('active','unactive_table');"
        href1 = "javascript:formTransferListElement('unactive','active_table');"
        icone1 = "/image/rightleft_red.png"
        icone2 = "/image/leftright.png"
        msg1 = "Retirer"
        msg2 = "Ajouter"
        btns = [{'href':href2, 'libelle':msg2, 'icone':icone2, 'title':msg2},
                {'href':href1, 'libelle':msg1, 'icone':icone1, 'title':msg1}]
        return btns
    
    def _get_transf_btns2(self):
        """ renvoit les btns de transfert de donnees input a liste pour les partages"""
        href1 = "javascript:formRemoveFromList('share_form');"
        href2 = "javascript:formAddNameToList1('share','share_table');"
        icone1 = "/image/rightleft_red.png"
        icone2 = "/image/leftright.png"
        msg1 = "Retirer"
        msg2 = "Ajouter"
        btns = [{'href':href2, 'libelle':msg2, 'icone':icone2, 'title':msg2},
                {'href':href1, 'libelle':msg1, 'icone':icone1, 'title':msg1}]
        return btns
    
    def _get_valid_btn(self):
        """ renvoit la description du bouton valider """
        validate = {}
        href = tools.make_form_link(self.server_nb, self.name, False, ['group','active', 'unactive', 'share_form','alpha_key'], 'group_msg_div_container')
        validate['href'] = tools.make_confirm_link("Êtes-vous sûr de vouloir modifier ce groupe?", href)
#        validate['href'] = "javascript:formValidForm('%s','%s',['group','active', 'unactive', 'share_form'])"%(self.server_nb, self.name)
        validate['icone'] = "/image/ok.gif"
        validate['libelle'] = "Valider"
        validate['title'] = "Modifier ce groupe"
        validation = tools.make_key_form_link(self.server_nb, self.name, False, ['group','active','unactive','share_form','alpha_key'], 'group_msg_div_container')
        return dict(validate=validate, validation=validation)

    def modify_group(self, dico):
        """ modifie un groupe
            :dico: - unactive est la liste des users du groupe
                   - active est la liste de users qui ne sont pas du groupe
                   - share_form est la liste des partages associés au groupe
        """
        if dico.has_key('alpha_key'):
            self.user_alpha = dico['alpha_key'][0]['value']

        ## recuperation du nom du groupe
        if not dico.has_key('group'):
            raise Exception, "Il manque des données pour la modification du groupe."
        group_form = tools.format_form_result(dico['group'])
        if not group_form.has_key('group_name'): 
            raise Exception, "Il manque des données pour la modification du groupe."
        group = group_form['group_name']
        self.current_group = group
        if not group: raise Exception, "Aucun groupe n'a été sélectionné."

        ## membres du groupe
        message2 = ""
        if dico.has_key('unactive') and dico.has_key('unactive'):
            try:
                myusers = horus.get_members(group)
            except:
                myusers = []
            users = tools.format_form_result(dico['unactive'], True).keys()
            no_users = tools.format_form_result(dico['active'], True).keys()
            for user in users:
                if user not in myusers:
                    if not horus.inscription(user, group): message2 += "erreur à l'inscription de %s\\n"%user
                    else: message2 += "- %s a bien été inscrit\\n"%user
            for user in no_users:
                if user in myusers:
                    if not horus.desinscription(user, group): message2 += "erreur à la désinscription de %s\\n"%user
                    else: message2 += "- %s a bien été désinscrit\\n"%user
        if message2 == "": message2 = "  Aucune modification\\n"
        message = "Modification des inscriptions au groupe %s :\\n%s" % (group, message2)

        ## partage associé
        if dico.has_key('share_form'):
            try: 
                myshares = horus.get_group_shares(group)
            except:
                myshares = []
            shares = {}
            if dico.has_key('share_form'):
                share_form = dico['share_form']
                resultat = tools.format_form_result(share_form, True)
                keys = resultat.keys()
                for key in keys:
                    if key.startswith('sticky'):
                        share_name = key[6:]
                        if not tools.test_login(share_name): raise BadShareName
                        if share_name in keys: shares[share_name] = resultat[key]
                    elif 'sticky'+key not in keys:
                        shares[key] = False
            if shares != {}:
                message += "\\nModification des partages du groupe %s :\\n"%group
                message2 = ""
                for share, sticky in shares.items():
                    if share not in myshares:
                        if not horus.add_share(share, group, sticky=sticky):
                            message2 += "- erreur lors de l'association du partage %s au groupe\\n"%share
                        else:
                            message2 += "- création et association du partage %s réussie\\n"%share
            # on supprime les partages qui ont ete enleve
            for name in myshares:
                if not shares.has_key(name):
                    if horus.del_share(name): message += "- suppression du partage %s\\n"%name
                    else: message += "- echec de la suppression du partage %s\\n"%name
            if message2 == "": message2 = "  Aucune modification\\n"
            message += message2

        return dict(message=message)

class GestionGroupDelete(Action):
    """ gere la suppression de partage """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'gestion_group_suppr'
    libelle = "Suppression"
    category = "Gestion/Groupes"
    description = 'supprimer un groupe'
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend", keys=['server', 'action'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['group'])
    
    def execute(self):
        """ renvoie les données pour l'affichage
            1 - renvoie le formulaire de suppression de groupe
            2 - supprime le groupe
        """
        params, self.server_nb = tools.get_request(self.request)
        menu = G.get_group_menu(self.server_nb, self.name)
        sstitre = self.description
        result = {'sstitre':sstitre, 'titre':titre, 'menus':menu}
        ## 2 -
        if self.form_result!={}:
            try:
                datas = self.del_group(self.form_result)
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name)
            except Exception, mess:
                datas = dict(message=mess.message)
            self.form_result = {}
            return self.send_frag(datas, template='horus_gestion_group', templates=used_templates)
        ## 1 -
        result['group_select'] = self._get_form()
        result['validate'] = self._get_valid_btn()
        result['retour']  = G.get_return_btn(self.server_nb)
        return self.send_frag(result, template='horus_gestion_group', templates=used_templates)

    def _get_form(self):
        """ renvoie la description du formulaire de suppresion de groupe """
        form = []
        groups = self._get_groups()
        for groupname in spec_groups:
            if groupname in groups:
                groups.remove(groupname)
        opts = [{'name':group, 'libelle':group}for group in groups]
        group_select = {'name':'group_name', 'options':[{'name':'groupe existant','options':opts}], 
                        'libelle':"Nom du Groupe"}
        return group_select
    
    def _get_groups(self):
        """ recupere la liste des groupes existants"""
        try:
            groups = horus.get_groups()
        except:
            groups = []
        return groups

    def del_group(self, dico):
        """ supprimer un groupe """
        if dico.has_key('group'):
            result = tools.format_form_result(dico['group'])
            if not result.has_key('group_name'):
                raise Exception, "Il manque des données pour la suppression de groupe."
            name = result['group_name']
            if name:
                if horus.del_group(name):
                    return dict(message="Le groupe %s a bien été supprimé"%name)
                else:
                    raise Exception, "Erreur : erreur lors de la suppression du groupe %s."%name
        raise Exception, "Erreur : il manque des données pour la suppression de groupe."

    def _get_valid_btn(self):
        """ renvoit la description du bouton valider """
        validate = {}
        href= tools.make_form_link(self.server_nb, self.name, False, ['group'], 'group_msg_div_container')
        validate['href'] = tools.make_confirm_link("Êtes-vous sûr de vouloir supprimer ce groupe ?", href) 
        validate['icone'] = "/image/ok.gif"
        validate['libelle'] = "Valider"
        validate['title'] = "Supprimer ce groupe"
        return validate
