# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# Action gestion_user*
#
# Cree Modifie et Supprime des utilisateurs
#
###########################################################################

"""Actions permettant de gérer les utilisateurs"""
from twisted.python import log
from ead2.backend.lib.action import * # import des actions et des types
from ead2.backend.actions import tools
from ead2.backend.config.config import SCRIPT_DIR
import commands
from os.path import join
from horus import backend as horus # import des actions (get_groups, mod_user, inscription ... )
from ead2.backend.actions.horus import gestion_tools as G
from ead2.lib.error import MissingKey, MissingValue, BadPassword, BadLogin, BadDriveLetter

titre = 'Gestion des utilisateurs'
used_templates = ['form', 'main', 'formselect', 'listlist', 'accordion', 'checklist']

def _get_groups():
    """ renvoie la liste des groupes """
    try:
        grs = horus.get_groups()
    except:
        grs = []
    if "DomainAdmins" in grs: grs.remove("DomainAdmins")
    return grs

def get_role_association_btn(server_nb, username):
    """ renvoie la descritpion du bouton d'appel pour l'édition de rôle """
    return {'href': tools.make_js_link(server_nb, 'role_manager',
                                        balise='div_role_container',
                                        form='usertoassociate',
                                        user=username),
                'libelle':"Associer un rôle à cet utilisateur",
                'title': '',
                'icone': "/image/action_role.png",
                "_class":'scribe_action_btn'
                }

class GestionUserCreate(Action):
    """ création d'utilisateur
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'gestion_user_create'
    libelle = "Création"
    category = "Gestion/Utilisateurs"
    description = 'Créer un utilisateur'
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend", keys=['server', 'action', '_type'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['user'])

    def execute(self):
        """ renvoie les données pour l'affichage
            1 - Renvoie le formulaire de création d'utilisateur ainsi que le menu
            2 - Renvoie seulement le formulaire (si la création a réussi)
            3 - Traite le retour formulaire et valide la création utilisateur
        """
        params, self.server_nb = tools.get_request(self.request)
        ## 2 -
        if params.has_key('_type'):
            datas = self._get_form()
            datas.update(self._get_valid_btn())
            return self.send_frag(datas, template='horus_gestion_user', templates=used_templates)
        ## 3 -
        if self.form_result!={}:
            try:
                datas = self.create_user(self.form_result)
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name,
                                            confirm=True, balise='user_form_div_container', _type='form')
            except (Exception, MissingKey, MissingValue, BadLogin, BadPassword), mess:
                datas = dict(message=mess.message)
            self.form_result = {}
            return self.send_frag(datas, template='horus_gestion_user', templates=used_templates)
        ## 1 -
        menu = G.get_user_menu(self.server_nb, self.name)
        sstitre = self.description
        result = {'sstitre':sstitre, 'titre':titre, 'menus':menu}
        result.update(self._get_form())
        result.update(self._get_valid_btn())
        result['users'] = ['<b>Utilisateurs existants:</b>']
        result['users'].extend(self._get_users())
        return self.send_all(result, template='horus_gestion_user', templates=used_templates)

    def _get_form(self):
        """ renvoie les datas pour la creation du formulaire """
        user_name = {'name':'user_name', 'libelle':"Nom de l'utilisateur",'onblur':"testVoid('user_name', 'error_user_name')", 'inline':'ok'}
        password= {'name':'password', 'libelle':"Mot de passe", 'onblur':"testVoid('password', 'error_password')", 'inline':'ok'}
        quota = {'name':'quota', 'libelle':"Quota utilisateur", 'default_value':'0', 'inline':'ok'}

        grs = _get_groups()
        groups = [{'name':gr, 'libelle':gr}for gr in grs]
        group_select = {'name':'group', 'options':[{'name':'groupes disponibles','options': groups}],
                'libelle':'Groupe principal', 'inline':'ok'}

        profils = [{'name':'itinerant', 'libelle':'itinerant'},
                   {'name':'local', 'libelle':'local', 'default':'ok'},
                   {'name':'obligatoire', 'libelle':'obligatoire'}]
        profil = {'name':'profil', 'options': [{'name':'profils disponibles', 'options':profils}],
                'libelle':'Profil utilisateur', 'inline':'ok'}
        disque = {'name':'disque', 'libelle': "Lettre de lecteur", "default_value":"U:", 'disabled':'true', 'inline':'ok'}
        enable = {'name':'enable', 'value':'on', 'checked':'True', 'libelle':"Activer l'utilisateur", 'inline':'ok'}
        usrs = [{'name':user, 'libelle':user}for user in self._get_users()]
        usrs.append({'name':'', 'libelle':'', 'default':''})
        group_copy = {'name':'copy', 'options':[{'name':'utilisateurs diponibles', 'options':usrs}],
                'libelle':"Copier les groupes d'un autre utilisateur", 'inline':'ok'}
        admin = {'name':'admin', 'libelle': "Membre du groupe DomainAdmins", 'inline':'ok', 'value':'admin'}
        shell = {'name':'shell', 'libelle':"Activation du shell (gestion de clients Linux)", 'inline':'ok', 'value':'shell'}
        return dict(user_name=user_name, password=password, quota=quota, group_select=group_select,
                profil=profil, disque=disque, enable=enable, group_copy=group_copy, admin=admin,
                shell=shell, focus='user_name')

    def _get_valid_btn(self):
        """ renvoie la description du bouton valider """
        validate = {}
        validate['href'] = tools.make_form_link(self.server_nb, self.name, True, ['user'], 'user_msg_div_container')
        validate['icone'] = "/image/ok.gif"
        validate['libelle'] = "Valider"
        validate['title'] = "Créer ce nouvel utilisateur"
        validation = tools.make_key_form_link(self.server_nb, self.name, False,  ['user'], 'user_msg_div_container')
        return dict(validate=validate, validation=validation)

    def _get_users(self):
        """ renvoie la liste des utilisateurs """
        try:
            usrs = horus.get_users()
        except:
            usrs = []
        return usrs

    def create_user(self, dico):
        """ crée un utilisateur avec un dico récupéré depuis l'interface
            :user_name: nom utilisateur
            :group: groupe principal
            :disque: lettre du lecteur associe
            :enable: actif ou pas
            :profil: 'local' ou 'itinerant'
            :copy: utilisateur modele pour la copie des groupes d'appartenance
            :shell: shell actif ou non
        """
        if not dico.has_key('user'):
            raise MissingKey, "Erreur : il manque des données pour créer l'utilisateur."
        result = tools.format_form_result(dico['user'], True)
        for key in ['user_name', 'enable', 'disque', 'quota', 'password',
                    'copy', 'profil', 'admin', 'group', 'shell']:
            if key not in result.keys():
                raise Exception, "Erreur : il manque des données pour la création d'utilisateur."

        name = result['user_name']
        if not tools.test_login(name):
            raise BadLogin

        if result['profil'] == 'itinerant': profil = True
        elif result['profil'] == 'obligatoire': profil = 'obligatoire'
        else: profil = False

        dd = 'U:' # on met u: comme lettre de lecteur

        copy = result['copy']
        if copy != '':
            try:
                grs = horus.get_user_groups(copy)
            except:
                raise Exception , "Erreur : erreur à la récupération des groupes de %s"%copy
        else: grs = []
        gr = result['group'].strip()
        # si l'utilisateur doit être domainAdmin, cela devient son groupe principale
        admin = result['admin']
        if admin:
            grs.append(gr)
            gr = "DomainAdmins"

        if result['quota'] != '': quota = result['quota']
        else:quota = '0'
        if not tools.test_number(quota):
            raise Exception, "Erreur : le quota doit être un nombre."

        if result['password'] != '':
            pwd = result['password']
        if not tools.test_pwd(pwd):
            raise BadPassword

        for i in [name, pwd, gr]:
            if i == '' or not tools.test_ldap_attribute(i):
                raise Exception, "Erreur : une variable est mal formée ou non renseignée."

        activity = result['enable']
        shell = result['shell']

        if not horus.is_user(name):
            a = horus.add_user(name, gr, grs, dd, profil, shell)
            if a:
                commande = '%s %s %s' % (join(SCRIPT_DIR,'set_quota.py'), name, quota)
                code, sortie = commands.getstatusoutput(commande)
                horus.mod_password(name, pwd)
                horus.set_active_users(name, activity)
                return dict(message="L'utilisateur %s a bien été créé."%name)
            else:
                raise Exception, "Erreur : erreur à la création de l'utilisateur."
        else: raise Exception, "Erreur : cet utiilisateur existe déjà"

class GestionUserModify(Action):
    """ modification d'utilisateur """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'gestion_user_modify'
    libelle = "Modification"
    category = "Gestion/Utilisateurs"
    description = 'modifier un utilisateur'
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend",
                            keys=['server', 'action', 'current_user'])
    form_result = Dict(default={}, doc="retour de formulaire en Json", keys=['user', 'active', 'unactive'])

    def execute(self):
        """ renvoie les données pour l'affichage
            1 - Tente de récupérer le nom de l'utilisateur en cours de traitement
            2 - Renvoie le formulaire de modification utilisateur
            3 - Traite le retour formulaire et valide la modification d'utilisateur
        """
        params, self.server_nb = tools.get_request(self.request)
        self.current_user = ''
        ## 3 -
        if self.form_result!={}:
            try:
                datas = self.modify_user(self.form_result)
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name,
                                   confirm=True, current_user=self.current_user)
            except (Exception, MissingKey, MissingValue, BadPassword), mess:
                datas = dict(message=mess.message)
            self.form_result = {}
            return self.send_frag(datas, template='horus_gestion_user',
                                                templates=used_templates)
        ## 1 -
        if 'current_user' in params.keys():
            self.current_user = params['current_user'][0]
        ## 2 -
        menu = G.get_user_menu(self.server_nb, self.name)
        sstitre = self.description
        result = {'sstitre':sstitre, 'titre':titre, 'menus':menu}
        result.update(self._get_form())
        if self.current_user:
            result['usertoassociate'] = {'name':'user', 'default_value':self.current_user}
            result['role'] = get_role_association_btn(self.server_nb, self.current_user)
        return self.send_all(result, template='horus_gestion_user',
                                        templates=used_templates)

    def _get_user_listlist(self):
        """ renvoie les datas pour les listes de choix des groupes """
        result = {}
        result['btns'] = self._get_transf_btns() # boutons de transfert liste a liste
        right_opts = [{'name':gr}for gr in self._get_unactive_groups(self.current_user)]
        result['left'] = {'title':"Groupes disponibles", 'form':"unactive",
                        'datas':right_opts}
        left_opts = [{'name':gr}for gr in self._get_active_groups(self.current_user)]
        result['right'] = {'title':"Groupes de l'utilisateur", 'form':"active",
                            'datas':left_opts} # les autres groupes
        result['title'] = "Associer des groupes à l'utilisateur"
        result['name'] = 'user_group_list'
        return result

    def _get_transf_btns(self):
        """ renvoie les btns de transfert de donnees liste a liste"""
        href1 = "javascript:formTransferListElement('active','unactive_table');"
        href2 = "javascript:formTransferListElement('unactive','active_table');"
        icone1 = "/image/rightleft_red.png"
        icone2 = "/image/leftright.png"
        libelle1 = "Retirer"
        libelle2 = "Ajouter"
        btns = [{'href':href1, 'libelle':libelle1, 'icone':icone1, 'title':libelle2},
                {'href':href2, 'libelle':libelle2, 'icone':icone2, 'title':libelle1}
                ]
        return btns

    def _get_users(self):
        """ renvoie la liste des utilisateurs """
        try:
            usrs = horus.get_users()
        except:
            usrs = []
        return usrs

    def _get_form(self):
        """ renvoie les données pour le formulaire de modification utilisateur """
        usrs = self._get_users()
        if not self.current_user:
            options = [{'name':usr, 'libelle':usr}for usr in usrs]
            options.append({'name':'','libelle':'', 'default':'ok'})
            user_name = {'name':'user_name', 'options':[{'name':'utilisateurs disponibles','options':options}],
                     'libelle':"Nom de l'utilisateur",
                    'onchange':tools.make_js_link(self.server_nb, self.name, code=True,  current_user = 'this.options[this.selectedIndex].value') }
            return dict(user_select=user_name)
        datas = self._get_user_data()
        if self.current_user in usrs:
            usrs.remove(self.current_user)
        # liste des utilisateurs a modifier
        users = [{'name':usr, 'libelle':usr}for usr in usrs]
        users.append({'name':self.current_user, 'libelle':self.current_user, 'default':'ok'})
        options = [{'name':usr, 'libelle':usr}for usr in usrs]
        options.append({'name':self.current_user,'libelle':self.current_user, 'default':'ok'})
        user_name = {'name':'user_name', 'options':[{'name':'utilisateurs disponibles','options':options}],
                 'libelle':"Nom de l'utilisateur",
                 'onchange':tools.make_js_link(self.server_nb, self.name, code=True,  current_user = 'this.options[this.selectedIndex].value') , 'inline':'ok'}
        password= {'name':'password', 'libelle':"Mot de passe", 'inline':'ok'}
        # quota
        if datas.has_key('current_quota'):current_quota = datas['current_quota']
        else:current_quota=''
        quota = {'name':'quota', 'libelle':"Quota utilisateur", 'default_value':current_quota, 'inline':'ok'}
        #construction de la liste des groupes
        grs = _get_groups()
        if datas.has_key('current_gr'):current_gr = datas['current_gr']
        else:current_gr = ''
        if current_gr in grs: grs.remove(current_gr)
        gr_opts = [{'name':gr, 'libelle':gr}for gr in grs]
        gr_opts.append({'name':current_gr, 'libelle':current_gr,  'default':'ok'})
        group_select = {'name':'group', 'options': [{'options':gr_opts}], 'libelle':'Groupe principal', 'inline':'ok'}
        # construction de la liste des profils
        profils = ['itinerant', 'local', 'obligatoire']
        if datas.has_key('current_profil'):current_profil = datas['current_profil']
        else: current_profil = 'local'
        if current_profil in profils: profils.remove(current_profil)
        opt_profils = [{'name':profil, 'libelle':profil} for profil in profils]
        opt_profils.append({'name':current_profil, 'libelle':current_profil, 'default':'ok'})
        profil = {'name':'profil', 'options':[{'options':opt_profils}], 'libelle':'Profil utilisateur', 'inline':'ok'}
        #construction de la liste des lettres de lecteur
        dds = G._get_dds()
        if datas.has_key('current_dd'): current_dd = datas['current_dd']
        else: current_dd = ''
        if current_dd: dds.remove(current_dd)
        dds_opts = [{'name':dd, 'libelle':dd}for dd in dds]
        dds_opts.insert(0, {'name':current_dd, 'libelle':current_dd, 'default':'ok'})
        disque = {'name':'disque', 'options': [{'options':dds_opts}],
                    'libelle':"Lecteur associé ('<b>U:</b>' conseillé)",
                     'inline':'ok'}
        #statut d'activite d'un utilisateur
        enable = {'name':'enable', 'value':'on', 'libelle':"Activer l'utilisateur", 'inline':'ok'}
        activity = True
        if datas.has_key('enable'): activity = datas['enable']
        if activity: enable['checked'] = 'True'
        # activation du shell
        shell = {'name':'shell', 'libelle':"Activation du shell (gestion de clients Linux)", 'inline':'ok', 'value':'shell'}
        if datas['shell']:
            shell['checked'] = 'true'
        # copie de groupes
        users = [{'name':usr, 'libelle':usr}for usr in usrs]
        users.append({'name':'', 'libelle':'','default':'ok'})
        group_copy = {'name':'copy', 'options':[{'options':users}], 'libelle':"Copier les groupes d'un autre utilisateur",
                    'inline':'ok'}
        user_group_list = self._get_user_listlist()
        return dict(user_select=user_name, password=password, quota=quota, group_select=group_select,
                profil=profil, mod_disque=disque, enable=enable, group_copy=group_copy, validate=self._get_valid_btn(),
                user_group_list=user_group_list, shell=shell)

    def modify_user(self, dico):
        """ modifie les attributs d'un utilisateur depuis un dico de type: {'user':dico de modif, 'unactive':, 'active':}
            :user:
                :user_name: nom de l'utilisateur
                :password: mot de passe
                :profil:
                :quota:
                :disque:
                :enable: active l'utilisateur ?
                :group: groupe principal
                :copy: nom d'un utilisateur dont on doit copier les groupes
            :active:
                liste des groupes de l'utilisateur
            :unactive:
                liste des groupes auxquels l'utilisateur n'appartient pas
        """
        if not dico.has_key('user'):
            raise MissingKey, "Erreur : il manque des données pour la validation de formulaire."
        # gestion des parametres utilisateurs
        recup = tools.format_form_result(dico['user'], True)
        self.current_user = recup['user_name']
        pwd, profil, dd, enable, group, enable, groups, shell = [None, None, None, None, None, False, [], None]
        if recup.has_key('password'):
            if recup['password'] != '':
                pwd = recup['password']
                if not tools.test_pwd(pwd):
                    raise BadPassword
        if recup.has_key('profil'):
            if recup['profil'] == 'itinerant':
                profil = True
            elif recup['profil'] == 'local':
                profil = False
            else:
                profil = 'obligatoire'
        if recup.has_key('quota'):
            if recup['quota'] != '':
                if not tools.test_number(recup['quota']): raise Exception, "Erreur : Le quota doit être un nombre."
                commande = '%s %s %s'%(join(SCRIPT_DIR,'set_quota.py'), self.current_user, recup['quota'])
                code, sortie = commands.getstatusoutput(commande)
                if not code: raise Exception, "Erreur : erreur d'attribution de quota."
        if recup.has_key('disque'):
            if recup['disque'] != '': dd = recup['disque']
            dd = dd.upper()
            if ':' not in dd: dd+=':'
            if not tools.test_drive_letter(dd):
                raise BadDriveLetter
        if recup.has_key('shell'): shell = recup['shell']
        if recup.has_key('enable'): enable = recup['enable']
        if recup.has_key('group'):
            if recup['group'] != '': gr = recup['group']
        if recup.has_key('copy'):
            try:
                groups = horus.get_user_groups(recup['copy'])
            except:
                groups = []
        if self.current_user is not None:
            a = horus.mod_user(self.current_user, homedrive=dd, enable=enable, profile=profil, gid=gr, shell=shell)
            if not a:
                raise Exception, "Echec lors de la modification de l'utilisateur %s"%self.current_user
            horus.set_active_users(self.current_user, enable) # on gere l activite d'un utilisateur
            if pwd != None:
                horus.mod_password(self.current_user, pwd) # on change le mot de passe
            if dico.has_key('active') and dico.has_key('unactive'):
                active = dico['active']
                unactive = dico['unactive']
                # on teste les changements pour eviter les changements dans le vide
                actual_groups = self._get_active_groups(self.current_user)
                actual_no_groups = self._get_unactive_groups(self.current_user)
                for gr in active:
                    if gr['name'] not in actual_groups:
                        horus.inscription(self.current_user, gr['name'])
                for ugr in unactive:
                    if ugr['name'] not in actual_no_groups:
                        if ugr['name'] not in groups:
                            horus.desinscription(self.current_user, ugr['name'])
                            # si on a demande a copier des groupes depus un utilisateur, on y donne priorite
                    elif ugr['name'] in groups:
                        horus.inscription(self.current_user, ugr['name'])
        return dict(message="L'utilisateur %s a bien été modifié"%self.current_user)

    def _get_user_data(self):
        """ renvoie les attributs de user """
        user = self.current_user
        if not user:
            return {}
        if not horus.is_user(user):
            return {}
        try:
            datas = horus.get_user_data(user)
        except:
            return {}
        if 'sambaProfilePath' in datas.keys():
            if datas['sambaProfilePath'][0].endswith("\\netlogon\\profil"):
                profil = 'obligatoire'
            else:
                profil = 'itinerant'
        else:
            profil = 'local'
        try:
            actives = horus.get_active_users()
            if user in actives:
                enable = True
            else:
                enable = False
        except:
            enable = True
        dd = datas['sambaHomeDrive'][0]
        gr = horus.get_gid(user)
        if datas.has_key('loginShell') and datas['loginShell'][0] == '/bin/bash':
            shell = True
        else:
            shell = False
        commande = '%s %s' % (join(SCRIPT_DIR,'get_quota.py'), user)
        code, sortie = commands.getstatusoutput(commande)
        quota = 0
        if code == 0:
            quota = sortie
        return {'current_gr':gr, 'current_profil': profil, 'current_dd': dd,
                'current_quota':quota, 'enable':enable, 'shell':shell}

    def _get_active_groups(self, user):
        """ renvoie les groupes actifs pour user"""
        if user is None:
            return []
        try:
            usr_grs = horus.get_user_groups(user)
        except:
            usr_grs = []
        return usr_grs

    def _get_unactive_groups(self, user):
        """ renvoie les groupes inactifs pour user"""
        if user is None:
            return []
        try:
            usr_grs = horus.get_user_no_groups(user)
        except:
            usr_grs = []
        return usr_grs

    def _get_valid_btn(self):
        """ renvoie la description du bouton valider """
        validate = {}
        validate['href'] = tools.make_form_link(self.server_nb,self.name, True, ['user', 'active', 'unactive'], 'user_msg_div_container')
        validate['icone'] = "/image/ok.gif"
        validate['libelle'] = "Valider"
        validate['title'] = "Modifier l'utilisateur"
        return validate

class GestionUserDelete(Action):
    """ Suppression d'utilisateur """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'gestion_user_suppr'
    libelle = "Suppression"
    category = "Gestion/Utilisateurs"
    description = 'Supprimer un utilisateur'
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend", keys=['server', 'action'])
    form_result = Dict(default={}, doc='retour formulaire pour la suppression', keys=['user'])

    def execute(self):
        """ renvoie les données pour l'affichage
            1 - renvoie le formulaire de suppression d'utilisateur
            2 - Traite le retour formulaire et supprime un utilisateur
        """
        params, self.server_nb = tools.get_request(self.request)
        menu = G.get_user_menu(self.server_nb, self.name)
        removing = None
        ## 2 -
        if self.form_result!={}:
            try:
                datas = self.del_user(self.form_result)
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name, confirm=True)
            except Exception, mess:
                datas = dict(message=mess.message)
            self.form_result = {}
            return self.send_frag(datas, template='horus_gestion_user', templates=used_templates)
        sstitre = self.description
        result = {'sstitre':sstitre, 'titre':titre, 'menus':menu}
        if removing:
            retour += " ainsi que son répertoire"
        ## 1 -
        result.update(self._get_form())
        result['validate'] = self._get_valid_btn()
        return self.send_all(result, template='horus_gestion_user', templates=used_templates)

    def _get_form(self):
        """ renvoie la description du formulaire de suppression utilisateur """
        form = []
        try:
            usrs = horus.get_users()
        except:
            usrs = []
        users = [{'name':usr, 'libelle':usr}for usr in usrs]
        user_select = {'name':'user_name', 'options':[{'name':'utilisateurs existants', 'options':users}],
                        'libelle':"Nom d'utilisateur"}
        del_rep = {'name':'remove_data', 'value':'on', 'checked':'True', 'libelle':"Supprimer son répertoire"}
        return dict(user_select=user_select, enable=del_rep)

    def del_user(self, dico):
        """ supprimer un utilisateur """
        if not dico.has_key('user'):
            raise Exception, "Erreur : il manque des données pour la suppression utilisateur."
        result = tools.format_form_result(dico['user'],True)
        name = result['user_name']
        removing = result['remove_data']
        if not horus.del_user(name, removing):
            raise Exception, "Erreur : une erreur est survenue lors de la suppression de %s."%name
        message = "L'utilisateur %s a bien été supprimé.\\n"%name
        if removing: message += "Ainsi que son répertoire."
        return dict(message=message)

    def _get_valid_btn(self):
        """ renvoie la description du bouton valider """
        validate = {}
        href = tools.make_form_link(self.server_nb, self.name, False, ['user'], balise='user_msg_div_container')
        validate['href'] = tools.make_confirm_link("Etes-vous sûr de vouloir supprimer l'utilisateur", href)
        validate['icone'] = "/image/ok.gif"
        validate['libelle'] = "Valider"
        validate['title'] = "Supprimer cet utilisateur"
        return validate
