# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# Outil de listing des fichiers de logs
#
#
###########################################################################

""" Liste les fichiers de log de dansguardian et les classe par jour """
from ead2.backend.actions.lib.logparser.config import LOG_DIR, log_root_name
from os.path import join, isdir
from os import listdir
import datetime, re
day_dict = ['lundi', 'mardi', 'mercredi', 'jeudi', 'vendredi', 'samedi', 'dimanche']
def get_today_index():
    timer = datetime.date
    return timer.weekday(timer.today())

def get_days(nums=1):
    """ renvoie un dico {'precweek': liste des jours de cette semaine, 'precweek': liste des jours de la semaine denière}
            avec les nums derniers jours dans l'ordre décroissant depuis aujourd'hui"""
    today_index = get_today_index()
    if (today_index+1) > nums:
        firstday_index = (today_index+1) - nums
    else:
        firstday_index = 0
    thisweek = day_dict[firstday_index:(today_index+1)]
    thisweek.reverse()
    left_nums = today_index+1 - nums
    if left_nums < 0:
        precweek = day_dict[left_nums:]
        precweek.reverse()
    else:
        precweek = []
    return thisweek, precweek

def get_logs(zone=0):
    root_name = log_root_name%zone
    loglist = [_file for _file in listdir(LOG_DIR) if root_name in _file and not '~' in _file]
    loglist.sort(compare_func)
    return loglist

def get_logs_for_form(zone):
    """ renvoie l'ensemble des logs classés avec jour et semaine formaté pour la création de formulaire """
    loglist = get_logs(zone)
    thisweek, precweek = get_days(len(loglist))
    first, second = [], []
    for day in thisweek: first.append({'libelle':day, 'name':loglist.pop(0)})
    for day in precweek: second.append({'libelle':day, 'name':loglist.pop(0)})
    return [{'name':'', 'options':[{'name':'vide', 'libelle':'-----'}]},
            {'name':"Cette semaine",'options':first}, {'name':"La semaine dernière", "options":second}]

###############################
# REFACTORING POUR DANSGUARIAN
###############################

def get_logs_filenames():
    """ renvoie la liste des fichiers de logs par zone """
    from amon.backend import get_filter_zones
    filenames = {}
    for _file in listdir(LOG_DIR):
        for instance_indice, ifaces in get_filter_zones().items():
            if len(ifaces)>0:
                instance_log_root_name = indiced_root_name%instance_indice
                filenames[instance_indice] = get_logs(instance_log_root_name)
    return filenames

def get_logs_form():
    """ renvoie l'ensemble des logs classés avec jour et semaine formaté pour la création de formulaire """
    loglist = get_logs_filenames()
    length = max([len(filenames) for filenames in loglist.values()])

    thisweek, precweek = get_days(length)
    first = [{'libelle':day, 'name':day+'0'}for day in thisweek]
    second = [{'libelle':day, 'name':day+'1'}for day in precweek]
    return [{'name':'', 'options':[{'name':'vide', 'libelle':'-----'}]},
            {'name':"Cette semaine",'options':first}, {'name':"La semaine dernière", "options":second}]

def get_filenames_from_days(day):
    """ renvoie le nom des fichiers de logs du jour day """
    week = int(day[-1:])
    indice = get_today_index() + 7*week
    day_index = day_dict.index(day[:-1])

    print "today %s"%get_today_index()
    print "Day_index %s"%day_index
    print "week : %s"%week
    print "indice : %s"%indice


## utilitaires de comparaison
def compare_func(a, b):
    """ fonction de comparaison """
    un = _get_nb_in_name(a)
    deux = _get_nb_in_name(b)
    return cmp(un, deux)

def _get_nb_in_name(name):
    """nombre present dans un nom (de fichier),
    utilisé pour parser 'access.log.1.gz', 'access.log.2.gz', 'access.log.3.gz',
    """
    regexp = "[0-9]+"
    if re.search(regexp, name):
        name = re.findall(regexp, name)
        return int(name[0])
    else:
        return -1
