# -*- coding: iso-8859-1 -*-
#
# module de gestion des devoirs
# un devoir est :
#    distribu => ramass => rendu => purg (supprime les donnes)
#
# lorsque des donnes sont ajoutes au devoir, elles sont stockes dans
# \\scribe\devoirs\user\user-00001\donnees
# des liens sont ensuite crs lors de la distribution


import shutil, os, pickle

#temp_dir = '/tmp/'
import tempfile
temp_dir = tempfile.gettempdir()

class GestDevoir:
    def __init__(self, pbroot, username):
        """ username: uid de l'utilisateur grant les devoirs
            pbroot: 
        """
        self.pbroot = pbroot
        self.username = username
        self.dev_dir = os.path.join('/home','workgroups', 'devoirs', username)
        self.data_f = os.path.join(self.dev_dir, 'devoirs.dat')
        
    def init_reps(self):
        """Initialise le nom des rpertoires temporaire et de destination
        dev_name = user-00001
        dest = \\scribe\devoirs\user\user-00001(\donnees)
        """
        self.dev_name = '%s-%05.f'%(self.username, int(self.dict['num_devoir']))
        self.tmpdir = os.path.join(temp_dir, self.dev_name)
        self.dondir = os.path.join(self.tmpdir, 'donnees')
        self.dest_dev = os.path.join(self.dev_dir, self.dev_name)
        
    def create(self):
        """Cre
        self.tmpdir = GetTempPath()\user-00001(\donnees)
        """
        self.load_dict()
        self.dict['num_devoir'] += 1
        self.init_reps()
        while os.path.exists(self.dest_dev):
            self.init_reps()
            self.dict['num_devoir'] += 1
        if os.path.exists(self.tmpdir):
            shutil.rmtree(self.tmpdir)
        os.makedirs(self.tmpdir)

    def load_dict(self):
        """charge \\scribe\devoirs\user\devoirs.dat
        """
        try: self.dict = pickle.load(file(self.data_f))
        except: self.dict = { 'num_devoir': 0, 'liste_devoirs': {} }
        
    def save_dict(self):
        """sauve \\scribe\devoirs\user\devoirs.dat
        """
        pickle.dump(self.dict, file(self.data_f,'w'))
        
    ##################
    ## Distribution ##
    ##################
    def add_Devoirs(self, flist):
        """Ajoute les fichiers de la premire listeBox
        """
        for i in flist:
            shutil.copy(i, self.tmpdir)
    
    def add_Donnees(self, flist):
        """Ajoute les fichiers de la seconde listeBox
        """
        if len(flist) > 0:
            os.makedirs(self.dondir)
            for i in flist:
                shutil.copy(i, self.dondir)
    
    def copy_devoir(self, flist, dlist):
        self.add_Devoirs(flist)
        self.add_Donnees(dlist)
        shutil.copytree(self.tmpdir, self.dest_dev)
        shutil.rmtree(self.tmpdir)

    def distribute(self, flist, dlist, groupe):
        """distribue le contenu de self.arch dans les dossiers des membres du groupe "groupe"
        le nom de destination HOMEDIR/perso/Devoirs/nom_prof-dev_num
        si le dossier existe dj, il est pralablement supprim
        """
        self.groupe = groupe
        self.flist, self.dlist = flist, dlist
        self.create()
        self.copy_devoir(self.flist, self.dlist)
        d = self.pbroot.callRemote("distribute", self.username, self.groupe, self.dev_name)
        d.addCallback(self.return_dist)
        return d

    def return_dist(self, num):
        self.dict['liste_devoirs'][self.dev_name] = (self.groupe, 'dist')
        self.save_dict()
        return num
    
    ###############
    ## Ramassage ##
    ###############
    def get_dist_devoirs(self):
        """retourne la liste des devoirs distribus non-ramasss
        """
        self.load_dict()
        liste = []
        for dev in self.dict['liste_devoirs'].keys():
            groupe, etat = self.dict['liste_devoirs'][dev]
            if etat == 'dist': liste.append('%s - %s'%(dev, groupe))
        liste.sort()
        return liste
    
    def ramasse(self, dev):
        self.dev_name, self.groupe = dev.split(' - ')
        d = self.pbroot.callRemote("ramasse", self.username, self.groupe, self.dev_name)
        d.addCallback(self.return_ram)
        return d
    
    def return_ram(self, ret):
        self.dict['liste_devoirs'][self.dev_name] = (self.groupe, 'ram')
        self.save_dict()
        return ret

    #############
    ## Rendage ##
    #############
    def get_ram_devoirs(self):
        """retourne la liste des devoirs distribus non-ramasss
        """
        self.load_dict()
        liste = []
        for dev in self.dict['liste_devoirs'].keys():
            groupe, etat = self.dict['liste_devoirs'][dev]
            if etat == 'ram': liste.append('%s - %s'%(dev, groupe))
        liste.sort()
        return liste
    
    def rendre(self, dev):
        self.dev_name, self.groupe = dev.split(' - ')
        d = self.pbroot.callRemote("rendre", self.username, self.groupe, self.dev_name)
        d.addCallback(self.return_ren)
        return d
    
    def return_ren(self, ret):
        self.dict['liste_devoirs'][self.dev_name] = (self.groupe, 'ren')
        self.save_dict()
        return ret

    #################
    ## Suppression ##
    #################
    def get_ren_devoirs(self):
        """retourne la liste des devoirs distribus non-ramasss
        """
        self.load_dict()
        liste = []
        for dev in self.dict['liste_devoirs'].keys():
            groupe, etat = self.dict['liste_devoirs'][dev]
            if etat == 'ren': liste.append('%s - %s'%(dev, groupe))
        liste.sort()
        return liste
    
    def suppr(self, dev):
        self.dev_name, self.groupe = dev.split(' - ')
        d = self.pbroot.callRemote("supprimer", self.username, self.groupe, self.dev_name)
        d.addCallback(self.return_suppr)
        return d
    
    def return_suppr(self, ret):
        del(self.dict['liste_devoirs'][self.dev_name])
        self.save_dict()
        return ret
