# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# Action Maj
#
# Programme des mises à jour du serveur de commande depuis l'ead
#
###########################################################################

""" Actions de programmation de mise à jour du serveur """
from twisted.python import log
from ead2.backend.lib.action import * # import des actions  et types
from ead2.backend.actions import tools
from ead2.lib.libead import uni
from ead2.backend.config.config import MAJ_TEMP_DIR, SCRIPT_DIR
from ead2.backend.config.filenames import maj
from os.path import join, isfile
import urllib
from creole.parsedico import parse_dico
from creole import maj as lance_maj

maj_mode = parse_dico()['type_maj']

maj_filename = maj['maj']
majauto_filename = maj['auto']

def get_maj_mode():
    if maj_mode.lower() == 'complete':return "La mise à jour est configurée comme Complète."
    else: return "La mise à jour est configurée comme Minimale (seuls les paquets de sécurité sont mis à jour)."

class Maj(Action):
    """ Page de programmation de mise à jour"""
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'maj'
    libelle = "Mise à jour"
    category = "Système"
    description = "Mise à jour du système"
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend",
                        keys=['server', 'action', 'active', 'cancel_maj', 'paquet'])
    form_result = Dict(default={}, doc='retour de formulaire', keys=['maj'])

    def execute(self):
        """ gestionnaire de l'applicatin de mise à jour
            1 - Renvoi la description du formulaire de programmation de mise à jour
            2 - Programme des mises à jour
            3 - Annule des mises à jour
            4 - Liste les paquets à mettre à jour
            5 - Active la mise à jour auto
        """
        params, self.server_nb = tools.get_request(self.request)
        result = {'titre':'Gestionnaire de mise à jour', "sstitre":"Mise à jour programmée"}
        result['mode_maj'] = get_maj_mode()
        retour_msg, retour_code = [None, None]
        ## 2 -
        if self.form_result.has_key('maj'):
            retour_msg = self._set_maj()
            self.form_result = {}
        ## 5 -
        if params.has_key('active'):
            retour_msg = self._set_maj(True, params['active'][0])
        ## 3 -
        if params.has_key('cancel_maj'):
            retour_msg = self._unset_maj()
        ## 4 -
        if params.has_key('paquet'):
            result['titre'] = "Liste des paquets disponibles"
            result['datas'] = self._get_pkgs()
            result['retour'] = self._get_retour_btn()
            d = {'template':'maj_paquet', 'data':{'content':result}}
            return 0, uni(urllib.quote(str(d)))
        if retour_msg is not None:
            result['retour_msg'] = retour_msg
        else:
            result['retour_msg'] = ''
        ## 1 -
        result['paquets'] = self._get_paquet_btn()
        result['formulaires'] = [{'name':'maj', 'formulaire':self._get_form(), 'titre':'Programmez une mise à jour',
                                 'valide':self._get_valide_btn('maj')}]
        result.update(self._get_maj(True))
        result.update(self._get_maj())
        return self.send_all(result, template='maj', templates=['main'])

# methode d'envoi des donnees de mise en forme

    def _get_form(self):
        """ renvoit le formulaire """
        tag = {'_type':'choice', 'name':'majheure', 'default_value': {'name':'0', 'libelle':'0 heure'},
                'values':self._get_hours(), 'libelle':'Mettre à jour dans '}
        return [tag]

    def _get_pkgs(self):
        """ renvoit la liste des paquets """
        a, b = tools.command_statusoutput('/usr/bin/Query-Auto -W')
        if a:
            log.err("Erreur lors du lancement de '/usr/bin/Query-Auto -W'")
            return "Erreur lors du listing des paquets"
        else:
            return b

    def _get_retour_btn(self):
        """ renvoit le bouton de retour pour la page de listing des paquets """
        return_btn = {}
        return_btn['href'] = tools.make_js_link(self.server_nb, self.name)
        return_btn['icone'] = "/image/back.png"
        return_btn['libelle'] = "Retour"
        return_btn['title'] = "Retour à la page de mise à jour"
        return return_btn

    def _get_valide_btn(self, formname):
        """ renvoit la description du bouton valider """
        validate = {}
        message = "Etes vous sûr de vouloir programmer cette mise à jour ?"
        a = tools.make_form_link(self.server_nb, self.name, False, [formname])
        validate['href'] = tools.make_confirm_link(message, a)
        validate['icone'] = "/image/ok.gif"
        validate['libelle'] = "OK"
        validate['title'] = "Programmer la mise à jour"
        return validate

    def _get_active_btn(self, active=False):
        """ renvoit la description du bouton activer/désactiver la mise à jour auto """
        btn = {}
        if active:
            btn['icone'] = "/image/ok.gif"
            btn['href'] = tools.make_js_link(self.server_nb, self.name, active='True')
            btn['libelle'] = "Activer"
            btn['title'] = "Activer la mise à jour auto"
        else:
            btn['icone'] = "/image/suppr.gif"
            btn['href'] = tools.make_js_link(self.server_nb, self.name, active='False')
            btn['libelle'] = "Désactiver"
            btn['title'] = "Désactiver la mise à jour auto"
        return btn

    def _get_paquet_btn(self):
        """ description du bouton pour l'affichage des paquets dispo """
        btn  = {}
        btn['href'] = tools.make_js_link(self.server_nb, self.name, paquet='liste')
        btn['icone'] = "/image/"
        btn['libelle'] = "Voir la liste des paquets à mettre à jour (cette action peut prendre du temps)"
        btn['title'] = "Liste les paquets disponibles pour la mise à jour"
        return btn

    def _get_hours(self):
        """ renvoit les heures (depuis 1->23) pour les formulaires """
        hours = []
        for i in range(24)[1:]:
            hour = {'name':'%s'%i, 'libelle':'%s heure'%i}
            hours.append(hour)
        return hours

    def _get_maj(self, auto=False):
        """ renvoit la liste des mises à jour programmée ou auto"""
        if auto:
            if isfile(majauto_filename):
                fichier = file(majauto_filename, 'r')
                lines = fichier.read()
                fichier.close()
                if lines:
                    btn = self._get_active_btn(False)
                    return dict(active={'texte':lines, 'btn':btn, 'msg':"Mise à jour automatique"})
            btn = self._get_active_btn(True)
            return dict(active={'texte':"La mise à jour automatique est désactivée", 'btn':btn,
                                    'msg':"Mise à jour automatique"})
        else:
            maj_filename = join(MAJ_TEMP_DIR, 'maj-diff.txt')
            if isfile(maj_filename):
                fichier = file(maj_filename, 'r')
                lines = fichier.read()
                fichier.close()
                if lines:
                    return dict(maj={'texte':lines, "btn":self._get_cancel_maj_btn()})
            return dict(maj={'texte':"Aucune mise à jour n'est programmée"})

    def _get_cancel_maj_btn(self):
        """ description du bouton d'annulation de la mise a jour programmee """
        cancel = {}
        cancel['href'] = tools.make_js_link(self.server_nb, self.name, cancel_maj='true')
        cancel['icone'] = "/image/suppr.gif"
        cancel['libelle'] = "Annuler"
        cancel['title'] = "Annuler la mise à jour"
        return cancel

# methode d'action sur les programmations de mise a jour
    def _unset_maj(self):
        """ annule les mise a jour programmee """
        retour = lance_maj.cancel_maj_differee()
        if retour:
            return "La mise à jour a été annulée."
        else:
            log.err("Erreur lors du lancement de creole.lance_maj.cancel_maj_differee()")
            return "Echec lors de l'annulation de la mise à jour."

    def _set_maj(self, auto=False, *kw):
        """ programme la mise à jour """
        if not auto:
            result = tools.format_form_result(self.form_result['maj'])
            heure = result['majheure']
            return self._set_maj_differe(heure)
        else:
            ## activite vient du frontend (str et pas booleen)
            activite = kw[0]
            if activite == 'True':
                return self._set_maj_auto()
            elif activite == 'False':
                return self._set_maj_auto(False)
            else:
                return "Erreur lors de l'activation/désactivation de la mise à jour automatique."

    def _set_maj_differe(self, nb_heure):
        """ programme une mise à jour différée dans nb_heure"""
        retour = lance_maj.prog_maj_differee(nb_heure)
        if retour:
            return "La mise à jour s'effectuera dans %s heures."%nb_heure
        else:
            log.err("Erreur lors de la programmation de mise à jour.")
            return "Erreur lors de la programmation de la mise à jour."

    def _set_maj_auto(self, activation=True):
        """ active la mise à jour automatique
            :activation: True/False, active/desactive
        """
        if activation:
            retour = lance_maj.enable_maj_auto()
            if retour:
                return "La mise à jour a été programmée."
            else:
                log.err("Erreur lors du lancement de creole.lance_maj.enable_maj_auto().")
                return "Erreur lors de l'activation/désactivation de la mise à jour automatique."
        else:
            retour = lance_maj.disable_maj_auto()
            if retour:
                return "La mise à jour automatique a été annulée."
            else:
                log.err("Erreur lors du lancement de creole.lance_maj.disable_maj_auto().")
                return  "Erreur lors de l'activation/désactivation de la mise à jour automatique."
