# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# gestion de l'épreuve assr
#
###########################################################################
"""Action permettant de gérer épreuve assr"""
from twisted.python import log
from ead2.backend.lib.action import * # import des actions et des types
from ead2.backend.actions import tools
from ead2.lib.error import MissingKey, MissingValue
from ead2.backend.config.filenames import assr
from os.path import isdir, isfile

import traceback

# liste des templates complémentaires utilisés
used_templates = ['main', 'accordion', 'form', 'formselect']

def get_menu(server_nb, page=''):
    """ renvoie la description du menu
        :page: page en cours d'affichage
    """
    config = {'title':"Configuration de l'application", 'icone':'/image/refresh.gif',
                'href':tools.make_js_link(server_nb, "scribe_assr_config"), 'libelle':"Configuration Épreuve@ssr"}
    download = {'title':'Installation par téléchargement', 'icone':'/image/refresh.gif',
                'href':tools.make_js_link(server_nb, "scribe_assr_download"), 'libelle':"Téléchargement Épreuve@ssr"}
    if page == 'config':
        config['href'] = ''
    elif page == 'download':
        download['href'] = ''
    return dict(menus=(config, download))

def get_config_status():
    if isfile(assr['conf_apache']) and isdir(assr['dir']):
        return 0, "L'application est <font color=red><b>Activée</b></font>"
    elif isdir(assr['dir']):
        return 1, "L'application est <font color=red><b>Désactivée</b></font>"
    else:
        return 2, ""

class AssrConfig(Action):
    """
    Action de gestion de la configuration d'épreuve assr
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'scribe_assr_config'
    libelle = "Configuration"
    category = "Outils/Épreuve@ssr"
    description = "Gestion de l'application Epreuve@SSR"
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend", keys=['action', 'server', 'suppr'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['assr_config'])

    def execute(self):
        """ renvoie les données pour l'affichage
            1 - renvoie le formulaire
            2 - valide le formulaire de configuration d'assr
            3 - supprime les données relatives à assr
        """
        params, self.server_nb = tools.get_request(self.request)
        ## 3 -
        if params.has_key('suppr'):
            try:
                datas = self.suppr_assr()
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name)
            except (MissingKey, MissingValue, Exception), mess:
                log.err(mess)
                traceback.print_exc()
                datas = {'message':mess.message}
            params.pop('suppr')
            return self.send_frag(datas, template='scribe_assr', templates=used_templates)
        ## 2 -
        if self.form_result != {}:
            try:
                datas = self._valid_form()
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name)
            except (MissingKey, MissingValue, Exception), mess:
                log.err(mess)
                traceback.print_exc()
                datas = {'message':mess.message}
            self.form_result = {}
            return self.send_frag(datas, template='scribe_assr', templates=used_templates)
        ## 1 -
        result = {'titre':self.description,
                  'entete':"@SSR (Attestations scolaires de sécurité routière)",
                  'status':get_config_status()[1]}
        result.update(self._get_form())
        return self.send_all(result, template='scribe_assr', templates=used_templates)

    def _get_form(self):
        """ renvoie les données pour la mise en forme du formulaire"""
        libelles = ["Activer @SSR", 'Désactiver @SSR']
        active = {'libelle':'', 'name':'activation', 'value':'active', 'inline':'ok'}
        inactive = {'libelle':'', 'name':'activation', 'value':'desactive', 'inline':'ok'}

        if get_config_status()[0] == 0:
            active['checked'] = 'ok'
        elif get_config_status()[0] == 1:
            inactive['checked'] = 'ok'

        user_force = {'libelle':"Forcer la re-création de l'utilisateur \"user.assr\"",
                      'name':'user_force',
                      'value':'on',
                      'inline':'ok'}
        return dict(libelles=libelles,
                    active=active,
                    inactive=inactive,
                    user_force=user_force,
                    suppr_btn=self._get_suppr_btn(),
                    validate=self._get_validate_btn())

    def _get_validate_btn(self):
        """ renvoie la description du bouton valider """
        href = tools.make_form_link(self.server_nb, self.name, True, ['assr_config'], 'div_message')
        title = ''
        icone = '/image/ok.gif'
        libelle = "Valider"
        return dict(href=href, title=title, icone=icone, libelle=libelle)

    def _get_suppr_btn(self):
        """ renvoie le bouton de suppression de assr """
        ref = tools.make_js_link(self.server_nb, self.name, confirm=True, balise='div_message', suppr='true')
        href = tools.make_confirm_link("Êtes-vous sûr de vouloir supprimer toutes les données relatives à Epreuve@ssr?", ref)
        title = ''
        icone = '/image/suppr.gif'
        libelle = "Supprimer épreuve@ssr et les données téléchargées."
        _class = 'btn'
        return dict(href=href, title=title, icone=icone, libelle=libelle, _class=_class)

    def _valid_form(self):
        """ valide le formulaire de configuration assr"""
        result = tools.format_form_result(self.form_result['assr_config'])
        if not 'activation'in result.keys():
            log.err("Assr_config il manque des données pour la validation du formulaire.")
            raise MissingKey, "Erreur: erreur à la configuration de épreuve @ssr, il manque des données"
        activation = result['activation'].strip()
        user_info = ''
        if activation == 'active':
            active = 'Activée'
            if result.has_key('user_force'):
                a, user_info = tools.command_statusoutput("%s -c -d"%assr['user_script'])
                log.msg("Création forcée de l'utilisateur user assr %s"%user_info)
            else:
                a, user_info = tools.command_statusoutput("%s -c 2>&1"%assr['user_script'])
            user_info = user_info.replace('\n', '\\n')
            user_info = user_info.replace('"', "'")
            if a:
                log.err("Erreur au lancement du script %s, retour: %s"%(assr['user_script'], user_info))
                raise Exception, "Erreur : problème à la création de l'utilisateur user.assr"
        elif activation == 'desactive': active = 'Désactivée'
        else: active = "un truc bizarre s'est produit"
        a,b = tools.command_statusoutput("%s %s"%(assr['script'], activation))
        if a:
            log.err("Erreur au lancement de %s avec l'option %s"%(assr['script'], activation))
            raise Exception, "Erreur : erreur à la configuration de Epreuve@ssr"
        return dict(message="L'application Epreuve@ssr a bien été %s,\\n %s"%(active, user_info),
                    modified_status="L'application Epreuve@ssr a été <font color=red><b>%s</b></font>"%active)

    def suppr_assr(self):
        """ supprime epreuveassr et toutes les données associées """
        active = 'Supprimée'
        a,b = tools.command_statusoutput("%s supprime"%assr['script'])
        if a:
            log.err("Erreur au lancement de %s avec l'option supprime, retour:%s"%(assr['script'], b))
            raise Exception, "Erreur : erreur à la configuration de Epreuve@ssr"
        return dict(message="L'application Epreuve@ssr a bien été %s"%active,
                    modified_status="L'application Epreuve@ssr a été <font color=red><b>%s</b></font>"%active)

class AssrDownload(Action):
    """
    télécharge les données relatives à l'épreuve@ssr
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'scribe_assr_download'
    libelle = "Téléchargement"
    category = "Outils/Épreuve@ssr"
    description = "Gestion de l'application Epreuve@SSR"
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend", keys=['action', 'server', 'cancel'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['assr_download'])

    def execute(self):
        """ renvoit les données pour l'affichage
            1 - vérifie que erpreuve@ssr est bien configuré
            2 - renvoie le formulaire pour le téléchargement
            3 - lance un téléchargement
            4 - annule un téléchargement
        """
        params, self.server_nb = tools.get_request(self.request)
        ## 1 -
        if not isfile(assr['conf_apache']) or not isdir(assr['dir']): # assr activée ??
            datas = {'message':"Activez Epreuve@ssr avant de télécharger.",
                        "redirect":tools.make_js_link(self.server_nb, 'scribe_assr_config', confirm=True)}
            return self.frag(datas)
        ## 4 -
        if params.has_key('cancel'): ## on annule un tel
            try:
                datas = self._cancel()
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name)
            except Exception, mess:
                log.err(mess)
                traceback.print_exc()
                datas = {'message':mess.message, 'cancel':self._get_cancel_btn()}
                datas.update(self._get_form(True))
            return self.frag(datas)
        ## 3 -
        if self.form_result != {}: # on lance un tel
            try:
                datas = self._valid_form()
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name)
            except (MissingKey, MissingValue, Exception), mess:
                log.err(mess)
                traceback.print_exc()
                datas = {'message':mess.message}
                datas.update(dict(validate=self._get_validate_btn()))
            self.form_result = {}
            return self.frag(datas)
        ## 2 -
        result = {'titre':self.description, 'entete':"Installation par téléchargement"}
        result.update(self._get_form())
        result.update(self._get_report())
        return self.send_all(result, template='scribe_assr', templates=used_templates)

    def frag(self, datas):
        """ renvoie des données pour un fragment de page"""
        return self.send_frag(datas, template='scribe_assr', templates=used_templates)

    def _get_form(self, disabled=False):
        """ renvoie les données pour la mise en forme du formulaire"""
        adresse={'libelle':"Adresse du serveur rsync", "name":"rsync_serv", 'inline':'ok'}
        user_name = {'libelle':"Nom d'utilisateur", "name":"user_name", 'inline':'ok'}
        password = {'libelle':"Mot de passe", 'name':'password', 'inline':'ok'}
        validate = self._get_validate_btn()

        if isfile(assr['url']):
            adresse['default_value'], user_name['default_value']= file(assr['url']).read().splitlines()
        if isfile(assr['pass']):
            password['default_value'] = file(assr['pass']).read().strip()

        dico = dict(adresse=adresse, user_name=user_name, password=password)

        if isfile(assr['at']):
            dico['status_texte'] = file(assr['at']).read()
            dico['cancel_btn'] = self._get_cancel_btn()
        elif not disabled:
            dico['validate'] = validate

        return dico

    def _get_report(self):
        """ renvoie les données pour la mise en forme du rapport de téléchargement assr """
        if isfile(assr['rapport']):
            rapport = file(assr['rapport'], 'r').read().splitlines()
            return dict(rapport=rapport, rapport_msg="Compte rendu du dernier téléchargement")
        return {}

    def _cancel(self):
        """ annule le téléchargement """
        a,b = tools.command_statusoutput("%s annul_telech"%assr['script'])
        if a:
            log.err("Assr_download: une erreur s'est produite à l'annulation du téléchargement, retour:%s"%b)
            raise Exception , "Erreur : Un problème est survenue à l'annulation du téléchargement de Epreuve@ssr."
        return dict(message="Téléchargement annulé.")

    def _valid_form(self):
        """ lance le téléchargement de  @ssr"""
        result = tools.format_form_result(self.form_result['assr_download'])
        if not result.has_key('rsync_serv') or not result.has_key('user_name') or not result.has_key('password'):
            log.err("Assr_download: il manque des données pour le téléchargement de épreuve assr.")
            raise MissingKey, "Erreur : une erreur s'est produite lors du lancement du téléchargement."
        rsync_serv = result['rsync_serv'].strip()
        user_name = result['user_name'].strip()
        password = result['password'].strip()

        a,b = tools.command_statusoutput("%s telecharge %s %s %s"%(assr['script'], rsync_serv, user_name, password))
        if a:
            log.err("Assr_download: une erreur s'est produite au lancement du téléchargement de Epreuve@ssr, retour:%s."%b)
            raise Exception , "Erreur : Un problème est survenu au lancement du téléchargement de Epreuve@ssr."
        else:
            message = "Le téléchargement de Epreuve@ssr a bien été activé.\\n"
            message += "Le téléchargement s'effectuera auprès du serveur %s,\\n avec le login %s"%(rsync_serv, user_name)
            return dict(message=message, cancel_btn=self._get_cancel_btn())

    def _get_validate_btn(self):
        return {'href':tools.make_form_link(self.server_nb, self.name, True, ['assr_download'], 'div_message'),
                'title':'Programmer le téléchargement',
                'icone':'/image/ok.gif',
                'libelle':"Télécharger"}

    def _get_cancel_btn(self):
        """ renvoie les données pour la création d'un bouton d'annulation du téléchargement """
        title = "Annuler le téléchargement d'Epreuve@ssr"
        href = tools.make_js_link(self.server_nb, self.name, balise='div_message', cancel="ok")
        libelle = "Annuler"
        icone = "/image/suppr.gif"
        return dict(href=href, title=title, icone=icone, libelle=libelle)
