# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# Action scribe_grouped_edition
# Edition d'utilisateur par groupe
#
###########################################################################
from twisted.python import log
from ead2.backend.lib.action import * # import des actions et des types
from ead2.backend.actions import tools
from ead2.lib.error import MissingKey, MissingValue
from scribe import backend
from ead2.backend.actions.scribe import scribe_tools
from ead2.backend.config.filenames import pwd_dir
from os.path import isfile, join
from os import unlink
import traceback
rapport_dir = pwd_dir['rapport']

used_templates = ['main', 'form', 'formselect', 'accordion', 'listlist', 'checklist']

def write_pwd(login, pwd, rapport_file):
    """ ecrit un rapport à la modification groupée de mot de passe """
    try:
        rapport_file.write("%s;%s\n"%(login, pwd))
    except:
        return False
    return True

def modify_password(login, rapport_file, passtype='name', passvalue=None):
    """ modifie le mot de passe d'un utilisateur
    """
    if passtype == 'name':
        prenom = backend.get_attr(login, 'givenName')[0]
        nom = backend.get_attr(login, 'sn')[0]
        pwd = '%s.%s'%(nom, prenom)
        pwd = pwd.lower()
        if not tools.test_pwd(pwd):
            write_pwd(login, "error", rapport_file)
            return False, "Modification de mot de passe impossible pour %s.\\n"%login
        backend.mod_password(login, pwd)
        write_pwd(login, pwd, rapport_file)
        return True, "Le mot de passe de %s est désormais '%s'.\\n"%(login, pwd)
    elif passtype == 'random':
        pwd = tools.command_output('/usr/bin/pwgen -1').strip()
        backend.mod_password(login, pwd)
        write_pwd(login, pwd, rapport_file)
        return True, "Le mot de passe de %s est désormais '%s'.\\n"%(login, pwd)
    elif passtype == 'date':
        date = backend.get_attr(login, 'dateNaissance')
        if date == []:
            write_pwd(login, "error", rapport_file)
            return False, "Modification de mot de passe impossible pour %s,\\n sa date de naissance n'est pas renseignée.\\n"%login
        pwd = backend.formate_date(date[0])
        pwd = pwd.replace('/', '')
        backend.mod_password(login, pwd)
        write_pwd(login, pwd, rapport_file)
        return True, "Le mot de passe de %s est désormais '%s'.\\n"%(login, pwd)
    elif passtype == 'main':
        pwd = passvalue
        if not tools.test_pwd(pwd) or not pwd:
            raise Exception, "Le mot de passe est mal formé"
        backend.mod_password(login, pwd)
        write_pwd(login, pwd, rapport_file)
        return True, "Le mot de passe de %s est désormais '%s'.\\n"%(login, pwd)
    return False, "Erreur dans la modfication de mot de passe, type inconnu"

class GroupedEdition(Action):
    """ permet l'édition groupé d'utilisateur """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'scribe_grouped_edition'
    libelle = "Edition groupée"
    category = "Gestion/"
    description = "édition groupée d'utilisateurs"
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend",
                                keys=['server', 'action', 'btns', 'form', 'btn', 'purge'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['search', 'userlist',
                                                                'inscription_edit', 'quota_edit',
                                                                'domain_edit', 'profil_edit',
                                                                'password_edit'])

    def execute(self):
        """ renvoit les données pour l'affichage
            1 - récupère le role de l'utilisateur et son login pour le cas d'un prof_admin
            2 - renvoie la description du formulaire de listing d'utilisateur
            3 - renvoie la liste des utilisateurs
            4 - renvoie les boutons d'action
            5 - renvoie les formulaires d'action
            6 - valide les actions effectuées
            7 - purge les fichiers de mot de passe
        """

        params, self.server_nb = tools.get_request(self.request)
        ## 1 -
        self.user = self.user_description['name']
        self.roles = self.user_description['role']

        result = {'titre':self.description}
        ## 5 -
        if params.has_key('form'):
            datas = self._get_action_form(params['form'][0])
            return self.frag(datas)
        ## 4 -
        if params.has_key('btn'):
            return self.frag(self._get_action_btns(int(params['btn'][0])))
        if params.has_key('btns'):
            return self.frag(self._get_action_btns())
        if params.has_key('purge'):
            try:
                datas = self.purge_pwd_files()
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name,
                                            balise='div_password_container',confirm=True, btn='4')
            except Exception, mess:
                log.err("Erreur dans grouped_edition.py: %s"%mess)
                datas = dict(message=mess.message)
            return self.frag(datas)
        ## 6 - 3 -
        if self.form_result != {}:# validation de formulaire
            try:
                filtered_datas = self._valid_form()
            except (MissingKey, MissingValue, Exception), mess:
                log.err("Erreur dans grouped_edition.py: %s"%mess)
                traceback.print_exc()
                filtered_datas = {'message':mess.message}
            filtered_datas.update({'toexec':tools.make_js_link(self.server_nb, self.name,
                                                    balise='action_div_container',confirm=True, btns='True')})
            self.form_result = {}
            return self.frag(filtered_datas)
        ## 2 -
        if scribe_tools.is_prof_admin(self.roles):
            result.update(scribe_tools._get_user_listing_form(prof_admin=self.user))
        else:
            result.update(scribe_tools._get_user_listing_form())
        result.update(dict(validate=self._get_valid_btn()))
        return self.send_all(result, template='scribe_edition', templates=used_templates)

    def frag(self, filtered_datas):
        """ renvoit les datas pour l'affichage de la liste des utilisateurs correpondant à la recherche"""
        return self.send_frag(filtered_datas, template='scribe_edition', templates=used_templates)

    def _valid_form(self):
        """ valide les retours de formulaire """
        if self.form_result.has_key('search'):
            return self._valid_user_listing()
        elif self.form_result.has_key('inscription_edit'):
            return self._valid_inscription()
        elif self.form_result.has_key('quota_edit'):
            return self._valid_quota()
        elif self.form_result.has_key('domain_edit'):
            return self._valid_domain()
        elif self.form_result.has_key('profil_edit'):
            return self._valid_profil()
        elif self.form_result.has_key('password_edit'):
            return self._valid_password()

    def _valid_inscription(self):
        """ valide les inscriptions à des groupes """
        if not self.form_result.has_key('userlist'):
            raise MissingKey, "Erreur : il manque des données pour l'inscription."
        userlist = self._get_user_list()

        resultat = tools.format_form_result(self.form_result['inscription_edit'])
        if not resultat.has_key('group_name'):
            raise MissingKey, "Erreur : il manque des données pour l'inscription."
        group = resultat['group_name']
        for user in userlist:
            if not backend.inscription(user, group):
                raise MissingKey, "Erreur : une erreur s'est produite lors de l'inscription de %s"%user
        return dict(message="Les utilisateurs ont bien été inscrits au groupe")

    def _valid_quota(self):
        """ valide l'attribution de quota à une liste d'utilisateur """
        if not self.form_result.has_key('userlist'):
            raise MissingKey, "Erreur : il manque des données pour l'édition de quota."
        userlist = self._get_user_list()
        resultat = tools.format_form_result(self.form_result['quota_edit'])
        if not resultat.has_key('quota'):
            raise MissingKey, "Erreur : il manque des données pour l'édition de quota."
        quota = resultat['quota']
        for user in userlist:
            if not backend.set_quota(user, quota):
                raise MissingKey, "Erreur : une erreur s'est produite lors de \\n l'attribution de quota à %s"%user
        return dict(message="%s Mo de quota disque ont été attribué aux utilisateurs sélectionnés"%quota)

    def _valid_domain(self):
        """ valide le changement de domaine mail d'une liste d'utilisateur """
        if not self.form_result.has_key('userlist'):
            raise MissingKey, "Erreur : il manque des données pour le changement de domaine mail."
        userlist = self._get_user_list()
        resultat = tools.format_form_result(self.form_result['domain_edit'])
        if not resultat.has_key('domain'):
            raise MissingKey, "Erreur : il manque des données pour le changement de domaine mail."
        domain = resultat['domain']
        for user in userlist:
            if not backend.mod_mail(user, 'local', "%s@%s"%(user,domain)):
                raise MissingKey, "Erreur : erreur lors du changement de domaine mail de %s."%user
        return dict(message="Les utilisateurs sélectionnés ont désormais un mail de type %s."% domain)

    def _valid_profil(self):
        """ valide le changement de profil d'une liste d'utilisateur """
        if not self.form_result.has_key('userlist'):
            raise MissingKey, "Erreur : il manque des données pour le changement de profil."
        userlist = self._get_user_list()
        resultat = tools.format_form_result(self.form_result['profil_edit'])
        if not resultat.has_key('_sambaProfilePath'):
            raise MissingKey, "Erreur : il manque des données pour le changement de profil."
        profil = resultat['_sambaProfilePath']
        profil = int(profil)
        for user in userlist:
            if not backend.set_profil(user, profil):
                raise MissingKey, "Erreur : erreur lors du changement de profil de l'utilisateur %s"%user
        return {'message':"Le profil %s a bien été attribué aux utilisateurs sélectionnés."%profil}

    def _valid_password(self):
        """ valide le changement de mot de passe pour une liste d'utilisateur """
        if not self.form_result.has_key('userlist'):
            raise MissingKey, "Erreur : il manque des données pour le changement de mot de passe."
        userlist = self._get_user_list()
        resultat = tools.format_form_result(self.form_result['password_edit'])
        if not resultat.has_key('password'):
            raise Exception, "Erreur: il manque des données pour le changement de mot de passe."
        rootname = self.user_description['name']
        for increment in range(15):
            name = rootname + str(increment) + '.txt'
            filename = join(rapport_dir, name)
            if not isfile(filename):
                break
        if isfile(filename):
            raise Exception, "Veuillez purger vos fichiers de mot de passe avant de continuer."
        message = "Modification des mots de passe:\\n"
        message += "La liste des nouveaux mot de passe se trouve dans: \\n '%s'\\n\\n"%filename
        openfile = file(filename, 'w')
        for user in userlist:
            ret_code, ret_msg = modify_password(user, openfile, resultat['password'], resultat.get('input_password', None))
            if not ret_code:
                message += ret_msg
            elif len(userlist) < 15:
                message += ret_msg
        openfile.close()
        return dict(message = message)

    def purge_pwd_files(self):
        """ purge les fichiers de mot de passe """
        message = ""
        rootname = self.user_description['name']
        name = rootname
        for increment in range(15):
            name = rootname + str(increment) + '.txt'
            filename = join(rapport_dir, name)
            if isfile(filename):
                unlink(filename)
                message += "Suppression de %s\\n"%name
        return dict(message=message)

    def _get_user_list(self):
        """ renvoie la liste des utilisateurs sélectionnés (checkbox)
            pour les actions(teste les droits de l'appelant)
        """
        userlist = tools.format_form_result(self.form_result['userlist'], False).keys()
        if userlist == []:
            raise Exception, "Aucun utilisateur n'a été sélectionné."
        # contrôle des droits d'édition, si prof_admin on filtre sinon on est admin
        if scribe_tools.is_prof_admin(self.roles):
            for user in userlist:
                if not scribe_tools.is_eleve_from_prof(user, self.user) and user != self.user:
                    log.msg("Erreur : tentative abusives d'édition de %s par %s."%(user, self.user))
                    raise Exception, "Erreur : action non autorisée."
        return userlist

    def _valid_user_listing(self):
        """ renvoie la description de la liste des utilisateurs (libelle et checkbox)
            d'après les critéres de recherche
        """
        request = tools.format_form_result(self.form_result['search'])
        keys = request.keys()
        if 'alpha_key' not in keys or 'domains' not in keys or 'user_class' not in keys or 'user_type' not in keys or 'namepart' not in keys or 'user_group' not in keys:
            return ""
        alpha_key, _type, domain, _class, namepart, group = request['alpha_key'], request['user_type'], request['domains'], request['user_class'], request['namepart'], request['user_group']
        userlist = []
        for user in scribe_tools.user_filter(_type, domain, _class, namepart, alpha_key, group):
            if scribe_tools.is_prof_admin(self.roles):
                if not scribe_tools.is_eleve_from_prof(user, self.user):
                    continue
            if user in backend.get_profs():_type = 'prof'
            else:_type='élève'
            userlist.append({'name':user, 'libelle':"%s (%s)"%(user, _type), 'value':user, 'checked':'true', 'inline':'ok'})
        return dict(userlist=userlist,
                    nb_users="Nombre d'utilisateurs : %s"%len(userlist),
                    all_users_btn=self._get_all_users_btn(),
                    none_users_btn=self._get_none_users_btn())

    def _get_action_form(self, _type):
        """ renvoie la description du formulaire d'édition groupée """
        if _type == 'inscription':
            return self._get_inscription_form()
        elif _type == 'quota':
            return self._get_quota_form()
        elif _type == 'domain':
            return self._get_domain_form()
        elif _type == 'profil':
            return self._get_profil_form()
        elif _type == 'password':
            return self._get_password_form()

    def _get_inscription_form(self):
        """ renvoie la description du formulaire d'inscription à des groupes """
        group_icone = "/image/scribe/action_groupe_over.png"
        group_names = backend.list_groups('Groupe')
        group_names.sort()
        groups = [{'name':gr, 'libelle':gr}for gr in group_names]
        subscribe_to_group = {'name':'group_name', 'options':[{'name':'', 'options':groups}]}

        toexec = self._get_toexec_links(0)

        texte = "Inscrivez les utilisateurs sélectionnés au groupe:"
        validate = {'href':tools.make_form_link(self.server_nb, self.name, True, ['inscription_edit', 'userlist'],
                            balise='div_inscription_container'),
                    'libelle':"Valider",
                    'title':"Inscrire à ce groupe",
                    'icone':"/image/ok.gif"}
        return dict(texte=texte,
                    group_icone=group_icone,
                    subscribe_to_group=subscribe_to_group,
                    toexec=toexec, validate=validate)

    def _get_quota_form(self):
        """ renvoie la description du formulaire de modification de quota """
        quota_icone = "/image/scribe/action_quota_over.png"
        quota_input = {'name':'quota', 'libelle':'Quota disque (en Mo)', 'onblur':"testNumber('quota', 'error_quota')"}

        toexec = self._get_toexec_links(1)

        texte = "Attribuez un quota disque aux utilisateurs sélectionnés"
        validate = {'href':tools.make_form_link(self.server_nb, self.name, True, ['quota_edit', 'userlist'],
                            balise='div_quota_container'),
                    'libelle':"Valider",
                    'title':"Attribuer les quotas",
                    'icone':"/image/ok.gif"}
        return dict(texte=texte,
                    quota_icone=quota_icone,
                    quota_input=quota_input,
                    toexec=toexec,
                    validate=validate)

    def _get_domain_form(self):
        """ renvoie la description du formulaire de modification de domaine mail """
        domain_icone = "/image/scribe/action_liste_over.png"
        domain_select = {'name':'domain', 'libelle':"Domaine mail",
                        'options':[{'name':'', 'options':[{'name':backend.get_domains()[0], 'libelle':"domaine restreint"},
                                                        {'name':backend.get_domains()[1], 'libelle':"domaine internet"}]}]
                        }

        toexec = self._get_toexec_links(2)

        texte = "Changez le domaine mail des utilisateurs sélectionnés"
        validate = {'href':tools.make_form_link(self.server_nb, self.name, True, ['domain_edit', 'userlist'],
                            balise='div_domain_container'),
                    'libelle':"Valider",
                    'title':"Changer le domaine mail",
                    'icone':"/image/ok.gif"}
        return dict(texte=texte,
                    domain_icone=domain_icone,
                    domain_select=domain_select,
                    toexec=toexec,
                    validate=validate)

    def _get_profil_form(self):
        """ renvoie la description du formulaire d'édition de profil """
        profil_icone = "/image/scribe/action_profil_over.png"
        profil_opts = [{'name':'1', 'libelle':'local'},
                        {'name':'2', 'libelle':"obligatoire - profil"},
                        {'name':'3', 'libelle':"obligatoire - profil2"}]
        profil_select = {'name':'_sambaProfilePath', 'libelle':"Profil utilisateur",
                         'options':[{'name':"Profil disponible", 'options':profil_opts}]}

        toexec = self._get_toexec_links(3)

        texte = "Changer le profil des utilisateurs sélectionnés"
        validate = {'href':tools.make_form_link(self.server_nb, self.name, True, ['profil_edit', 'userlist'],
                            balise='div_profil_container'),
                    'libelle':"Valider",
                    'title':"Changer le profil",
                    'icone':"/image/ok.gif"}
        return dict(texte=texte,
                    profil_icone=profil_icone,
                    profil_select=profil_select,
                    toexec=toexec,
                    validate=validate)

    def _get_password_form(self):
        """ renvoie la description du formulaire d'édition de mot de passe """
        purge_btn = {'href': tools.make_js_link(self.server_nb, self.name, purge=True,
                                balise='div_password_container'),
                     'title':'Purger les fichiers de mot de passe',
                     'libelle':'Purger les fichiers de mot de passe',
                     'icone':'/image/suppr.gif',
                     '_class':'btn'}
        pwd_icone = '/image/scribe/action_password.png'
        texte = 'Génération groupée de mot de passe'
        radio_date = {'name':'password',
                      'value':'date',
                      'libelle':'Depuis la date de naissance (que pour les élèves)',
                      'inline':'ok'}
        radio_random = {'name':'password',
                        'value':'random',
                        'libelle':'Aléatoirement',
                        'inline':'ok'}
        radio_name = {'name':'password',
                        'value':'name',
                        'libelle':"De la forme 'nom.prenom'",
                        'inline':'ok',
                        'checked':'ok'}
        radio_main = {'name':'password',
                        'value':'main',
                        'libelle':"Le même mot de passe pour tous",
                        'inline':'ok'}
        input_password = {'name':'input_password', 'libelle':"-->", 'inline':'ok'}

        toexec = self._get_toexec_links(4)

        validate = {'href':tools.make_form_link(self.server_nb, self.name, True, ['password_edit', 'userlist'],
                            balise='div_password_container'),
                    'libelle':"Valider",
                    'title':"Générer de nouveau mot de passe",
                    'icone':"/image/ok.gif"}
        return dict(texte=texte,
                    pwd_icone=pwd_icone,
                    radio_date=radio_date,
                    radio_random=radio_random,
                    radio_name=radio_name,
                    radio_main=radio_main,
                    input_password=input_password,
                    toexec=toexec,
                    validate=validate,
                    purge_btn=purge_btn)


    def _get_toexec_links(self, actual_number):
        """ renvoie les liens javascript à éxécuter """
        if scribe_tools.is_admin(self.user_description['role']):
            action_list = ['inscription', 'quota', 'domain', 'profil', 'password', 'role']
        else:
            action_list = ['inscription', 'quota', 'domain', 'profil', 'password',]
        toexec = ''
        for indice, value in enumerate(action_list):
            if indice != actual_number:
                toexec += tools.make_js_link(self.server_nb, self.name,
                                         balise='div_%s_container'%value,
                                         confirm=True,
                                         btn='%d'%indice)
        return toexec

    def _get_action_btns(self, num=''):
        """ renvoie la description des boutons pour les actions possibles
            :num: numéro du bouton demandé (on les renvoie pas tous à chaque fois)
        """
        # l'attribut name permet de créer la div où s'effectuera l'action
        btn1 = {'href': tools.make_js_link(self.server_nb, self.name,
                                            balise='div_inscription_container',
                                            form='inscription'),
                    'libelle':"Inscrire ces utilisateurs à d'autres groupes",
                    'title': '',
                    'icone': "/image/scribe/action_groupe.png",
                    '_class':'scribe_action_btn',
                    'name':"div_inscription_container"}

        btn2 = {'href': tools.make_js_link(self.server_nb, self.name,
                                            balise='div_quota_container',
                                            form='quota'),
                    'libelle':"Définir des quotas disques pour ces utilisateurs",
                    'title': '',
                    'icone': "/image/scribe/action_quota.png",
                    '_class':'scribe_action_btn',
                    'name':"div_quota_container"}

        btn3 = {'href': tools.make_js_link(self.server_nb, self.name,
                                            balise='div_domain_container',
                                            form='domain'),
                    'libelle':"Changer le domaine mail pour ces utilisateurs",
                    'title': '',
                    'icone': "/image/scribe/action_liste.png",
                    '_class':'scribe_action_btn',
                    'name':"div_domain_container"}

        btn4 = {'href': tools.make_js_link(self.server_nb, self.name,
                                            balise='div_profil_container',
                                            form='profil'),
                    'libelle':"Changer le profil pour ces utilisateurs",
                    'title': '',
                    'icone': "/image/scribe/action_profil.png",
                    '_class':'scribe_action_btn',
                    'name':"div_profil_container"}

        btn5 = {'href': tools.make_js_link(self.server_nb, self.name,
                                            balise='div_password_container',
                                            form='password'),
                    'libelle':"Générer un nouveau mot de passe pour ces utilisateurs.",
                    'title': '',
                    'icone': "/image/scribe/action_password.png",
                    '_class':'scribe_action_btn',
                    'name':"div_password_container"}

        btn6 = {'href': tools.make_js_link(self.server_nb, 'role_manager',
                                            balise='div_role_container',
                                            form='userlist'),
                    'libelle':"Associer un rôle à ces utilisateurs",
                    'title': '',
                    'icone': "/image/action_role.png",
                    "_class":'scribe_action_btn',
                    'name':"div_role_container"}
        if scribe_tools.is_admin(self.user_description['role']):
            action_btns = [btn1, btn2, btn3, btn4, btn5, btn6]
        else:
            action_btns = [btn1, btn2, btn3, btn4, btn5]
        if num !='' :return dict(action_btn=action_btns[num])
        return dict(action_btns = action_btns)

    def _get_valid_btn(self):
        """ renvoie la description du bouton de validation de la recherche
            la cible du retour est la balise : user_div_container
        """
        icone = '/image/ok.gif'
        title = 'Lister selon les filtres'
        libelle = 'Lister'
        href = tools.make_form_link(self.server_nb, self.name, True, ['search'], 'user_td_container')
        return dict(icone=icone, title=title, libelle=libelle, href=href)

    def _get_none_users_btn(self):
        """ renvoie la description du bouton de déselection des utilisateurs de la liste """
        icone = ''
        title = 'Aucun utilisateur'
        libelle = 'Aucun'
        href = "javascript:formSelectNone('userlist')"
        return dict(icone=icone, title=title, libelle=libelle,
                href=href)

    def _get_all_users_btn(self):
        """ renvoie la description du bouton de selection de
            tous les utilisateurs de la liste
        """
        icone = ''
        title = 'Tous les utilisateurs'
        libelle = 'Tous'
        href = "javascript:formSelectAll('userlist')"
        return dict(icone=icone, title=title, libelle=libelle,
                href=href)
