# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# Action de gestion des comptes mails
#
#
###########################################################################
from twisted.python import log
from ead2.backend.lib.action import * # import des actions  et types
from ead2.backend.actions import tools
from scribe import backend
import traceback
used_templates=['main', 'form', 'formselect']

def get_close_btn():
    """ renvoie la description du bouton de fermeture """
    href = "javascript:setVoid('mail_workspace');"
    title = "Fermer la section"
    libelle = "Fermer"
    icone = "/image/supprimer.gif"
    _class = "scribe_close_btn"
    return dict(href=href,
                title=title,
                libelle=libelle,
                icone=icone,
                _class=_class)

def get_del_btn(server_nb, name, mail):
    """ renvoie la description du bouton de suppression de compte mail """
    href = tools.make_js_link(server_nb, name, balise='mail_workspace', todel=mail)
    href = tools.make_confirm_link("Êtes-vous sûr de vouloir supprimer ce compte mail ?", href)
    title = "Supprimer ce compte mail"
    libelle = "Supprimer"
    icone = ""
    _class = "simple_link"
    return dict(href=href,
                title=title,
                libelle=libelle,
                icone=icone,
                _class=_class)

def get_mod_pwd_btn(server_nb, name, mail):
    """ renvoie la description du bouton de modification de mot de passe """
    href = tools.make_js_link(server_nb, name, balise='mail_workspace', modpwd=mail )
    title = "Modifier le mot de passe pour ce compte mail."
    libelle = "Changer le mot de passe"
    icone = ""
    _class = "simple_link"
    return dict(href=href,
                title=title,
                libelle=libelle,
                icone=icone,
                _class=_class)

def get_validate(server_nb, name, forms, balise='content'):
    """
        renvoie la description d'un bouton de validation de formulaire
    """
    href= tools.make_form_link(server_nb, name, True, forms, balise)
    title = "Valider"
    libelle = "Valider"
    icone = "/image/ok.gif"
    return dict(href=href,
                title=title,
                libelle=libelle,
                icone=icone
                )

def get_mod_pwd_form(server_nb, name, account):
    """
        renvoie la description du formulaire de modification de mot de passe
    """
    account_input = {'name':'moduid', 'default_value':account}
    password = {'name':'modpassword',
                'libelle':'Nouveau mot de passe',
                'inline':'ok',
                "onblur":"testVoid('modpassword', 'error_modpassword');"}
    confirm = {'name':'confirm',
               'libelle':'Confirmer mot de passe',
               'inline':'ok',
               "onblur":"testVoid('confirm', 'error_confirm');"}
    validate = get_validate(server_nb, name, ['modmail'], "mail_workspace")
    return dict(account_input=account_input,
                password=password,
                confirm=confirm,
                validate=validate,
                close_btn=get_close_btn()
                )

def valid_create(resultat):
    """ valide la création d'un compte mail """
    # on traite le resultat (execution d'une action ...)
    uid = resultat['uid']
    password = resultat['password']
    domain = resultat['domain']

    if not tools.test_mail("%s@%s" % (uid, domain)):
        raise Exception , "Adresse mail mal formée."

    if not tools.test_pwd(password):
        raise Exception , "Mot de passe mal formé."

    a,b = backend.ajout_mail(uid, password, domain)
    if not a:
        raise Exception , b
    # on pourrait renvoyer des données plutôt qu'un message
    return dict(message="Le compte mail %s@%s a été créé." % (uid, domain))

def valid_mod(resultat):
    """ valide la modification d'un compte mail """
    # on traite le resultat (execution d'une action ...)
    uid = resultat['moduid']
    password = resultat['modpassword']
    confirm = resultat['confirm']
    if confirm != password:
        raise Exception, "Les mots de passe sont différents."
    if not tools.test_pwd(password):
        raise Exception , "Mot de passe mal formé."

    a,b = backend.mod_mail_password(uid, password)
    if not a:
        raise Exception , b
    # on pourrait renvoyer des données plutôt qu'un message
    return dict(message="Le mot de passe du compte mail %s a bien été modifié." % (uid))


class MailCreate(Action):
    """ Création de compte mail """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = "scribe_mail_create" #celui de perm_nomdumodule.ini
    libelle = "Créer"
    category = "Gestion/Mail"
    description = "Créer un compte mail"

    request = Dict(default={}, doc="arguments de la requete en cours cote frontend",
                        keys=['server', 'action', 'modpwd', 'todel'])#server et action sont les clés systématiques

    form_result = Dict(default={}, doc='retour de formulaire', keys=["mail", "modmail"])

    def execute(self):
        params, self.server_nb = tools.get_request(self.request)
        # Dans le cas d'un formulaire:
        if self.form_result.has_key('mail'):
            try:
                # formatage de données pour traitement
                resultat = tools.format_form_result(self.form_result['mail'])
                # datas: dico utilisé pour la templatisation
                datas = valid_create(resultat)
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name,
                                                     confirm=False)
            except Exception, message:
                traceback.print_exc()
                #si il y a une erreur soulevée par nos soins, on renvoie le message
                datas = dict(message=message.message)
            # Important: on vide le formulaire
            self.form_result = {}
            return self.send_frag(datas, template='scribe_mail',
                                  templates=used_templates)

        elif self.form_result.has_key('modmail'):
            try:
                resultat = tools.format_form_result(self.form_result['modmail'])
                datas = valid_mod(resultat)
            except Exception, message:
                traceback.print_exc()
                datas = dict(message=message.message)
            # Important: on vide le formulaire
            self.form_result = {}
            return self.send_frag(datas, template='scribe_mail',
                                  templates=used_templates)

        if params.has_key('modpwd'):
            datas = get_mod_pwd_form(self.server_nb, self.name, params['modpwd'][0])
            return self.send_frag(datas, template='scribe_mail',
                                  templates=used_templates)

        if params.has_key('todel'):
            result = backend.del_mail(params['todel'][0])
            if result:
                msg = "Le compte mail %s a été supprimé." % params['todel'][0]
            else:
                msg = "Erreur à la suppression du compte %s" % params['todel'][0]
            datas = dict(message=msg)
            datas['toexec'] = tools.make_js_link(self.server_nb, self.name,
                                                 confirm=False)
            return self.send_frag(datas, template='scribe_mail',
                                  templates=used_templates)

        result = {'titre':"Création de compte mail"}
        result.update(self._get_form())
        result.update(self._get_mails())
        return self.send_all(result, template='scribe_mail',
                            templates=used_templates)

    def _get_form(self):
        """ renvoie la description du formulaire de création de compte mail """
        uid = {'name':'uid',
               'libelle':'Login',
               'inline':'ok',
               'onblur':"testVoid('uid', 'error_uid');"}
        password = {'name':"password",
                    "libelle":"mot de passe",
                    'inline':'ok',
                    "onblur":"testVoid('password', 'error_password');"}
        domains_opts = [{'name':dom, 'libelle':dom}for dom in backend.get_domains()]
        domain = {'name':'domain',
                   'libelle':'',
                   'options':[{'name':'domaine', 'options':domains_opts}],
                   'inline':'ok'
                   }
        return dict(uid=uid,
                    password=password,
                    domain=domain,
                    validate=self._get_validate_btn()
                    )

    def _get_mails(self):
        """ renvoie la liste des comptes mails """
        table_libelle = "Comptes mail existants"
        mails = []
        _mails = backend.get_mails()
        for mail in _mails:
            del_btn = get_del_btn(self.server_nb, self.name, mail[0])
            mod_pwd_btn = get_mod_pwd_btn(self.server_nb, self.name, mail[0])
            mails.append((mail[1], mod_pwd_btn, del_btn))
        def compare(a,b):
            if a[0]>b[0]:
                return 1
            elif a[0] == b[0]:
                return 0
            return -1
        mails.sort(compare)
        return dict(list_mail=mails,
                    table_libelle=table_libelle,
                    )

    def _get_validate_btn(self):
        """ renvoie la description du bouton valider """
        href = tools.make_form_link(self.server_nb, self.name,
                                    True, ['mail'], "mail_div_msg")
        icone = "/image/ok.gif"
        libelle = "Valider"
        title = "Créer ce compte mail"
        return dict(href=href,
                    icone=icone,
                    libelle=libelle,
                    title=title)

