# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# Action spécifique aux profs
# gestion des préférences
#
# modification du mail, section utilisateur, création de groupe
#       
###########################################################################
from twisted.python import log
from ead2.backend.lib.action import * # import des actions et des types
from ead2.backend.actions import tools # import get_request, make_js_link, make_form_link, format_form_result
from ead2.backend.actions.scribe import scribe_tools
from ead2.lib.error import MissingKey, MissingValue
from scribe import backend
used_templates = ['main', 'formselect', 'form', 'listlist', 'accordion']

def get_menu(server_nb, user=''):
    """ renvoie la description du menu
        :page: page en cours d'affichage
    """
    password = {'title':'Modifier votre mot de passe', 'icone':'/image/scribe/action_pwd.png', 
            'href':tools.make_js_link(server_nb, 'scribe_user_password', balise='mod_user_div_container', user=user),
            'libelle':"Mot de passe", "_class":"scribe_action_btn"}
    mail = {'title':'Modifier votre adresse mail', 'icone':'/image/scribe/action_liste.png',
                    'href':tools.make_js_link(server_nb, 'scribe_prof_mod_mail', 'mod_user_div_container'), 
                    'libelle':'Mail', "_class":"scribe_action_btn"}
    groupe = {'title':"Inscrivez-vous dans des groupes", 'icone':'/image/scribe/action_groupe.png',
                    'href': tools.make_js_link(server_nb, 'scribe_prof_mod_groupe', 'mod_user_div_container'),
                    'libelle':'Groupe', '_class':"scribe_action_btn"}
    return dict(menus=(mail, password, groupe))

class ProfPreference(Action):
    """ 
        page d'entrée pour l'édition des préférences d'un prof:
        - modification de son mail
        - modification du mot de passe
        - inscription au groupe
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'scribe_prof_preference'
    libelle = "Préférences"
    category = ""
    description = "Modifier vos préférences"
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend", 
                                keys=['server', 'action'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=[])

    def execute(self):
        """ renvoie les données pour l'affichage (le menu) """
        self.params, self.server_nb = tools.get_request(self.request)
        result = {'titre':self.description}
        result.update(get_menu(self.server_nb, self.user_description['name']))
        return self.send_all(result, template='scribe_prof_preference', templates=used_templates)

class ProfModMail(Action):
    """ Modifie le mail d'un prof """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'scribe_prof_mod_mail'
    libelle = "Mail"
    category = None
    description = "Modification du mail d'un utilisateur"
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend", keys=['server', 'action'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['mod_mail'])

    def execute(self):
        """ renvoie les données pour l'affichage """
        params, self.server_nb = tools.get_request(self.request)
        self.user = self.user_description['name']
        result = {}
        if self.form_result != {}:
            try:
                result.update(self._valid_form())
            except (MissingValue, MissingKey, Exception), mess:
                log.err("Erreur dans prof.py : %s"%mess)
                result['message'] = mess.message
            self.form_result = {}
        result.update(self._get_form())
        return self.send_all(result, template='scribe_prof_preference', templates=used_templates)

    def _get_form(self, _type=None):
        """ renvoie la description du formulaire d'édition de mot de passe """
        user_name = {'name':'user_name', 'default_value':self.user}
        datas = backend.get_user_data(self.user)
        #edition des mails
        _mail_locale = "%s@%s"%(self.user, backend.get_domains()[1])
        mail = {'libelle':"Adresse e-mail (personalisée <b>ou</b> locale)",
                'radios':[{'name':'mailtype', 'value':'perso','inline':'ok'},
                    {'name':'mailtype', 'value':'local','inline':'ok'}],
                    'radios_libelle':['Adresse personalisée :', 'Adresse locale: <br /> %s'%_mail_locale],
                    'input':{'name':"mailperso", "libelle":''}
                    }
        if datas.has_key('mailDir'):
            if datas['mailDir'][0] != '':
                mail['radios'][1]['checked'] = 'true'
                if datas.has_key('mail'): _mail = datas['mail'][0]
                else: _mail = _mail_locale
            else:
                _mail = "mail non communiqué"
        elif datas.has_key('mail'):
                if datas['mail'][0] != '':
                    mail['radios'][0]['checked'] = 'true'
                    _mail = datas['mail'][0]
                    mail['input']['default_value'] = _mail
        else: # il n'a pas encore de mail
            mail['radios'][0]['checked'] = 'true'
            _mail = "mail non communiqué"
        mail['libelle'] = "Adresse e-mail (personalisée <b>ou</b> locale): <br /> <b>%s</b>"%_mail
        return dict(user_name=user_name, mail=mail, validate=self._get_validate_btn(), 
                        icone='/image/scribe/action_liste_over.png')
    
    def _get_validate_btn(self):
        """ renvoie la description du bouton valider """
        href = tools.make_form_link(self.server_nb, self.name, True, ['mod_mail'], balise='mod_user_div_container')
        title = "Modifier le mail"
        libelle = "Valider"
        icone = "/image/ok.gif"
        return dict(href=href, title=title, libelle=libelle, icone=icone)

    def _valid_form(self):
        """ valide la modification du mail du prof """
        if not self.form_result.has_key('mod_mail'):
            raise MissingKey, "Erreur : Il manque des données pour la modification du mail."
        resultat = tools.format_form_result(self.form_result['mod_mail'])
        if not resultat.has_key('mailtype'):
            raise MissingKey, "Erreur : il manque des valeurs pour la modification du mail."
        mailtype = resultat['mailtype']
        if mailtype == 'perso':
            if not resultat.has_key('mailperso'):
                raise MissingKey, "Erreur : il manque des valeurs à la modification du mail."
            mailperso = resultat['mailperso'].strip()
            if mailperso == '': raise MissingKey, "Erreur : il manque des valeurs à la modification du mail."
        else:
            mailperso = ''
        if not backend.mod_mail(self.user, mailtype, mailperso):
            raise Exception , "Erreur : erreur à la modification du mail."
        return dict(message = "Le mail de %s a bien été modifié."%self.user)

class ProfModGroupe(Action):
    """ Inscrit un utilisateur à des groupes
        Action spécifique aux profs ou prof_admin
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'scribe_prof_mod_groupe'
    libelle = "Groupes"
    category = None
    description = "Modification des groupes d'un utilisateur"
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend", keys=['server', 'action'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['groupe_form'])

    def execute(self):
        """ renvoie les données pour l'affichage 
            1 - renvoie le formulaire d'inscription aux groupes
            2 - valide les inscriptions
        """
        params, self.server_nb = tools.get_request(self.request)
        self.user = self.user_description['name']
        result = {'sstitre':"Inscription à un groupe"}
        ## 2 -
        if self.form_result != {}:
            try:
                result.update(self._valid_form())
            except (MissingValue, MissingKey, Exception), mess:
                log.err("Erreur dans prof.py : %s"%mess)
                result['message'] = mess.message
            self.form_result = {}
        ## 1 -
        result.update(self._get_form())
        return self.send_all(result, template='scribe_prof_preference', templates=used_templates)

    def _get_form(self, _type=None):
        """ renvoie la description du formulaire d'édition de mot de passe """
        user_name = {'name':'user_name', 'default_value':self.user}
        user_groupes = [gr for gr in backend.get_user_groups(self.user)if gr in backend.list_groups('groupe')]
        groupes_dispo = [gr for gr in backend.list_groups('groupe') if gr not in user_groupes]
        liste = {'title':"Vos groupes", 'formid':'groupe_form','id':'groupe_table',
                'datas':user_groupes}
        select = {'name':'groupe_select', 'libelle':"Groupes disponibles",
                'options':[{'name':'Groupes disponibles','options':[{'name':gr, 'libelle':gr}for gr in groupes_dispo]}],
                'id':'groupe_select' }
        groupe = {'name':"groupe", 'btns':self._get_transf_btns('groupe'), 'select':select,
                'liste':liste, 'title':""}
        return dict(user_name=user_name, groupe=groupe, validate=self._get_validate_btn(), 
                        icone='/image/scribe/action_groupe_over.png')
    
    def _get_transf_btns(self, racine='admin_class'):
        """ renvoit les btns de transfert de donnees d'une select à une liste 
            :racine:racine des noms html des balises concernées (qu'on retrouve à la validation)
        """
        href2 = "javascript:formRemoveFromList('%s_form');"%racine
        href1 = "javascript:formAddNameToList3('%s_select','%s_table');"%(racine, racine)
        icone2 = "/image/scribe/rightleft_red.png"
        icone1 = "/image/scribe/leftright.png"
        msg2 = "Désinscrire"
        msg1 = "Inscrire"
        btns = [{'href':href2, 'libelle':msg2, 'icone':icone2, 'title':msg2},
                {'href':href1, 'libelle':msg1, 'icone':icone1, 'title':msg1}]
        return btns

    def _get_validate_btn(self):
        href = tools.make_form_link(self.server_nb, self.name, True, ['groupe_form'], balise='mod_user_div_container')
        title = "Modifier les inscriptions aux groupes"
        libelle = "Valider"
        icone = "/image/ok.gif"
        return dict(href=href, title=title, libelle=libelle, icone=icone)

    def _valid_form(self):
        """ valide l'inscription aux groupes """
        if not self.form_result.has_key('groupe_form'):
            raise MissingKey, "Erreur : il manque des données pour la gestion des groupes."
        groupe = tools.format_form_result(self.form_result['groupe_form'], '2')
        groupe = self._filter_groupe(groupe)
        if not self._set_groupe(groupe):
            raise Exception, "Erreur : une erreur est survenue lors de la gestion des groupes."
        message = "La modification des groupes de %s s'est bien passée: \\n"%self.user
        if groupe[0] != []: 
            message += "Il a été inscrit dans les groupes :\\n"
            for g in groupe[0]: message += "    - %s\\n"%g
        if groupe[1] != []:
            message += "Il a été désinscrit des groupes : \\n"
            for g in groupe[1]:
                message += "    - %s\\n"%g
        return dict(message=message)

    def _filter_groupe(self, groupes):
        """ filtre les groupes auxquels l'utilisateur appartient , renvoie [les groupes du user,les autres groupes] """
        user_groupes = [cl for cl in backend.get_user_groups(self.user) if cl in backend.list_groups('groupe')]
        return [cl for cl in groupes.keys() if cl not in user_groupes],[cl for cl in user_groupes if cl not in groupes.keys()]
    
    def _set_groupe(self, groupe):
        """ enregistre un utilisateur dans une liste de groupes 
            :groupe: liste de groupes
        """ 
        for gr in groupe[0]:
            if not backend.inscription(self.user, gr):
                return False
        for gr in groupe[1]:
            if not backend.desinscription(self.user, gr):
                return False
        return True

class ProfUser(Action):
    """ action d'entrée pour la gestion des élèves 
        renvoie vers ProfUserCreate et ProfModUser
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'scribe_prof_user'
    libelle = "Mes Élèves"
    category = "Gestion"
    description = "Gestion des élèves"
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend", keys=['server', 'action'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=[])

    def execute(self):
        params, self.server_nb = tools.get_request(self.request)
        result = {'titre':"Gestion des élèves"}
        result.update(self._get_menu())
        return self.send_all(result, template='scribe_prof_user', templates=used_templates)

    def _get_menu(self):
        """ renvoie la description du menu """
        create = {'title':"Création d'utilisateur", "icone":"/image/scribe/action_user_create.png",
                'libelle':"Créer élève", '_class':'scribe_action_btn', 
                  "href":tools.make_js_link(self.server_nb, 'scribe_prof_user_create', 'main_div_container')}
        modify = {'title':"Modification d'utilisateur", "icone":"/image/scribe/action_user_modify.png",
                'libelle':"Modifier élève", '_class':'scribe_action_btn',
                "href":tools.make_js_link(self.server_nb, 'scribe_prof_user_modify', 'main_div_container')}
        return dict(menus=(create,modify))

class ProfCreateUser(Action):
    """ action de création d'élève pour les prof administrateur de classe (prof_admin)
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'scribe_prof_user_create'
    libelle = "Utilisateurs"
    category = None
    description = "Création d'un élève"
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend", keys=['server', 'action'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['add_user'])

    def execute(self):
        """ renvoie les données pour l'affichage
            1 - renvoie du formulaire de création d'élève
            2 - création de l'élève avec contrôle des droits du prof
        """
        params, self.server_nb = tools.get_request(self.request)
        self.user = self.user_description['name']

        self.user_type = scribe_tools.get_role(self.user_description['role'])

        result = {'sstitre':"Création d'un utilisateur"}
        ## 2 -
        if self.form_result != {}:
            try:
                result.update(self._valid_form())
                result['toexec'] = tools.make_js_link(self.server_nb, self.name, confirm=True, balise='main_div_container')
            except (MissingValue, MissingKey, Exception), mess:
                log.err("Erreur dans prof.py : %s"%mess)
                result['message'] = mess.message
            self.form_result = {}
            return self.send_frag(result, template='scribe_prof_user', templates=used_templates)
        ## 1 -
        result.update(self._get_form())
        result.update(self._get_validate_btn())
        result.update(self._get_help())
        return self.send_all(result, template='scribe_prof_user', templates=used_templates)

    def _get_form(self):
        """ renvoie la description du formulaire de création d'utilisateur """
        form = scribe_tools._get_pupil_form(self.user_type, self.user)
        form.update(dict(icone='/image/scribe/action_user_create_over.png'))
        return form

    def _get_help(self):
        """ renvoie la description du lien vers l'aide """
        return dict(help=dict(href=tools.make_help_link(self.server_nb, 'help', help_page='scribe_user')))

    def _get_validate_btn(self):
        """ renvoie la description du bouton valider """
        href = tools.make_form_link(self.server_nb, self.name, True, ['add_user'], 'return_msg_container')
        title = "Créer l'utilisateur"
        icone = "/image/ok.gif"
        libelle = "Valider"
        return dict(validate=dict(href=href, title=title, icone=icone, libelle=libelle))
    
    def _valid_form(self):
        """ valide les retours formulaires, en l'occurence crée un élève """
        return scribe_tools.create_pupil(self.form_result)

class ProfModUser(Action):
    """ 
        entrée pour la modification d'élève pour les prof administrateur de classe (prof_admin)
        s'appuye sur le listing (scribe_user_table, la modification, modification de mdp, suppression d'élève de user.py)
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'scribe_prof_user_modify'
    libelle = "Modifier"
    category = None
    description = "modification d'un élève"
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend", keys=['server', 'action'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['add_user'])

    def execute(self):
        """ renvoie les données pour l'affichage 
            1 - renvoie le formulaire pour lister les élèves
            2 - filtre les élèves depuis un formulaire
        """
        params, self.server_nb = tools.get_request(self.request)
        self.user = self.user_description['name']
        self.user_type = scribe_tools.get_role(self.user_description['role'])
        ## 2 -
        if self.form_result != {}:# validation de formulaire
            filtered_datas = self._valid_form()
            self.form_result = {}
            return self.send_frag(filtered_datas, template='scribe_prof_user', templates=used_templates)
        ## 1 -
        result = {}
        result.update(scribe_tools._get_user_listing_form(self.user))
        result.update(dict(validate=self._get_validate_btn()))
        return self.send_all(result, template='scribe_prof_user', templates=used_templates)

    def _get_validate_btn(self):
        """ renvoie la description du bouton valider """
        icone = '/image/ok.gif'
        title = 'Lister selon les filtres'
        libelle = 'Lister'
        href = tools.make_form_link(self.server_nb, 'scribe_user_table', True, ['search'], 'user_div_container')
        return dict(icone=icone, title=title, libelle=libelle, href=href)

class ProfGroup(Action):
    """ action d'entrée pour la gestion des groupes """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'scribe_prof_group'
    libelle = "Les groupes"
    category = "Gestion"
    description = "gestion des groupes"
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend", keys=['server', 'action'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=[])

    def execute(self):
        """ renvoie les données pour l'affichage (le menu )"""
        params, self.server_nb = tools.get_request(self.request)
        result = {'titre':"Gestion des groupes"}
        result.update(self._get_menu())
        return self.send_all(result, template='scribe_prof_group', templates=used_templates)

    def _get_menu(self):
        """ renvoie la description du menu """
        create = {'title':"Création de groupe", "icone":"/image/scribe/action_group_create.png",
                'libelle':"Créer groupe", '_class':'scribe_action_btn', 
                  "href":tools.make_js_link(self.server_nb, 'scribe_prof_group_create', 'main_div_container')}
        modify = {'title':"Modification d'utilisateur", "icone":"/image/scribe/action_user_modify.png",
                'libelle':"Modifier un élève", '_class':'scribe_action_btn',
                "href":tools.make_js_link(self.server_nb, 'scribe_prof_user_modify', 'main_div_container')}
        return dict(menus=(create,))

class ProfGroupCreate(Action):
    """ outil de création de groupe par un prof_admin (que des groupes de type Groupe)"""
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'scribe_prof_group_create'
    libelle = "Créer"
    category = None
    description = "Création de groupe"
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend", keys=['server', 'action'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['add_group'])

    def execute(self):
        """ renvoie les données pour l'affichage de la création de groupe
            1 - renvoie le formulaire de création de groupe
            2 - valide la création
        """
        params, self.server_nb = tools.get_request(self.request)
        ## 2 -
        if self.form_result != {}:
            try:
                datas = self.create_group()
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name, confirm=True, balise="main_div_container")
            except Exception, mess:
                log.err("Erreur dans prof.py : %s"%mess)
                datas = dict(message = mess.message)
            self.form_result = {}
            return self.send_frag(datas, template='scribe_prof_group', templates=used_templates)
        ## 1 -
        result = {'icone':'/image/scribe/action_group_create_over.png'}
        result.update(scribe_tools._groupe_form("du groupe"))
        result.update(self._get_valid_btn())
        return self.send_all(result, template='scribe_prof_group', templates=used_templates)

    def _get_valid_btn(self):
        """ bouton valider pour la creation d'user passe en meme temps le type en cours (classe, nivo...)"""
        validate={}
        validate['href'] = tools.make_form_link(self.server_nb, self.name, True, ['add_group'], 'return_msg_container')
        validate['icone'] = "/image/ok.gif"
        validate['libelle'] = "Valider"
        validate['title'] = "Créer ce groupe"
        return dict(validate=validate)

    def create_group(self):
        """ cree un groupe """
        if not self.form_result.has_key('add_group'):
            raise MissingKey, "Erreur : il manque des données pour la création du groupe."
        creation = tools.format_form_result(self.form_result['add_group'])
        return dict(message=scribe_tools.group_create(creation, 'Groupe', 'du groupe'))
