# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# Outils pour l'ead de scribe
#
###########################################################################
from twisted.python import log
from scribe import backend
import commands
from os.path import isfile
from os import popen
from ead2.backend.actions import tools
from ead2.lib.error import MissingKey, MissingValue, BadGroupName
from ead2.backend.config.filenames import printers_conf

## outil de listing utilisateur

def filter_by_alpha(users, alpha):
    """ filtre une liste d'utilisateur sur la première lettre """
    result = []
    for user in users:
        if user.lower().startswith(alpha.lower()):
            result.append(user)
    return result

def user_filter(_type='', domain='', _class='', namepart='', alpha_key='', group=''):
    """ filtre les utilisateurs selon des parametres
        on cree le filtre ldap et on lance une recherche
        :alpha_key: première lettre du nom de l'utilisateur
        :_type: type d'utilisateur prof/eleve (pupil/teacher)
        :domain: domaine mail
        :group: groupe des utilisateurs
        :class: classe de l'utilisateur (pour les élèves à priori,
                marche aussi pour les profs admin de classe)
        :size: nombre d'utilisateur par page
    """
    filtre = "(&%s" % backend.user_filters
    if _type:
        if _type == 'pupil':
            filtre += backend.eleve_filters
        else:
            filtre += backend.prof_filters
    if domain:
        filtre += "(mail=*@%s)" % domain
    if _class:
        filtre += "(Divcod=%s)" % _class
    if namepart:
        filtre += "(sn=*%s*)" % namepart
    filtre += ")"
    log.msg(filtre)
    usrs = backend.ldapsearch(filtre, ['uid'])
    users = [user[1]['uid'][0]for user in usrs]
    if group:
        gr_users = backend.get_members(group)
        users = [user for user in users if user in gr_users]
    users.sort()
    if alpha_key:
        users = filter_by_alpha(users, alpha_key)
    return users

def _get_alpha_links():
    """ renvoit la description des liens alphabetique pour un objet select"""
    alphas = [_get_select_default_value()]
    alphas.extend([{'name':chr(i).upper(), 'libelle':chr(i).upper()}for i in range(97, 123)])
    return alphas

def _get_select_default_value():
    """ renvoie l'option vide pour un objet select """
    return {'name':'', 'libelle':'', 'default':'ok'}

def _get_user_listing_form(prof_admin=None):
    """ renvoie la description du formulaire de recherche utilisateur
        :prof_admin: nom du prof dans le cas d'un prof_admin
                     (permet de restreindre les critères de recherche)
    """
    alpha_keys = {'name':'alpha_key', 'libelle': 'par la 1ère lettre du login',
            'options':[{'name':'commençant par', 'options':_get_alpha_links()}] }
    if prof_admin != None: # prof_admin ne peut modifier que des élèves
        user_type = {'name':'user_type', 'default_value':'pupil'}
    else:
        user_types = [_get_select_default_value(), {'name':'teacher', 'libelle':'professeur'},
                    {'name':'pupil', 'libelle':'élève'}]
        user_type = {'name':'user_type', 'libelle':"de type", 'options':[{'name':'', 'options':user_types}]}
    # classe
    if prof_admin != None: # prof_admin ne peut modifier que des élèves de ses classes
        opts = filter_class_by_prof(prof_admin, backend.get_classes())
    else:
        opts = backend.get_classes()
    classes = [{'name':key, 'options':[{'name':item, 'libelle':item}for item in items]}for key, items in opts.items()]
    if prof_admin is None:
        classes.insert(0, {'name':'', 'options':[_get_select_default_value()]})
    classe = {'name':'user_class', 'libelle':'de la classe', 'options':classes}
    # groupes

    if prof_admin is None:
        gr_types = ['Groupe', 'Matiere', "Responsables" , "Equipe"]
        groups = []
        for key in gr_types:
            group_names = backend.list_groups(key)
            group_names.sort()
            groups.append({'name':key, 'options':[{'name':gr, 'libelle':gr}for gr in group_names]})
        groups.insert(0, {'name':'', 'options':[_get_select_default_value()]})
        group = {'name':'user_group', 'libelle':'associé au groupe', 'options':groups}
    else:
        group = {'name':'user_group', 'default_value':''}
    #domaine
    domains = [{'name':domain, 'libelle':domain}for domain in backend.get_domains()]
    domains.insert(0 , _get_select_default_value())
    domain = {'name':'domains', 'libelle':'du domaine', 'options':[{'name':'', 'options':domains}]}
    nums = [{'name':str(10*num), 'libelle':str(10*num)}for num in range(6)[1:]]
    #nom
    namepart = {'name':'namepart', 'libelle':"Partie du nom"}
    #nombre d'utilisateur
    size = {'name':'size', 'libelle':'par page de', 'options':[{'name':'', 'options':nums}]}
    return dict(alpha_keys=alpha_keys, user_type=user_type, classe=classe, domain=domain, namepart=namepart,
                size=size, group=group)

def sort_users(users):
    """ trie les nom des utilisateurs par ordre alphabétique
        prenom.nom par nom
    """
    _list = []
    for user in users:
        login = user.split('.')
        if len(login)>1:
            _list.append("%s.%s"%(login[1], login[0]))
        else:
            _list.append("%s"%login[0])
    _list.sort()
    users = []
    for user in _list:
        login = user.split('.')
        if len(login)>1:
            users.append("%s.%s"%(login[1], login[0]))
        else:
            users.append("%s"%login[0])
    return users

## quotas

def _get_quotas(_type='depasse'):
    """ renvoie les quotas des utilisateurs du domaine
        dans le cas _type: qui prend les valeurs "proche", "depasse" ou "tous"
    """
    cmd = "/usr/sbin/repquota -a"
    return_code, quota_lines = tools.command_statusoutput(cmd)
    #commands.getstatusoutput(cmd)
    if return_code:
        raise Exception, "Erreur : un problème s'est produit à la récupération des quotas"
    quota_lines = quota_lines.splitlines()[5:]
    result = []
    for line in quota_lines:
        user = line.split()
        if len(user) < 5:
            continue
        username = user[0]
        quota = int(user[3]) / 1000
        if quota == 0 or username.startswith('#'):
            continue
        used = int(user[2])/1000
        delai = user[5]
        if used > quota:
            result.append({'username':username, 'used':used, 'quota':quota, 'delai':delai, 'color':'#fd8758'})
        elif used+3 > quota and _type == 'proche':
            result.append({'username':username, 'used':used, 'quota':quota, 'color':'#fddc58'})
        elif _type == 'tous':
            if used+3 > quota:
                result.append({'username':username, 'used':used, 'quota':quota, 'color':'#fddc58'})
            else:
                result.append({'username':username, 'used':used, 'quota':quota})
    return result

## virus

def _get_viruses():
    """ renvoie la liste des virus détectés """
    file1 = "/var/log/syslog"
    file2 = "/var/log/syslog.1.gz"
    max_vir = 30
    mois = {}
    mois["jan"] = "janvier" #idem
    mois["feb"] = "février"
    mois["fév"] = "février"
    mois["mar"] = "mars" #idem
    mois["apr"] = "avril"
    mois["avr"] = "avril"
    mois["may"] = "mai"
    mois["mai"] = "mai"
    mois["jun"] = "juin"
    mois["jui"] = "juin"
    mois["jul"] = "juillet" #idem
    mois["aug"] = "août"
    mois["aou"] = "août"
    mois["sep"] = "septembre" #idem
    mois["oct"] = "octobre"
    mois["nov"] = "novembre"
    mois["dec"] = "décembre"
    result = []
    viruses = []
    if isfile(file1):
        fic = popen('grep "infected with virus" '+file1)
        lines = fic.readlines()
        lines.reverse()
        result.extend(lines)
        fic.close()
    if isfile(file2):
        fic = popen('zgrep "infected with virus" '+file2)
        lines = fic.readlines()
        lines.reverse()
        result.extend(lines)
        fic.close()
    if len(result) > max_vir:
        result = result[:max_vir]
    for ligne in result:
        try :
            moisfr = mois[ligne[0:3].lower()]
        except :
            moisfr = ligne[0:3]
        date = ligne[4:6]+" "+moisfr
        ligne = ligne[ligne.find("'")+1:]
        next = ligne.find("'")
        ## fichier
        fichier = ligne[0:next]
        ligne = ligne[next+23:]
        next = ligne.find("'")
        ## virus
        virus = ligne[0:next]
        ## client
        client = ligne[next+12:-2]
        viruses.append({'client':client, 'fichier':fichier, 'date':date, 'virus':virus})
    if len(result) == 0:
        return []
    return viruses

## historique de connexion

def _get_history():
    """ renvoie l'historique des connexions """
    cmd = "/usr/share/ead/historique.sh"
    return_code, result = command_statusoutput(cmd)
    #commands.getstatusoutput(cmd)
    if return_code:
        raise Exception, "Erreur à la récupération de l'historique des connexions."
    lines = [{'date':" ".join(line.split()[0:5]), 'user':line.split()[5], 'poste':"%s (%s)"%(line.split()[6], line.split()[8]), 'system':line.split()[7]} for line in result.splitlines()]
    if len(lines) == 0:
        return [{'default':"Aucune entrée n'a été enregistrée."}]
    return lines

### gestion des imprimantes

def _parse_printers(printers_str):
    """ parse le résultat du script /user/share/ead/printers.sh
        les tests s'effectue sur les deux langues (an et fr)
        normalement dans l'ead2 c'est fr.
    """
    printers = []
    printer = {}
    for line in printers_str.splitlines():
        line = line.strip()
        if line.startswith("printer") or line.startswith("l'imprimante"):
            if printer != {}:
                printers.append(printer)
                printer = {}
            elt = line.split()
            printer_name = elt[1]
            printer['name'] = printer_name
            if "disabled" in line or "désactivée" in line:
                printer['stopped'] = 'ok'
            elif "idle" in line or "ne fait rien" in line:
                printer['stopped'] = "idle"
            else:
                printer['stopped'] = ""
        elif line.startswith('Description'):
            description = line[13:].strip()
            if "No Information Available" in description:
                printer['description'] = "Description : %s" % printer_name
            else:
                printer['description'] = line
        elif line.startswith('Location') or line.startswith('Lieu'):
            lieu = line.split(':')[1]
            if 'Location Unknown' in lieu:
                printer['lieu'] = "Emplacement non renseigné"
            else:
                printer['lieu'] = "Emplacement : %s" % lieu
    printers.append(printer)
    return printers

printer_script = printers_conf['script']
def _get_printers():
    """ renvoie la description des imprimantes disponibles """
    cmd = "%s all 2>&1" % printer_script
    return_code, printers_str = tools.command_statusoutput(cmd)
    # commands.getstatusoutput(cmd)
    if return_code:
        raise Exception, "Erreur à la récupération de la liste des imprimantes."
    printers = _parse_printers(printers_str)
    return printers

def stop_printer(printer_name):
    """ stoppe une imprimante """
    cmd = "%s disable %s 2>&1" % (printer_script, printer_name)
    return_code, return_str = tools.command_statusoutput(cmd) #commands.getstatusoutput(cmd)
    if return_code:
        raise Exception, "Erreur à l'arrêt de l'imprimante %s, \\n %s" % (printer_name, return_str)
    # on controle que ça a marché
    printers = _get_printers()
    for printer in printers:
        if printer['name'] == printer_name:
            if printer['stopped'] != 'ok':
                return "Compte rendu de l'arrêt de %s \\n Erreur : vérifiez votre configuration." % printer_name
    return "Compte rendu de l'arrêt de %s \\n Réussite de l'arrêt de l'imprimante" % printer_name

def start_printer(printer_name):
    """ démarre une imprimante """
    cmd = "%s enable %s 2>&1" % (printer_script, printer_name)
    return_code, return_str = tools.command_statusoutput(cmd) #commands.getstatusoutput(cmd)
    if return_code:
        raise Exception, "Erreur au redémarrage de l'imprimante %s, \\n %s" % (printer_name, return_str)
    # on controle que ça a marché
    printers = _get_printers()
    for printer in printers:
        if printer['name'] == printer_name:
            if printer['stopped'] == 'ok':
                return "Compte rendu du démarrage de %s. \\n Erreur : vérifiez votre configuration." % printer_name
    return "Compte rendu du démarrage de %s. \\n Réussite du démarrage de l'imprimante." % printer_name

def _get_printers_job(printer_name=None, completed=False):
    """ renvoie la liste des travaux pour printer_name
        :completed:True/False  <travaux terminé>/<travaux en cours>
    """
    jobs = []
    if completed:
        cmd = "%s completed" % printer_script
    else:
        cmd = "%s not-completed" % printer_script
    if printer_name:
        cmd += " %s 2>&1" % printer_name
    else:
        cmd += " 2>&1"
    return_code, return_str = tools.command_statusoutput(cmd) #commands.getstatusoutput(cmd)
    if return_code not in [256, 0]:
        raise Exception, "Erreur lors de la récupération des travaux sur %s." % printer_name
    for job in return_str.splitlines():
        job = job.split()
        user = job[1]
        printid = job[0]
        date = job[3] + ' ' + job[4] + ' ' + job[5] + ' ' + job[6]
        heure = job[7]
        jobs.append({'user':user, 'date':date, 'printid':printid, 'heure':heure})
    return jobs

def cancel_print(printid):
    """ annulation d'un travail d'impression """
    cmd = "%s cancel %s" % (printer_script, printid)
    return_code, return_str = tools.command_statusoutput(cmd) #commands.getstatusoutput(cmd)
    if return_code:
        raise Exception, "Erreur lors de l'annulation de %s" % printid
    return "L'impression %s a bien été annulée." % printid

## Gestion des utilisateurs

## outils pour la gestion des roles (prof_admin et prof)
def get_prof_class(login):
    """ renvoie les classes du prof login """
    classes = backend.get_classes()
    return filter_class_by_prof(login, classes)

def get_prof_class_list(login):
    """ renvoie la liste des classes d'un prof """
    cls = []
    for classe in get_prof_class(login).values():
        cls.extend(classe)
    return cls

def filter_class_by_prof(prof, classes):
    """ renvoie les classes du prof prof """
    groups = backend.get_admin_classes(prof)
    for niveau, classe_list in classes.items():
        nivo = []
        for classe in classe_list:
            if classe in groups:
                nivo.append(classe)
        if nivo != []:
            classes[niveau] = nivo
        else:
            classes.pop(niveau)
    return classes

def is_eleve_from_prof(eleve, prof):
    """ l'eleve est il membre d'une classe administrée par prof ?"""
    if backend.is_admin(eleve):
        #c'est un prof
        return False
    cls = get_prof_class_list(prof)
    for classe in cls:
        if eleve in backend.get_members(classe):
            return True
    return False

## création de prof
def _get_teacher_form():
    """ renvoit la description du formulaire de création de professeur
    :user, name, forname, password, date de naissance: inputs de login nom prenom,
                                                       mot de passe, date de naissance
    :civilite: select pour Mr/Mme/Mlle
    :date:
    :mail:radios et input pour la configuration du mail
    :quota: input pour quotas disque attribué
    :profile: select pour profil utilisateur local/profil1/profil2
    :principal: select pour prof principal?
    :admin: checkbox pour administrateur?
    :shell: activer le shell (/bin/bash)
"""
    user = {'name':'user', 'libelle':'Login (prenom.nom conseillé)',
           'onblur':"testVoid('user', 'error_user')", 'inline':'ok'}
    name = {'name':'name', 'libelle':"Nom de l'utilisateur",
           'onblur':"testVoid('name', 'error_name')", 'inline':'ok'}
    forname = {'name':'forname', 'libelle':"Prénom de l'utilisateur",
            'onblur':"testVoid('forname', 'error_forname');", 'inline':'ok'}
    date = {'name':'date', 'libelle':"Date de naissance (format jj/mm/aaaa)",
            'onblur':"testVoid('date', 'error_date');", 'inline':'ok'}
    password = {'name':'password', 'libelle':"Mot de passe",
                "onblur": "testVoid('password', 'error_password');", 'inline':'ok'}
    civilite_opts = [{'name':'1', 'libelle':'Mr.'}, {'name':'2', 'libelle':'Mme'},
                    {'name':'3', 'libelle':'Mlle'}]
    civilite = {'name':'civilite', 'options':[{'name':'', 'options':civilite_opts}],
                'libelle':"Civilité", 'inline':'ok'}

    default_domain = backend.get_domains()[1]
    mail = {'libelle':"Adresse e-mail (personalisée <b>ou</b> locale)",
            'radios':[{'name':'mailtype', 'value':'perso','inline':'ok', 'checked':'ok'},
                {'name':'mailtype', 'value':'local','inline':'ok'}],
            'radios_libelle':['Adresse personalisée :', 'Adresse locale:<br /> login@%s' % default_domain],
            'input':{'name':"mailperso", "libelle":''}
            }
    quota = {'name':'user_quota', 'libelle':"Quota disque (0 pour inactif)", 'inline':'ok',
                'default_value':'0', 'onblur':"testVoid('user_quota', 'error_user_quota')"}
    options = [{'name':'1', 'libelle':'local'}, {'name':'2', 'libelle':"obligatoire - profil", 'default':'ok'},
                {'name':'3', 'libelle':"obligatoire - profil2"}]
    profile = {'name':'_sambaProfilePath', 'libelle':"Profil utilisateur",
            'options':[{'name':"Profil disponible", 'options':options}], 'inline':'ok'}
    opts = [{'name':'', 'libelle':'non', 'default':'ok'}]
    opts = backend.get_classes()
    classes = [{'name':key, 'options':[{'name':item, 'libelle':"de la classe de %s" % item}for item in items]}for key, items in opts.items()]
    classes.insert(0, {'name':'', 'options':[{'name':'0', 'libelle':'non', 'default':'ok'}]})
    principal = {'name':'principal', 'libelle':'Prof principal', 'inline':'ok', 'options':classes}
    admin = {'name':'admin', 'libelle':"Membre du groupe DomainAdmins", 'inline':'ok', 'value':'admin'}
    shell = {'name':'shell', 'libelle':"Activation du shell (gestion de clients Linux)", 'inline':'ok', 'value':'shell'}
    return dict(user=user, name=name, forname=forname, password=password, date=date, civilite=civilite, mail=mail,
            quota=quota, profile=profile, principal=principal, admin=admin, shell=shell)

def create_teacher(form_result):
    """ cree un prof """
    if not form_result.has_key('add_user'):
        return ''
    creation = tools.format_form_result(form_result['add_user'])
    formkeys = ['user', 'name', 'forname', 'password', 'civilite', 'user_quota', '_sambaProfilePath', 'mailtype', 'mailperso', 'principal', 'date']
    for val in formkeys:
        if val not in creation.keys():
            raise MissingKey, "Il manque une information : %s" % val
    for i in ['name', 'forname', 'password']:
        try:
            creation[i] = creation[i].encode('utf8')
        except:
            continue

    user, name, forname, password, date = (creation['user'].strip(),
                                     creation['name'].strip(),
                                     creation['forname'].strip(),
                                     creation['password'].strip(),
                                     creation['date'].strip())
    if not tools.test_date(date):
        raise MissingKey, "Erreur : le format de la date de naissance %s est incorrect." % date

    civilite,  quota, profil, mailtype, mailperso, principal = (creation['civilite'].strip(),
                                                                creation['user_quota'].strip(),
                                                                creation['_sambaProfilePath'].strip(),
                                                                creation['mailtype'].strip(),
                                                                creation['mailperso'].strip(),
                                                                creation['principal'].strip())

    if not tools.test_login(user):
        raise MissingKey, 'Le login ne doit contenir que les caratères de a à z, de 0 à 9 et -_.'
    for var in [user, name, forname, password, civilite, quota, profil, mailtype, principal]:
        if not var:
            raise MissingValue, "L'information n'est pas renseignée"
    for key, value in {'nom':name, 'prenom':forname, 'mot de passe':password, 'mail':mailtype}.items():
        if not tools.test_ldap_entry(value):
            raise MissingValue, "La variable %s n'est pas au bon format." % key

    if principal == '0':
        principal = ''

    if mailtype == 'perso' and mailperso == '':
        raise Exception, "Erreur : il manque l'adresse mail perso."

    if '@' not in mailperso and mailperso != '':
        raise Exception, "Erreur : adresse mail invalide."

    if mailtype == 'perso':
        mail = mailperso
    else:
        mail = 'local'
    
    shell = creation.has_key('shell')

    # test d'existence
    if backend.is_user(user) or backend.is_system_user(user):
        raise Exception, "L'utilisateur %s existe déjà" % user

    if backend.ajout_prof(user, password, name, forname, date, principal, civilite, mail, quota, profil, shell):
        if creation.has_key('admin'):
            backend.inscription(user, 'DomainAdmins')
        return dict(message="Réussite de la création du professeur %s %s" % (forname, name))
    else:
        raise Exception, "Erreur : échec lors de la création du professeur %s %s" % (forname, name)

## les élèves

def _get_pupil_form(role='admin', login=''):
    """ renvoie la description du formulaire de creation d'eleve
    :user, name, forname, password, date: inputs de login, nom, prenom, mot de passe, date de naissance
    :civilite: select pour Mr/Mme/Mlle
    :user_number:input pour le numéro elenoet de l'élève
    :quota: input pour quotas disque attribué
    :profile: select pour profil utilisateur local/profil1/profil2
    :divcod: select pour le choix de la classe
    @role : role associé à l'utilisateur appelant
                    (permet de restreindre la création aux classes d'un prof_admin)
    @login : login du prof_admin
    :shell: activer le shell (/bin/bash)
    """
    user = {'name':'user', 'libelle':'Login (prenom.nom conseillé)',
            'onblur':"testVoid('user', 'error_user')", 'inline':'ok'}
    domains = [{'name':domain, 'libelle':domain}for domain in backend.get_domains()]
    domains[0]['default'] = 'ok'
    domains = {'name':'domains', 'libelle':'@', 'inline':'ok',
               'options':[{'name':'', 'options':domains}]}
    name = {'name':'name', 'libelle':"Nom de l'utilisateur",
            'onblur':"testVoid('name', 'error_name')", 'inline':'ok'}
    forname = {'name':'forname', 'libelle':"Prénom de l'utilisateur",
            'onblur':"testVoid('forname', 'error_forname');", 'inline':'ok'}
    password = {'name':'password', 'libelle':"Mot de passe",
                "onblur": "testVoid('password', 'error_password');", 'inline':'ok'}
    date = {'name':'date', 'libelle':"Date de naissance (format jj/mm/aaaa)",
            'onblur':"testVoid('date', 'error_date');", 'inline':'ok'}
    civilite_opts = [{'name':'1', 'libelle':'Mr.'}, {'name':'2', 'libelle':'Mme'}, {'name':'3', 'libelle':'Mlle'}]
    civilite = {'name':'civilite', 'options':[{'name':'', 'options':civilite_opts}],
                'libelle':"Civilité", 'inline':'ok'}
    user_number = {'name':'user_number', 'libelle':"Numéro de l'élève (ELENOET dans GEP)", 'inline':'ok',
            'onblur':"testVoid('user_number', 'error_user_number')"}
    quota = {'name':'user_quota', 'libelle':"Quota disque (0 pour inactif)", 'inline':'ok',
                'default_value':'0', 'onblur':"testVoid('user_quota', 'error_user_quota')"}
    options = [{'name':'1', 'libelle':'local'}, {'name':'2', 'libelle':"obligatoire - profil", "default":"ok"},
                {'name':'3', 'libelle':"obligatoire - profil2"}]
    profile = {'name':'_sambaProfilePath', 'libelle':"Profil utilisateur",
            'options':[{'name':"Profil disponible", 'options':options}], 'inline':'ok'}
    classes = backend.get_classes()
    if role == 'prof_admin':
        classes = filter_class_by_prof(login, classes)
        if classes == {}:
            return dict(message="Erreur : vous n'êtes administrateur d'aucune classe")
    options = [{'name':key, 'options':[{'name':item, 'libelle':item}for item in items]}for key, items in classes.items()]
    divcod = {'name':'divcod', 'options':options,
                'inline':'ok', 'libelle':"Classe"}
    shell = {'name':'shell', 'libelle':"Activation du shell (gestion de clients Linux)", 'inline':'ok', 'value':'shell'}
    return dict(user=user, domains=domains, name=name, forname=forname, password=password, date=date,
            civilite=civilite, user_number=user_number, quota=quota,
            profile=profile, divcod=divcod, shell=shell)

def create_pupil(form_result):
    """ cree un élève
    """
    if not form_result.has_key('add_user'):
        return ''
    ## on teste le retour de toutes les valeurs
    formkeys = ['user', 'name', 'forname', 'password', 'domains', 'civilite', '_sambaProfilePath', 'divcod', 'user_number', 'user_quota', 'date']
    creation = tools.format_form_result(form_result['add_user'])
    for key in formkeys:
        if key not in creation.keys():
            raise MissingKey, "Il manque une information : %s" % key
    for i in ['name', 'forname', 'password']:
        try:
            creation[i] = creation[i].encode('utf8')
        except:
            continue
    ## on récupère les valeurs
    user, name, forname, password = (creation['user'].strip(),
                                     creation['name'].strip(),
                                     creation['forname'].strip(),
                                     creation['password'].strip())

    domain, civilite, profil, divcod = (creation['domains'].strip(),
                                        creation['civilite'].strip(),
                                        creation['_sambaProfilePath'].strip(),
                                        creation['divcod'].strip())

    user_number, quota, date = (creation['user_number'].strip(),
                                creation['user_quota'].strip(),
                                creation['date'].strip())

    if not tools.test_login(user):
        raise MissingKey, 'Le login ne doit contenir que les caratères de a à z, de 0 à 9 et -_.'
    if not tools.test_date(date):
        raise MissingKey, "Erreur : le format de la date de naissance %s est incorrect." % date
    ## on teste les valeurs vides
    for var in [date, user_number, quota, domain, civilite, profil, divcod, user, name, forname, password]:
        if not var:
            raise MissingValue, "Une des variables n'est pas renseignée."
    for var in [domain, user, name, forname, password]:
        if not tools.test_ldap_entry(var):
            raise MissingValue, "Une des variables n'est pas au bon format."
    try:
        quota = int(quota)
    except:
        raise MissingValue, "La variable quota n'est pas un entier"

    shell = creation.has_key('shell')

    # test d'existence
    if backend.is_user(user) or backend.is_system_user(user):
        raise Exception, "L'utilisateur %s existe déjà" % user

    ## on ajoute l'utilisateur
    if backend.ajout_eleve(user, password, name, forname, date, divcod, user_number,
            civilite, domain, quota=quota, profil=profil, shell=shell):
        return dict(message="Réussite de la création de l'élève %s %s" % (forname, name))
    else:
        raise Exception, "Echec lors de la création de l'élève %s %s" % (forname, name)

## outil de gestion de groupe
def _groupe_form(msg):
    """ renvoie la description du formulaire pour la création d'un groupe, une matière ou un niveau
    :msg: message associé au type de groupe à créer
    @group_name : nom du groupe (input)
    @share_check : avec partage ? (checkbox)
    @share_status : statut du partage
    @liste_check : avec liste de diffusion ? (checkbox)
    @liste_status : statut de la liste de diffusion (internet ou restreint)
    """
    group_name = {'name':'group_name', 'libelle':'Intitulé %s' % msg,
                    'inline':'ok','onblur':"testVoid('group_name', 'error_group_name')"}
    share_check = {'name':'share_check', 'libelle':"Avec Partage", 'inline':'ok', 'value':'share_check'}
    share_status = {'name':'share_status', 'libelle':"Partage",
                'options':[{'name':'', 'options':[{'name':'rw', 'libelle':'lecture/écriture (pour les élèves)'},
                    {'name':'ro', 'libelle':'lecture seule (pour les élèves)'},
                    {'name':'dt', 'libelle':'modèle données/travail'}
                    ]}], 'inline':'ok'
                }
    liste_check = {'name':'liste_check', 'libelle':"Avec liste de diffusion", 'inline':'ok', 'value':'liste_check'}
    liste_status = {'name':'liste_status', 'libelle':"Liste de diffusion",
                'options':[{'name':'', 'options':[{'name':'restreint', 'libelle':"domaine restreint (conseillé)"},
                                                    {'name':'internet', 'libelle':"domaine internet"}]}],
                'inline':'ok'}
    return dict(group_name=group_name, share_check=share_check, share_status=share_status,
                    liste_check=liste_check, liste_status=liste_status)

def group_create(form_result, _type, msgs):
    """ crée un groupe autre qu'une classe (niveau, groupe, equpe péda, matiere, responsable de classe)
        :form_result: résultat de formulaire de type dico avec pour clé:
            'group_name' : nom du groupe
            'share_status' : droits du partage
            'liste_status' : type de liste de diffusion
         et éventuellement:
            'share_check' : association d'un partage?
            'liste_check' : association d'une liste de diffusion?
        :_type: type de groupe à créer
        :msgs: message associé au groupe à créer
    """
    for i in ['group_name', 'share_status', 'liste_status']:
        if i not in form_result.keys():
            raise MissingKey, "Erreur : Des données manque pour la création %s" % msgs
    name = form_result['group_name']
    if not tools.test_login(name):
        raise BadGroupName #MissingValue, "Erreur : Le nom n'est pas au bon format"
    if form_result.has_key('share_check'):
        share_status = form_result['share_status']
    else:
        share_status = ''
    if form_result.has_key('liste_check'):
        liste_status = form_result['liste_status']
    else:
        liste_status = ''
    res = backend.ajout_groupe(name, _type, domaine=liste_status, optrw=share_status)
    if res:
        return "Réussite de l'ajout %s %s." % (msgs, name)
    else:
        raise MissingKey, "Erreur : échec à la création %s %s." % (msgs, name)

def class_create(form_result):
    """ ajoute une classe
        :form_result : résultat de formulaire avec pour clé:
            - group_name : nom de la classe
            - level : niveau de la classe
            - liste_status : type de liste de diffusion
    """
    for i in ['group_name', 'level', 'liste_status']:
        if i not in form_result.keys():
            raise MissingKey, "Erreur : Des données manque pour la création de la classe"
    name, level, liste_status = form_result['group_name'], form_result['level'], form_result['liste_status']
    if not tools.test_login(name):
        raise BadGroupName
    for i in name, level, liste_status:
        if not i.strip():
            raise MissingValue, "Il manque des valeurs"
    res = backend.ajout_classe(name, level, liste_status)
    if res:
        return "Réussite de l'ajout de la classe %s" % name
    else:
        raise Exception, "Erreur : échec à la création de la classe %s" % name

# gestion des rôles sur scribe
def is_admin(roles):
    return ('admin_pedago' in roles) or ('admin' in roles)

def is_prof_admin(roles):
    return (not is_admin(roles)) and ('prof_admin' in roles)

def is_prof(roles):
    return (not is_prof_admin(roles)) and ('prof' in roles) and (not is_admin(roles))

def get_role(roles):
    if is_admin(roles):
        return 'admin'
    elif is_prof_admin(roles):
        return 'prof_admin'
    else:
        return 'prof'
