# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# Action scribe_user*
# Creation d'utilisateur
# Recherche puis modification d'utilisateur
# modfication de mot de passe
# Supression d'utilisateur
#
###########################################################################
"""Actions permettant de gérer les utilisateurs"""
from twisted.python import log
from ead2.backend.lib.action import * # import des actions et des types
from ead2.backend.actions import tools
from ead2.lib.error import MissingKey, MissingValue
import commands
from os.path import join
from scribe import backend
from ead2.backend.actions.scribe import scribe_tools

import traceback

# liste des templates complémentaires utilisés
used_templates = ['main', 'form', 'formselect', 'accordion', 'listlist', 'checklist']

def _get_menu(server_nb, current, _type=None):
    """ renvoit la description du menu de gestion utilisateur
    :current: section en cours de traitement (ex: ajout utilisateur user_create)
    :_type: parametre de la section en cours de traitement (ex: pupil/teacher)
    """
    create_pupil = {'icone':'/image/scribe/user_create.gif', 'libelle':"Création d'élève",
            'title':"Créer un utilisateur", 'href':tools.make_js_link(server_nb, 'scribe_user_create', _type='pupil')}
    create_teacher = {'icone':'/image/scribe/teacher_create.gif', 'libelle':"Création de professeur",
                        'title':"Créer un utilisateur", 'href':tools.make_js_link(server_nb, 'scribe_user_create', _type='teacher')}
    _list = {'icone':'/image/scribe/user_list.gif', 'libelle':"Liste",
            'title':"Lister des utilisateurs", 'href':tools.make_js_link(server_nb, 'scribe_user_list')}
    if current == 'scribe_user_create' and _type=='pupil':
        create_pupil['href'] = ''
    elif current == 'scribe_user_create' and _type=='teacher':
        create_teacher['href'] = ''
    elif current == 'scribe_user_list':
        _list['href'] = ''
    return dict(menus=[create_pupil, create_teacher, _list])

def get_role_association_btn(server_nb, username):
    """ renvoie la descritpion du bouton d'appel pour l'édition de rôle """
    return {'href': tools.make_js_link(server_nb, 'role_manager',
                                        balise='div_role_container',
                                        form='usertoassociate',
                                        user=username),
                'libelle':"Associer un rôle à cet utilisateur",
                'title': '',
                'icone': "/image/action_role.png",
                "_class":'scribe_action_btn'
                }

class UserCreate(Action):
    """ création d'utilisateur
    :type: type d'utilisateur en cours de création ( élève, prof )
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'scribe_user_create'
    libelle = "Création d'utilisateur"
    category = "Gestion/Utilisateurs/"
    description = 'Créer un utilisateur'
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend", keys=['server', 'action', '_type', 'datas'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['add_user', 'current_type'])

    def execute(self):
        """ renvoit les données pour l'affichage
            1 - récupère le type de l'utilisateur à créer
            2 - renvoie le formulaire de création
                le formulaire est conditionné par le role de l'appelant (cas prof_admin)
            3 - renvoie le menu si besoin
            4 - crée l'utilisateur selon le retour formulaire
                a - récupère les données
                b - controle d'intégrité
                c - crée l'utilisateur
        """
        params, self.server_nb = tools.get_request(self.request)
        result = {'titre':"Gestion des utilisateurs"}
        ## 1 -
        self._type = 'pupil'
        if params.has_key('_type'):
            self._type = params['_type'][0].strip()

        if params.has_key('datas'): ## on demande que le formulaire
            result = self._get_form()
            result.update(self._get_valid_btn())
            return self.frag(result)

        if self.form_result !={} :
            if self.form_result.has_key('current_type'):
                self._type = tools.format_form_result(self.form_result['current_type'])['_type']
                self.form_result.pop('current_type')
            try:
                result.update(self._valid_form())
                # si on a créé un user, on vide le formulaire
                result['toexec'] = tools.make_js_link(self.server_nb, self.name, confirm=True,
                            balise='user_create_div_container',
                                                datas='form', _type=self._type)
            except (MissingKey, MissingValue, Exception), mess:
                log.err("Erreur dans user.py : %s"%mess)
                traceback.print_exc()
                result['message'] = mess.message
            self.form_result = {}
            return self.frag(result)

        typedico={'pupil':'élève', 'teacher':'professeur'}
        result['sstitre'] = "Créer un %s"%typedico[self._type]
        result['_type'] = {'name':'_type', 'default_value':self._type}
        result.update(_get_menu(self.server_nb, self.name, self._type))
        result.update(self._get_form())
        result.update(self._get_valid_btn())
        return self.send_all(result, template='scribe_users', templates=used_templates)

    def frag(self, result):
        """ renvoie la description pour un fragment de page """
        return self.send_frag(result, template='scribe_users', templates=used_templates)

    def _get_form(self):
        """ renvoie la description des formulaires de création d'utilisateurs"""
        if self._type == 'pupil':
            return scribe_tools._get_pupil_form()
        else:
            return scribe_tools._get_teacher_form()

    def _get_valid_btn(self):
        """ bouton validere pour la creation d'user passe en meme temps le type en cours de creation eleve/prof"""
        validate={}
        validate['href'] = tools.make_form_link(self.server_nb, self.name, True, ['current_type', 'add_user'], 'user_create_div_msg')
        validate['icone'] = "/image/ok.gif"
        validate['libelle'] = "Valider"
        validate['title'] = "Créer cet utilisateur"
        return dict(validate=validate)

    def _valid_form(self):
        """ valider les formulaires """
        if self._type == 'pupil':
            return scribe_tools.create_pupil(self.form_result)
        else:
            return scribe_tools.create_teacher(self.form_result)

class UserListing(Action):
    """ renvoie le formuliare de listing utilisateur
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'scribe_user_list'
    libelle = "Recherche d'utilisateur"
    category = "Gestion/Utilisateurs/"
    description = 'Lister des utilisateurs'
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend",
                    keys=['server', 'action'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=[])

    def execute(self):
        """ renvoit les données pour l'affichage
        """
        params, self.server_nb = tools.get_request(self.request)
        result = {'titre':"Gestion des utilisateurs", "sstitre":self.description}
        result.update(_get_menu(self.server_nb, self.name))
        result.update(scribe_tools._get_user_listing_form())
        result.update({'validate':self._get_valid_btn()})
        return self.send_all(result, template='scribe_users', templates=used_templates)

    def _get_valid_btn(self):
        """ renvoie la description du bouton de validation de la recherche
            la cible du retour est la balise : user_div_container
        """
        icone = '/image/ok.gif'
        title = 'Lister selon les filtres'
        libelle = 'Lister'
        href = tools.make_form_link(self.server_nb, 'scribe_user_table', True, ['search'], 'user_div_container')
        return dict(icone=icone, title=title, libelle=libelle, href=href)

class UserTable(Action):
    """ action gérant les tableaux d'utilisateurs listés
    :alpha: alphabétique
    :domain: domaine mail
    :classe: classe de l'utilisateur
    :_type: eleve/prof
    :namepart: portion du nom des utilisateurs recherches
    :size: nombre d'utilisateur par page
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'scribe_user_table'
    libelle = "Tableau d'utilisateur"
    category = None
    description = 'Tableau des utilisateurs'
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend",
                    keys=['server', 'action', 'page', 'alpha_key', '_type', 'domain', '_class', 'size', 'namepart', 'group'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['search'])

    def execute(self):
        """ renvoit les données pour l'affichage
            1 - renvoie la liste des utilisateurs selon les critères demandés
            2 - renvoie la page suivante si le listing est en cour
        """
        params, self.server_nb = tools.get_request(self.request)
        self.user_alpha = 'a'
        page = 1 # permet de savoir quelle page de listing afficher en cas de nombreux utilisateurs

        ## 2 -
        if params.has_key('page'): # dans le cas de page multiple
            page = int(params['page'][0])
            params.pop('page')
        if params.has_key('alpha_key'):
            alpha_key, _type, domain = params['alpha_key'][0], params['_type'][0], params['domain'][0]
            _class, namepart = params['_class'][0], params['namepart'][0]

            if params.has_key('group'): group = params['group'][0]
            else: group = ''

            size = int(params['size'][0]) #nombre d'item par page

            filtered_datas = self.filter_users(alpha_key, _type, domain, _class, namepart, group, size, page)

            return self.frag(filtered_datas)

        ## 1 -
        if self.form_result != {}:# validation de formulaire
            filtered_datas = self._valid_form()
            self.form_result = {}
            return self.frag(filtered_datas)
        return self.send_all(dict(message="Veuillez renouveller le listing utilisateur."), template='scribe_users', templates=used_templates)

    def frag(self, filtered_datas):
        """ renvoit les datas pour l'affichage de la liste des utilisateurs correpondant à la recherche"""
        return self.send_frag(filtered_datas, template='scribe_users', templates=used_templates)

    def _valid_form(self):
        """ valide les retours de formulaire """
        request = tools.format_form_result(self.form_result['search'])
        keys = request.keys()
        if 'alpha_key' not in keys or 'domains' not in keys or 'user_class' not in keys or 'user_type' not in keys or 'namepart' not in keys or 'size' not in keys:
            return ""
        alpha_key, _type, domain, _class, namepart = request['alpha_key'], request['user_type'], request['domains'], request['user_class'], request['namepart']

        if request.has_key('group'):
            group = request['group']
        else:
            group = ''

        size = int(request['size'])

        return self.filter_users(alpha_key, _type, domain, _class, namepart, group, size)

    def filter_users(self, alpha_key="", _type="", domain="", _class="", namepart='', group='', size=20, page=1):
        """ filtre les utilisateurs selon les parametres  on cree le filtre ldap et on lance une recherche
            :alpha_key: première lettre du nom de l'utilisateur
            :_type: type d'utilisateur prof/eleve (pupil/teacher)
            :domain: domaine mail
            :class: classe de l'utilisateur (pour les élèves à priori, marche aussi pour les profs admin de classe)
            :size: nombre d'utilisateur par page
        """
        namepart.strip()
        if (scribe_tools.is_prof_admin(self.user_description['role'])
                  or scribe_tools.is_prof(self.user_description['role'])):
            if _class == "":
                return dict(message="Il manque des paramètres.")
            elif _class not in scribe_tools.get_prof_class_list(self.user_description['name']):
                return dict(message="Action non autorisée.")

        users = scribe_tools.user_filter(_type, domain, _class, namepart, alpha_key, group)
        length = len(users)
        if length > size: # on affiche sur plusieurs pages
            #FIXME: l'ordre reste-t-il le meme dans les resultats de requetes ldap
            max_index = size
            if page is not None:
                max_index = size * page
            min_index = max_index - size
            user_btns=self.get_btns(users, alpha_key, _type, domain, _class, namepart, group, size, page)
            users = self._get_table_users(users[min_index:max_index])
            return dict(users=users, user_btns=user_btns, nb_users="Nombre d'utilisateurs : %s"%length)
        users = self._get_table_users(users)
        return dict(users=users, nb_users="Nombre d'utilisateurs : %s"%length)

    def _get_table_users(self, users):
        """ renvoie la description du tableau de listing utilisateur
        """
        result = []
        for user in users:
            edit_pwd = {'title':'', 'libelle':"Changer le mot de passe",'icone':'', '_class':'scribe_simple_link',
                        'href': tools.make_js_link(self.server_nb, 'scribe_user_password', balise='mod_user_div_container',
                            user=user)}
            modify = {'title':'', 'libelle':"Editer", 'icone':'','_class':'scribe_simple_link',
                            'href':tools.make_js_link(self.server_nb, 'scribe_user_modify', balise='mod_user_div_container',
                                user=user)}
            _href = tools.make_js_link(self.server_nb, 'scribe_user_delete', confirm=True, balise='user_div_container', user=user)
            href = tools.make_confirm_link("Êtes-vous sûr de vouloir supprimer l'utilisateur %s"%user, _href)
            delete = {'title':'', 'libelle':"Supprimer", "icone":"", "_class":'scribe_simple_link',
                            'href':href}
            result.append({'name':user, 'pwd_link':edit_pwd, 'mod_link':modify, 'del_link':delete})
        return result

    def get_btns(self, users, alpha_key, _type, domain, _class, namepart, group, size, page):
        """ renvoit la description des boutons de passage de page en page """
        next = {'libelle': 'suivant', 'icone':'', 'title':'',
                'href':tools.make_js_link(self.server_nb, self.name, balise='user_div_container',
                                page=page+1, alpha_key=alpha_key, _type=_type, domain=domain,
                                _class=_class, namepart=namepart, group=group, size=size)}
        prec = {'libelle': 'précédent', 'icone':'', 'title':'',
                'href':tools.make_js_link(self.server_nb, self.name, balise='user_div_container',
                                page=page-1, alpha_key=alpha_key, _type=_type, domain=domain,
                                _class=_class, namepart=namepart, group=group, size=size)}
        if page == 1 :
            return [next]
        elif ((page) * size)>=len(users):
            return [prec]
        else:
            return [prec, next]

class UserModify(Action):
    """ Modifie les utilisateurs
        accessible par prof_admin avec un controle pour les modifs et les renvoi de formulaire.
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'scribe_user_modify'
    libelle = "Modification"
    category = None
    description = 'Modifier un utilisateur'
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend", keys=['server', 'action', 'user'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['mod_user', 'current_type',
                            "equipe_form", 'admin_class_form', 'matiere_form', 'groupe_form', 'spec_group_form'])

    def execute(self):
        """ renvoit les données pour l'affichage
            1 - récupère le nom et le type d'utilisateur à modifier
            2 - renvoie le formulaire approprié
            3 - tente la modification et si la modification est réussie on rafraichit le formulaire
                a - récupère les données du formulaire
                b - filtre pour voir les valeurs modifiées et vérifie leur intégrité
                c - applique les modifications
        """
        params, self.server_nb = tools.get_request(self.request)
        result = {}#'titre':"Gestion des utilisateurs"}
        ## 1-
        self.user = ''
        self._type = ''
        datas = {}
        if params.has_key('user'):
            self.user = params['user'][0]
        if self.form_result != {}: ## on modifie l'utilisateur
            if self.form_result.has_key('current_type'):
                self._type = tools.format_form_result(self.form_result['current_type'])['_type']
                self.form_result.pop('current_type')
            ## 3-
            try:
                datas = self._valid_form()
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name, confirm=True, balise='mod_user_div_container', user=self.user)
            except (MissingKey, MissingValue, Exception), mess:
                traceback.print_exc()
                log.err("Erreur dans user.py : %s"%mess)
                datas = dict(message=mess.message)
            self.form_result ={}
            return self.send_frag(datas, template='scribe_users', templates=used_templates)
        ## 2-
        datas = self._get_user()
        result['_type'] = {'name':'_type', 'default_value':self._type }
        result.update(self._get_form(datas))
        result.update(self._get_valid_btn())
        result.update(self._get_close_btn())
        if scribe_tools.is_admin(self.user_description['role']):
            result['role'] = get_role_association_btn(self.server_nb, self.user)
        return self.send_all(result, template='scribe_users', templates=used_templates)

    def _get_user(self):
        """ recupere les attributs de l'annuaire pour l'utilisateur courant, recupere le type au passage"""
        if self.user:
            datas = backend.get_user_data(self.user)
        else:
            return {}
        if backend.is_admin(self.user): self._type = 'teacher'
        else: self._type='pupil'
        return datas

### fonctions de description de formulaire
    def _get_form(self, datas):
        """ renvoie la description du formulaire de modification d'utilisateur """
        if datas == {}: return {}
        if self._type == 'teacher':
            return self._get_teacher_form(datas)
        else:
            return self._get_pupil_form(datas)

    def _get_teacher_form(self, datas):
        """ renvoie la description du formulaire de modification de prof """
        if scribe_tools.is_prof_admin(self.user_description['role']): # seul un admin peut modifier des profs
            return {'message':'Action non autorisée'}

        #login
        user_login = {'name':'login', 'libelle':"Identifiant", 'disabled':'ok',
                        'default_value':self.user, 'inline':'ok'}
        #identité
        if datas.has_key('displayName'):name = datas['displayName'][0]
        else:name=''
        user_name = {'name':'displayName', 'libelle':"Identité", 'default_value':name, 'inline':'ok'}
        #civilite
        civilite_opts = [{'name':'1', 'libelle':'Mr.'}, {'name':'2', 'libelle':'Mme'},
                        {'name':'3', 'libelle':'Mlle'}]
        if datas.has_key('codecivilite'):civilite_opts[int(datas['codecivilite'][0])-1]['default'] = 'ok'
        civilite = {'name':'codecivilite', 'options':[{'name':'', 'options':civilite_opts}],
                    'libelle':"Civilité", 'inline':'ok'}
        #edition des mails
        _mail_locale = "%s@%s"%(self.user, backend.get_domains()[1])
        mail = {'libelle':"Adresse e-mail (personalisée <b>ou</b> locale)",
                'radios':[{'name':'mailtype', 'value':'perso','inline':'ok'},
                    {'name':'mailtype', 'value':'local','inline':'ok'}],
                    'radios_libelle':['Adresse personalisée :', 'Adresse locale: <br /> %s'%_mail_locale],
                    'input':{'name':"mailperso", "libelle":''}
                    }
        if datas.has_key('mailDir'):
            if datas['mailDir'][0] != '':
                mail['radios'][1]['checked'] = 'true'
                if datas.has_key('mail'): _mail = datas['mail'][0]
                else: _mail = _mail_locale
            else:
                _mail = "mail non communiqué"
        elif datas.has_key('mail'): #compte mail perso
                if datas['mail'][0] != '':
                    mail['radios'][0]['checked'] = 'true'
                    _mail = datas['mail'][0]
                    mail['input']['default_value'] = _mail
        else:
#            mail['radios'][0]['checked'] = 'true'
            _mail = "mail non communiqué"
        mail['libelle'] = "Adresse e-mail (personalisée <b>ou</b> locale): <br /> <b>%s</b>"%_mail
        #date de naissance
        _date=''
        if datas.has_key('dateNaissance'): _date = backend.formate_date(datas['dateNaissance'][0])
        date = {'name':'dateNaissance', 'libelle':"Date de naissance", 'inline':'ok', "default_value":_date}
        # quota
        _quota = backend.get_quota(self.user)
        quota =  {'libelle':"Quotas disque (0 pour inactif)", 'default_value':_quota, 'name':'quota', 'inline':'ok'}
        # profil
        options = [{'name':'1', 'libelle':'local'}, {'name':'2', 'libelle':"obligatoire - profil"},
                    {'name':'3', 'libelle':"obligatoire - profil2"}]
        profil = backend.get_profil(self.user)
        options[profil-1]['default'] = 'ok'
        profile = {'name':'_sambaProfilePath', 'libelle':"Profil utilisateur",
                'options':[{'name':"Profil disponible", 'options':options}], 'inline':'ok'}
        # shell
        shell = {'name':'shell', 'libelle':"Activation du shell (gestion de clients Linux)", 'inline':'ok', 'value':'shell'}
        if datas.has_key('loginShell') and datas['loginShell'][0] == '/bin/bash':
            shell['checked'] = 'true'
        # administration de classe
        if datas['typeadmin'][0] == '2': classes = backend.get_admin_classes(self.user)
        else: classes = []
        available_classes = backend.get_classes()
        for key, item in available_classes.items():
            if item in classes:available_classes[key].remove(item)
        liste = {'title':'Classes à administrer', 'formid':'admin_class_form','id':'admin_class_table',
                 'datas':classes}
        available = [{'name':level, 'options':[{'name':cl, 'libelle':cl}for cl in _class]}for level,_class in available_classes.items()]
        select = {'name':'admin_class_select',
                'options':available, 'libelle':"classe disponibles", 'id':'admin_class_select'}
        classadmin = {'name':'classadmin', 'btns':self._get_transf_btns('admin_class'), 'select':select,
                        'liste':liste, 'title':"Administration de classes"}
        ## attachement à un groupe
        user_groupes = [gr for gr in backend.get_user_groups(self.user)if gr in backend.list_groups('groupe')]
        groupes_dispo = [gr for gr in backend.list_groups('groupe') if gr not in user_groupes]
        liste = {'title':"Groupes de l'enseignant", 'formid':'groupe_form','id':'groupe_table', 'datas':user_groupes}
        select = {'name':'groupe_select', 'libelle':"Groupes",
                'options':[{'name':'Groupes disponibles','options':[{'name':gr, 'libelle':gr}for gr in groupes_dispo]}],
                'id':'groupe_select' }
        groupe = {'name':"groupe", 'btns':self._get_transf_btns('groupe'), 'select':select,
                'liste':liste, 'title':"Groupes"}
        ## attachement à une matière
        user_matieres = [mat for mat in backend.get_user_groups(self.user)if mat in backend.list_groups('matiere')]
        matieres_dispo = [mat for mat in backend.list_groups('matiere') if mat not in user_matieres]
        liste = {'title':"Matières de l'enseignant", 'formid':'matiere_form','id':'matiere_table', 'datas':user_matieres}
        select = {'name':'matiere_select', 'libelle':"Matières",
                'options':[{'name':'Matières disponibles','options':[{'name':mat, 'libelle':mat}for mat in matieres_dispo]}],
                'id':'matiere_select' }
        matiere = {'name':"matiere", 'btns':self._get_transf_btns('matiere'), 'select':select,
                'liste':liste, 'title':"Matières dispensées"}
        ## attachement à une équipe  pédagogique
        user_equipes = [gr for gr in backend.get_user_groups(self.user)if gr in backend.list_groups('equipe')]
        equipes_dispo = [gr for gr in backend.list_groups('equipe')if gr not in user_equipes]
        liste = {'title':"Equipes pédagogiques de l'enseignant", 'formid':'equipe_form', 'id':'equipe_table', 'datas':user_equipes}
        select = {'name':'equipe_select', 'libelle':"Equipes",
                'options':[{'name':'Equipes disponibles','options':[{'name':gr, 'libelle':gr}for gr in equipes_dispo]}],
                'id':'equipe_select'}
        equipe = {'name':"equipe", 'btns':self._get_transf_btns('equipe'), 'select':select,
                'liste':liste, 'title':"Equipes pédagogiques"}
        ## groupes speciaux
        spec_groupes = backend.list_groups('Administrateurs')
        user_spec_groupes = [gr for gr in backend.get_user_groups(self.user)if gr in spec_groupes]
        spec_groupes = [gr for gr in spec_groupes if gr not in user_spec_groupes]
        liste = {'title':"Groupes spéciaux de l'enseignant", 'formid':'spec_group_form', 'id':'spec_group_table', 'datas':user_spec_groupes}
        select = {'name':'spec_group_select', 'libelle':"Groupes",
                  'options':[{'name':'Groupes disponibles','options':[{'name':gr, 'libelle':gr}for gr in spec_groupes]}],
                  'id':'spec_group_select'}
        spec_group = {'name':"spec_group", 'btns':self._get_transf_btns('spec_group'), 'select':select,
                    'liste':liste, 'title':"Groupes spéciaux"}
        return dict(mod_user_login=user_login,name=user_name, civilite=civilite, mail=mail,
                        profile=profile, date=date, quota=quota, classadmin=classadmin, matiere=matiere,
                        equipe=equipe, groupe=groupe, spec_group=spec_group, shell=shell)

    def _get_pupil_form(self, datas):
        """ renvoie la description du formulaire de modification d'élève """
        if scribe_tools.is_prof_admin(self.user_description['role']): # seul l'admin peut modifier tous les eleves
            prof = self.user_description['name']
            if not scribe_tools.is_eleve_from_prof(self.user, prof):
                return {'message':'Action non autorisée'}
        user_login = {'name':'login', 'libelle':"Identifiant", 'disabled':'ok',
                        'default_value':self.user, 'inline':'ok'}
        if datas.has_key('displayName'):name = datas['displayName'][0]
        else:name=''
        user_name = {'name':'displayName', 'libelle':"Identité", 'default_value':name, 'inline':'ok'}
        civilite_opts = [{'name':'1', 'libelle':'Mr.'}, {'name':'2', 'libelle':'Mme'},
                        {'name':'3', 'libelle':'Mlle'}]
        if datas.has_key('codecivilite'):civilite_opts[int(datas['codecivilite'][0])-1]['default'] = 'ok'
        civilite = {'name':'codecivilite', 'options':[{'name':'', 'options':civilite_opts}],
                    'libelle':"Civilité", 'inline':'ok'}
        #adresse
        adress = "adresse non communiquee"
        if datas.has_key('postalAddress'):
            if datas['postalAddress'] != ['none']:
                adress = datas['postalAddress'][0]
        adresse = {'name':'postalAddress', 'libelle':"Adresse", 'default_value':adress, 'inline':'ok'}
        #mail et domaine
        mail = {'libelle':"Adresse e-mail",'name':"email", 'inline':'ok', 'disabled':'ok'}
        _domains = backend.get_domains()
        dom = ''
        if datas.has_key('mail'):
            mail['default_value'] = datas['mail'][0].split('@')[0]
            dom =  datas['mail'][0].split('@')[1]
            _domains.remove(dom)
        domains = [{'libelle':domain, 'name':domain}for domain in _domains]
        domains.append({'libelle':dom, 'name':dom, 'default':'ok'})
        maildomain = {'name':'maildomain', 'options':[{'name':'', 'options':domains}]}
        #mail parent
        mailparent = [{"name":'mail_parent1', 'libelle':"Mail parent n°1", 'inline':'ok'}]
        if datas.has_key('mailparent'):
            mailparent[0]['default_value'] = datas['mailparent'][0]
            mailparent.append({"name":'mail_parent2', 'libelle':"Mail parent n°2", "inline":"ok"})
            if len(datas['mailparent'])>1:mailparent[1]['default_value'] = datas['mailparent'][1]
        #num ELENOET
        num=''
        if datas.has_key('employeeNumber'): num = datas['employeeNumber'][0]
        user_number = {'name':'employeeNumber', 'libelle':"Numéro de l'élève (ELENOET dans GEP)",'inline':'ok',
                        "default_value":num}
        #date de naissance
        _date=''
        if datas.has_key('dateNaissance'): _date = backend.formate_date(datas['dateNaissance'][0])
        date = {'name':'dateNaissance', 'libelle':"Date de naissance", 'inline':'ok', "default_value":_date,
                '_blur':"testVoid('dateNaissance', 'error_dateNaissance')"}
        #quota
        _quota = backend.get_quota(self.user)
        quota =  {'libelle':"Quotas disque (0 pour inactif)", 'default_value':_quota, 'name':'quota', 'inline':'ok'}
        #profil
        options = [{'name':'1', 'libelle':'local'}, {'name':'2', 'libelle':"obligatoire - profil"},
                    {'name':'3', 'libelle':"obligatoire - profil2"}]
        profil = backend.get_profil(self.user)
        options[profil-1]['default'] = 'ok'
        profile = {'name':'_sambaProfilePath', 'libelle':"Profil utilisateur",
                'options':[{'name':"Profil disponible", 'options':options}], 'inline':'ok'}
        # shell
        shell = {'name':'shell', 'libelle':"Activation du shell (gestion de clients Linux)", 'inline':'ok', 'value':'shell'}
        if datas.has_key('loginShell') and datas['loginShell'][0] == '/bin/bash':
            shell['checked'] = 'true'
        #nivo
        nivo = datas['Meflcf'][0]
        niveau = {'name':'Meflcf', 'libelle':"Niveau", "inline":"ok", "default_value":nivo, 'disabled':'ok'}
        #classe
        _classe = datas['Divcod'][0]
        opts = backend.get_classes()
        #on parcoure les nivo et les classes à la recherche de celle déjà configurée
        classes = []
        for key, items in opts.items():
            nivo = []
            for item in items:
                opt = {'name':item, 'libelle':item}
                if item == _classe:opt['default'] = 'ok'
                nivo.append(opt)
            classes.append({'name':key, 'options':nivo})
        classe = {'name':'_Divcod', 'libelle':"Classe", "inline":"ok", "options":classes}
        ## attachement à un groupe
        user_groupes = [gr for gr in backend.get_user_groups(self.user)if gr in backend.list_groups('groupe')]
        groupes_dispo = [gr for gr in backend.list_groups('groupe') if gr not in user_groupes]
        liste = {'title':"Groupes de l'élève", 'formid':'groupe_form','id':'groupe_table', 'datas':user_groupes}
        select = {'name':'groupe_select', 'libelle':"Groupes",
                'options':[{'name':'Groupes disponibles','options':[{'name':gr, 'libelle':gr}for gr in groupes_dispo]}],
                'id':'groupe_select' }
        groupe = {'name':"groupe", 'btns':self._get_transf_btns('groupe'), 'select':select,
                'liste':liste, 'title':"Groupes"}
        return dict(mod_user_login=user_login,name=user_name, civilite=civilite, maildomain=maildomain,
                    maileleve=mail, mailparent=mailparent, profile=profile, quota=quota, adresse=adresse,
                    user_number=user_number, divcod=classe, niveau=niveau, date=date, groupe=groupe, shell=shell)

### fonction de gestion du retour de formulaire de modification

    def _valid_form(self):
        if self._type == 'teacher':
            return self._modify_teacher()
        else:
            return self._modify_pupil()

    def _modify_teacher(self):
        """ modifie un prof """
        if scribe_tools.is_prof_admin(self.user_description['role']): # un prof_admin ne peut modifier que ses eleves
            return dict(message="Action interdite")
        ## recupération des résultats de formulaire et formatage
        modification = tools.format_form_result(self.form_result['mod_user'])
        self.user = modification['login']
        administration = tools.format_form_result(self.form_result['admin_class_form'],'2' )
        groupe = tools.format_form_result(self.form_result['groupe_form'], '2')
        matiere = tools.format_form_result(self.form_result['matiere_form'], '2')
        equipe = tools.format_form_result(self.form_result['equipe_form'], '2')
        spec_groupes = tools.format_form_result(self.form_result['spec_group_form'], '2')
        ## filtrage pour n'avoir que les données modifiées
        modification = self._filter_teacher_modif(modification)
        administration = self._filter_teacher_admin(administration)
        groupe = self._filter_teacher_groupe(groupe)
        matiere = self._filter_teacher_matiere(matiere)
        equipe = self._filter_teacher_equipe(equipe)
        spec_groupes = self._filter_teacher_spec_groupe(spec_groupes)
        ## application des modifications
        if modification: self._set_teacher_modification(modification)
        self._set_teacher_administration(administration)
        if groupe != ([],[]):self._set_teacher_groupe(groupe)
        if matiere != ([],[]):self._set_teacher_groupe(matiere)
        if equipe!= ([],[]):self._set_teacher_groupe(equipe)
        if spec_groupes != ([],[]):self._set_teacher_groupe(spec_groupes)
        return dict(message="Les modifications ont bien été attribuées à %s"%self.user)

    def _modify_pupil(self):
        """ modifie un élève """
        ## recuperation des valeurs
        modification = tools.format_form_result(self.form_result['mod_user'])
        self.user = modification['login']
        if scribe_tools.is_prof_admin(self.user_description['role']): # un prof_admin ne peut modifier que ses eleves
            prof = self.user_description['name']
            if not scribe_tools.is_eleve_from_prof(self.user, prof):
                return dict(message='Action non autorisée.')
        groupe = tools.format_form_result(self.form_result['groupe_form'], '2')
        ## filtrage des donnees
        modification = self._filter_pupil_modif(modification)
        groupe = self._filter_teacher_groupe(groupe)
        ## application des changements
        if modification: self._set_pupil_modification(modification)
        if groupe != ([],[]):self._set_teacher_groupe(groupe)
        return dict(message="Les modifications ont bien été attribuées à %s"%self.user)

### fonctions de filtrage de données

    def _filter_pupil_modif(self, modif):
        """ filtre les modifications renvoie le dictionnaire de
            modification en supprimant les champs inchangés"""
        datas = self._get_user()
        for key, item in datas.items():
            if key in modif.keys():
                if modif[key] == item[0]:
                    modif.pop(key)
        for key, item in modif.items():
            if key in ['displayName', 'postalAddress']:
                try: modif[key] = modif[key].encode('utf8')
                except: continue

            if key in ['displayName', 'mailperso', 'mail_parent1', 'mail_parent2']:
                if not tools.test_ldap_attribute(item): raise MissingKey, "Erreur : Format d'entrée incorrect %s."%item
        #quota
        quota = backend.get_quota(self.user)
        if quota == modif['quota']:
            modif.pop('quota')
        else:
            if not tools.test_number(modif['quota']):
                raise Exception, "Erreur : Le quota doit être un nombre."
            modif['_quota'] = int(modif['quota'])
            modif.pop('quota')
        # shell
        old_shell = datas.has_key('loginShell') and datas['loginShell'][0] == '/bin/bash'
        if not modif.has_key('shell'):
            modif['shell'] = False
        if modif['shell'] == old_shell:
            modif.pop('shell')
        # numéro élève
        if modif.has_key('employeeNumber'):
            if not tools.test_number(modif['employeeNumber']):
                raise Exception, "Erreur : Le numéro d'élève doit être un nombre."
        # profil
        profil = backend.get_profil(self.user)
        if modif.has_key('_sambaProfilePath'):
            modif['_sambaProfilePath'] = int(modif['_sambaProfilePath'])
            if modif['_sambaProfilePath'] == profil:
                modif.pop('_sambaProfilePath')
        # mail parent
        mailparent = [modif['mail_parent1']]
        if modif.has_key('mail_parent2'):mailparent.append(modif['mail_parent2'])
        if datas.has_key('mailparent'):
            if mailparent == datas['mailparent']:
                modif.pop('mail_parent1')
                modif.pop('mail_parent2')
        # mail et domaine eleve
        domain = modif['maildomain']
        if datas.has_key('mail'):
            if domain == datas['mail'][0].split('@')[1]: modif.pop('maildomain')

        # date de naissance
        date = backend.deformate_date(modif['dateNaissance'])
        if datas.has_key('dateNaissance'):
            if datas['dateNaissance'][0] == date:
                modif.pop('dateNaissance')
        elif not modif['dateNaissance']:
            raise Exception, "Erreur : Il manque la date de naissance."
        elif not tools.test_date(modif['dateNaissance']):
            modif.pop('dateNaissance')
            raise Exception, "Erreur : le format de la date de naissance est incorrect (jj/mm/aaaa)"
        else:
            modif['dateNaissance'] = date
        return modif

    def _filter_teacher_modif(self, modif):
        """ filtre les modifications renvoie le dictionnaire de
            modification en supprimant les champs inchangés"""
        datas = self._get_user()
        for key, item in datas.items():
            if key in modif.keys():
                if modif[key] == item[0]:
                    modif.pop(key)
        for key, item in modif.items():
            if key in ['displayName']:
                try: modif[key] = modif[key].encode('utf8')
                except: continue
            if key in ['displayName', 'mailperso']:
                if not tools.test_ldap_attribute(item): raise MissingKey, "Erreur : Format d'entrée incorrect %s."%item
        # quotas
        quota = backend.get_quota(self.user)
        if quota == modif['quota']:
            modif.pop('quota')
        else:
            if not tools.test_number(modif['quota']):
                raise Exception, "Erreur : Le quota doit être un nombre."
            modif['_quota'] = int(modif['quota'])
            modif.pop('quota')
        # shell
        old_shell = datas.has_key('loginShell') and datas['loginShell'][0] == '/bin/bash'
        if not modif.has_key('shell'):
            modif['shell'] = False
        if modif['shell'] == old_shell:
            modif.pop('shell')
        # profil
        profil = backend.get_profil(self.user)
        if modif.has_key('_sambaProfilePath'):
            modif['_sambaProfilePath'] = int(modif['_sambaProfilePath'])
            if modif['_sambaProfilePath'] == profil:
                modif.pop('_sambaProfilePath')
        ## mail modifies?
        if datas.has_key('mail') and modif.has_key('mailtype'):
            if modif['mailtype'] == 'perso':
                if datas['mail'][0] == modif['mailperso']:
                    modif.pop('mailperso')
                    modif.pop('mailtype')
            else:
                if datas.has_key('mailDir'):
                    modif.pop('mailperso')
                    modif.pop('mailtype')
        # date de naissance
        date = backend.deformate_date(modif['dateNaissance'])
        if datas.has_key('dateNaissance'):
            if datas['dateNaissance'][0] == date:
                modif.pop('dateNaissance')
            else:
                modif['dateNaissance'] = date
        elif not modif['dateNaissance']:
            raise Exception, "Erreur : Il manque la date de naissance."
        elif not tools.test_date(modif['dateNaissance']):
            modif.pop('dateNaissance')
            raise Exception, "Erreur : le format de la date de naissance est incorrect (jj/mm/aaaa)"
        else:
            modif['dateNaissance'] = date
        return modif

    def _filter_teacher_admin(self, admin):
        """ filtre les classes à administrer par l'enseignant
            renvoie le couple nouvelle classe de l'utilisateur et classes à quitter
        """
        return [cl for cl in admin.keys() if cl in backend.list_groups('classe')]

    def _filter_teacher_groupe(self, groupes):
        """ filtre les groupes auxquels l'enseignant appartient """
        user_groupes = [cl for cl in backend.get_user_groups(self.user) if cl in backend.list_groups('groupe')]
        return [cl for cl in groupes.keys() if cl not in user_groupes],[cl for cl in user_groupes if cl not in groupes.keys()]

    def _filter_teacher_matiere(self, matieres):
        """ filtre les matieres que l'enseignant dispense"""
        user_matieres = [cl for cl in backend.get_user_groups(self.user) if cl in backend.list_groups('matiere')]
        return [cl for cl in matieres.keys() if cl not in user_matieres],[cl for cl in user_matieres if cl not in matieres.keys()]

    def _filter_teacher_equipe(self, equipes):
        """ filtre les équipes auxquels l'enseignant appartient"""
        user_equipes = [cl for cl in backend.get_user_groups(self.user) if cl in backend.list_groups('equipe')]
        return [cl for cl in equipes.keys() if cl not in user_equipes],[cl for cl in user_equipes if cl not in equipes.keys()]
    def _filter_teacher_spec_groupe(self, groupes):
        """ filtre les groupes speciaux auxquels l'enseignant appartient """
        user_groupes = [cl for cl in backend.get_user_groups(self.user) if cl in backend.list_groups('Administrateurs')]
        return [cl for cl in groupes.keys() if cl not in user_groupes],[cl for cl in user_groupes if cl not in groupes.keys()]

### fonctions d'application des modifications

    def _set_pupil_modification(self, modif):
        """ modifie les attributs d'un élève"""
        datas = self._get_user()
        # modification des attributs simples
        for key, value in modif.items():
            if key in datas.keys():
                if not backend.set_attr(self.user, key, value):
                    raise MissingKey, "Erreur : Problème de modification de l'attribut %s"%key
            elif key == 'shell':
                if value:
                    backend.set_attr(self.user, 'loginShell', '/bin/bash')
                else:
                    backend.set_attr(self.user, 'loginShell', '/bin/false')
        #gestion des quotas:
        if modif.has_key('_quota'):backend.set_quota(self.user, modif['_quota'])
        # gestion du mail:
        if modif.has_key('maildomain') and modif.has_key('email'):
            mail = "%s@%s"%(modif['email'], modif['maildomain'])
            if datas.has_key('mail'):
                if mail != datas['mail'][0]:
                    #FIXME: attribution du mail à un user à priori c bon???
                    backend.mod_mail(self.user, 'local', mail=mail)
        ## gestion du mail des parents
        parentmail = []
        if modif.has_key('mail_parent1'):
            if '@' not in modif['mail_parent1'] and modif['mail_parent1'] != '':
                raise Exception, "Erreur : l'adresse mail 1 des parents n'est pas valide"
            parentmail.append(modif['mail_parent1'])
        if modif.has_key('mail_parent2'):
            if '@' not in modif['mail_parent2'] and modif['mail_parent2'] != '':
                raise Exception, "Erreur : l'adresse mail 2 des parents n'est pas valide"
            parentmail.append(modif['mail_parent2'])
        if datas.has_key('mailparent'): configured=datas['mailparent']
        else: configured = None
        if '' in parentmail: parentmail.remove('')
        if parentmail != configured: backend.set_attr(self.user, 'mailparent', parentmail)
        ## changement de classe:
        if modif.has_key('_Divcod'):
            if modif['_Divcod'] != datas['Divcod']:
                backend.change_classe(self.user, modif['_Divcod'])
        ## profile
        if modif.has_key('_sambaProfilePath'):
            backend.set_profil(self.user, modif['_sambaProfilePath'])

    def _set_teacher_modification(self, modif):
        """ modifie les attributs d'un prof
            modif: dictionnaire {'clé':valeur} revenant du formulaire puis filtré
        """
        #FIXME: gestion des erreurs
        datas = self._get_user()
        for key, value in modif.items():
            ## on ne modifie que les termes déjà existant
            if key in datas.keys():
                backend.set_attr(self.user, key, value)
            elif key == 'dateNaissance': ## hack pour mettre une date de naissance quand il n'y en a pas encore
                backend.set_attr(self.user, key, value)
            elif key == 'shell':
                if value:
                    backend.set_attr(self.user, 'loginShell', '/bin/bash')
                else:
                    backend.set_attr(self.user, 'loginShell', '/bin/false')

        #gestion des quotas:
        if modif.has_key('_quota'):backend.set_quota(self.user, modif['_quota'])
        ## modification du mail
        #FIXME: attribution du mail à un user???
        if modif.has_key('mailtype'):
            mailtype = modif['mailtype']
            if modif.has_key('mailperso') and mailtype=='perso': mailperso = modif['mailperso']
            else:mailperso = ''
            if mailtype == 'perso' and mailperso == '': raise MissingKey, "Il manque l'adresse mail perso"
            backend.mod_mail(self.user, mailtype, mailperso)
        ## profile
        if modif.has_key('_sambaProfilePath'):
            backend.set_profil(self.user, modif['_sambaProfilePath'])
        ## quota
        if modif.has_key('quota'):
            backend.set_quota(self.user, int(modif['quota']))

    def _set_teacher_administration(self, administration):
        """ enregistre un prof comme administrateur d'une classe
            :administration: liste de classes
            à priori il suffit d'inscrire les groupes pour supprimer ceux qui manquent
        """
        backend.set_admin_classes(self.user, administration)

    def _set_teacher_groupe(self, groupe):
        """ enregistre un prof dans une liste de groupes
            :groupe: [liste de groupes à inscrire, liste de groupe où désinscrire]
        """
        for gr in groupe[0]:
            backend.inscription(self.user, gr)
        for gr in groupe[1]:
            backend.desinscription(self.user, gr)

### description des boutons pour la modification des utilisateurs
    def _get_valid_btn(self):
        """ renvoie la description du bouton valider """
        validate={}
        validate['href'] = tools.make_form_link(self.server_nb, self.name, True,
                    ['spec_group_form', 'mod_user', 'current_type', 'admin_class_form', 'matiere_form', 'equipe_form', 'groupe_form'], 'return_msg_container')
        validate['icone'] = "/image/ok.gif"
        validate['libelle'] = "Valider"
        validate['title'] = "Modifier cet utilisateur"
        return dict(validate=validate)

    def _get_close_btn(self):
        """ renvoie la description du bouton de fermeture de la section modify """
        href = "javascript:setVoid('mod_user_div_container');"
        title = "Fermer la section"
        libelle = "Fermer"
        icone = "/image/supprimer.gif"
        _class = "scribe_close_btn"
        return dict(close_btn=dict(href=href,
                                    title=title,
                                    libelle=libelle,
                                    icone=icone,
                                    _class=_class))


    def _get_transf_btns(self, racine='admin_class'):
        """ renvoit les differents btns de transfert de donnees d'une select à une liste
            :racine:racine des noms html des balises concernées
        """
        href2 = "javascript:formRemoveFromList('%s_form');"%racine
        href1 = "javascript:formAddNameToList3('%s_select','%s_table');"%(racine, racine)
        icone2 = "/image/scribe/rightleft_red.png"
        icone1 = "/image/scribe/leftright.png"
        msg2 = "Retirer"
        msg1 = "Ajouter"
        btns = [{'href':href2, 'libelle':msg2, 'icone':icone2, 'title':msg2},
                {'href':href1, 'libelle':msg1, 'icone':icone1, 'title':msg1}]
        return btns

class UserDelete(Action):
    """ Supprime un utilisateur
        cette action est accessible par prof_admin
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'scribe_user_delete'
    libelle = "Suppression"
    category = None
    description = 'Supprimer un utilisateur'
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend", keys=['server', 'action', 'user'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['user'])

    def execute(self):
        """ supprime un utilisateur (depuis un lien js, pas de formulaire) """
        params, self.server_nb = tools.get_request(self.request)
        self.user = ''
        result = {}
        if params.has_key('user'):
            self.user = params['user'][0].strip()
            try:
                result['message'] = self.del_user()
            except (MissingValue, MissingKey, Exception), mess:
                log.err("Erreur dans user.py : %s"%mess)
                traceback.print_exc()
                result['message'] = mess.message
            result['toexec'] = tools.make_form_link(self.server_nb,
                                                        'scribe_user_table',
                                                        False, ['search'],
                                                        'user_div_container')
        return self.send_all(result, template='scribe_users', templates=used_templates)

    def del_user(self):
        """ supprime un utilisateur """
        if scribe_tools.is_prof_admin(self.user_description['role']): # un prof_admin ne peut modifier que ses eleves
            prof = self.user_description['name']
            if not scribe_tools.is_eleve_from_prof(self.user, prof):
                return "Action non autorisée."
        if self.user == '': raise MissingValue, "Il manque un paramètre, impossible de supprimer"
        if self.user == 'admin': raise Exception, "Il est fortement déconseillé de supprimer admin"
        a = backend.del_user(self.user)
        return "L'utilisateur a bien été supprimé"


class UserModPwd(Action):
    """ Modifie le mot de passe d'un utilisateur
        cette action est accessible par prof_admin et prof donc on a des controles
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'scribe_user_password'
    libelle = "Mot de passe"
    category = None
    description = "Modification du mot de passe d'un utilisateur"
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend", keys=['server', 'action', 'user'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['mod_pwd'])

    def execute(self):
        """ éxécute l'action et renvoie les données pour la mise en forme
            1 - récupère le nom du user dont le mot de passe doit être modifié
            2 - renvoie les données pour le formulaire de modification de mot de passe selon le role de l'appelant
                a - si prof modifie son propre mdp, on doit lui demander l'ancien
                b - si prof_admin modifie le mdp d'un élève, on doit vérifier les droits
                c - sinon c'est un admin qui demande
            3 - modifie le mot de passe
        """
        params, self.server_nb = tools.get_request(self.request)
        self.user = ''
        result = {}
        ## 1 -
        if params.has_key('user'):
            self.user = params['user'][0].strip()
        elif self.form_result != {}:
            ## 3 -
            try:
                result['message'] = self._valid_form()
            except (MissingValue, MissingKey, Exception), mess:
                log.err("Erreur dans user.py : %s"%mess)
                traceback.print_exc()
                result['message'] = mess.message
            self.form_result = {}

        ## 2 - a
        if scribe_tools.is_prof(self.user_description['role']): # c'est un prof
            self.user = self.user_description['name']
            result.update(self._get_form('prof'))
            return self.send_all(result, template='scribe_users', templates=used_templates)
        ## deux possibilités, soit le prof_admin veut changer son mdp, soit il change celui de l'élève
        elif scribe_tools.is_prof_admin(self.user_description['role']):
            prof = self.user_description['name']
            # on controle les droits
            if not scribe_tools.is_eleve_from_prof(self.user, prof) and self.user != prof:
                # si on passe ici c'est qu'on essaye de truander
                log.msg('ALERT : tentative abusive de modification de mot de passe.')
                result.update(dict(message="Action non autorisée."))
            ## 2 - a
            elif prof == self.user:
                result.update(self._get_form('prof'))
            ## 2 - b
            else:
                result.update(self._get_form())
            return self.send_all(result, template='scribe_users', templates=used_templates)

        ## 2 - c
        result.update(self._get_form())
        return self.send_all(result, template='scribe_users', templates=used_templates)

    def _get_form(self, _type=None):
        """ renvoie la description du formulaire d'édition de mot de passe
            si c'est un prof qui modifie son propre mot de passe, on demande l'ancien
        """
        user_name = {'name':'user_name', 'default_value':self.user}
        pwd_input = {'name':'password', 'libelle':"Nouveau mot de passe", "inline":"ok",
                        "onblur":"testVoid('password', 'error_password');"}
        re_pwd_input = {'name':'repassword', 'libelle':"Réentrez le mot de passe", "inline":"ok",
                            "onblur":"testVoid('password', 'error_password');"}
        if _type == 'prof': ## renvoyé en cas de connexion d'un professeur ou un prof admin
            password = {'name':'old_password', 'libelle':'Ancien mot de passe', 'inline':'ok',
                    "onblur":"testVoid('old_password', 'error_old_password');"}
            return dict(user_name=user_name, old_pwd=password, pwd_input=pwd_input,
                            title="Modification de mot de passe pour %s"%self.user,
                            re_pwd_input=re_pwd_input, validate=self._get_validate_btn(),
                            icone="/image/scribe/action_pwd_over.png")
        else:
            return dict(user_name=user_name, pwd_input=pwd_input,
                        title="Modification de mot de passe pour %s"%self.user,
                        re_pwd_input=re_pwd_input, validate=self._get_validate_btn())

    def _get_validate_btn(self):
        """ renvoie la description du bouton valider """
        href = tools.make_form_link(self.server_nb, self.name, True, ['mod_pwd'], balise='mod_user_div_container')
        title = "Modifier le mot de passe"
        libelle = "Valider"
        icone = "/image/ok.gif"
        return dict(href=href, title=title, libelle=libelle, icone=icone)

    def _valid_form(self):
        """ valide la modification de mot de passe
        """
        if not self.form_result.has_key('mod_pwd'):
            raise MissingKey, "Erreur : Il manque des données pour la modification de mot de passe."
        resultat = tools.format_form_result(self.form_result['mod_pwd'])
        if not resultat.has_key('password') or not resultat.has_key('repassword') or not resultat.has_key('user_name'):
            raise MissingKey, "Erreur : Il manque des données pour la modification de mot de passe."
        pwd1 = resultat['password'].strip()
        pwd2 = resultat['repassword'].strip()
        user_name = resultat['user_name'].strip()
        self.user = user_name
        roles = self.user_description['role']
        prof = self.user_description['name']

        # L'utilisateur n'est pas administrateur (prof ou prof_admin)
        if scribe_tools.is_prof_admin(roles):
            if not scribe_tools.is_eleve_from_prof(self.user, prof) and not self.user == prof:
                raise Exception, "Erreur : action non autorisée."
        elif scribe_tools.is_prof(roles):
            if self.user != prof:
                raise Exception, "Erreur : action non autorisée."

        # si un prof change son mdp, il doit connaitre le précédent
        if scribe_tools.is_prof_admin(roles) or scribe_tools.is_prof(roles):
            if self.user == prof:
                if not resultat.has_key('old_password'):
                    raise MissingKey, "Erreur : Il manque des données pour la modification de votre mot de passe."
                old_pwd = resultat['old_password']
                if not backend.authenticate(user_name, old_pwd):
                    raise MissingKey, "Erreur : erreur d'authentification"

        # modification du mot de passe
        if pwd1 != pwd2: raise MissingKey, "Erreur : Les mots de passe ne correspondent pas"
        if not backend.mod_password(user_name, pwd1):
            raise MissingKey, "Erreur : erreur lors de la modification du mot de passe de %s"%user_name
        return "Le mot de passe de %s a bien été modifié"%user_name
