# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# Action ServiceRestart et ServiceStop 
#  
# Redémarre et stoppe des services
#       
###########################################################################
"""
Actions spécifiques au services qui tournent sur le serveur
"""
from twisted.python import log
from ead2.backend.lib.action import *
from ead2.lib.libbackend import clean_color_codes    
from ead2.backend.actions.tools import get_request, make_confirm_link, make_js_link
from creole import fonctionseole

# liste des services gérés par le module
list_services = fonctionseole.get_services_actifs()

used_templates = ['main']

def get_retour_btn():
    """ représentation de l'affichage du bouton "retour" """
    return {'href': "javascript:Refresh()", 'libelle': "Revenir à la page d'accueil", 'icone': "/image/back.png"}

class Daemon(Action):
    """ Arrete ou redemarre un demon """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'daemon'
    libelle = "Services (mode expert)"
    description = "Stopper ou redémarrer un service"
    category = "Système"
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend", 
                        keys=['server', 'action', 'service_restart', 'service_stop', 'all_restart'])

    def execute(self):
        params, self.server_nb = get_request(self.request)
        message = ''
        result = {'titre':self.description}
        if params.has_key('service_restart'):
            message = self.restart(params['service_restart'][0])
        elif params.has_key('service_stop'):
            message = self.stop(params['service_stop'][0])
        elif params.has_key('all_restart'):
            message = self.restart(list_services)
        result['message'] = message
        result['all_restart'] = self._get_all_restart_btn()
        result['services'] = self._get_services()
        result['retour_btn']  = get_retour_btn()
        return self.send_all(result, template='services', templates=used_templates)

    def _get_all_restart_btn(self):
        """ renvoie la description du bouton de redémarrage de tous les services """
        jsl = make_js_link(self.server_nb, self.name, confirm=True, all_restart='true')
        rest_link = {'href':make_confirm_link("Etes-vous sûr de vouloir redémarrer tous les services (sauf ead2) ?", jsl),
                                    'libelle':'Redémarrer', 'icone': '/image/redemarrer_serv.gif'}
        return {"restart":rest_link, 'libelle':"Redémarrer tous les services (sauf ead2)"}

    def _get_services(self):
        """renvoit la liste des services et la description des boutons pour les redémarrer """
        result = []
        for serv in list_services:
            jsl = make_js_link(self.server_nb, self.name, confirm=True, service_restart=serv)
            rest_link = {'href':make_confirm_link("Etes-vous sûr de vouloir redémarrer ce service ?", jsl),
                    'libelle':'Redémarrer', 'icone': '/image/redemarrer_serv.gif'}
            jsl = make_js_link(self.server_nb, self.name, confirm=True, service_stop=serv)
            stop_link = {'href':make_confirm_link("Etes-vous sûr de vouloir stopper ce service ?", jsl),
                    'libelle':'Stopper', 'icone': '/image/arreter_serv.gif'}
            libelle = serv
            result.append({'restart':rest_link, 'stop':stop_link, 'libelle':libelle})
        return result

    def stop(self, service):
        log.msg('On arrete %s' % service)
        if service not in list_services:
            return "Service inconnu"
        return_code, output = tools.command_statusoutput("/etc/init.d/%s stop" % service)
        if return_code >= 256:
            return_code /= 256
        output = clean_color_codes(output)
        if return_code > 0:
            log.err("Erreur lors du lancement de '/etc/init.d/%s stop'" % service)
            return "Erreur lors de l'arrêt du service : %s" % service
        else:
            return "Le service a été correctement stoppé : %s" % service

    def restart(self, service):
        log.msg('On redemmarre %s'%service)
        if type(service) == list:
            fonctionseole.restart_services(ead2=True)
            return "Tous les services (sauf ead-server et ead-web) ont été redémarrés"
        if service not in list_services: return "Service inconnu"
        return_code, output = tools.command_statusoutput("/etc/init.d/%s restart" % service)
        if return_code >= 256:
            return_code /= 256
        output = clean_color_codes(output)
        if return_code > 0:
            log.err("Erreur lors du lancement de '/etc/init.d/%s restart'" % service)
            return "Erreur lors de l'arrêt du service : %s" % service
        else:
            return "Le service a été correctement redémarré : %s" % service
