# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#    
# Redémarre et stoppe des services depuis une liste simplifiée
#       
###########################################################################
"""
Redémarrage et arrêt des services
"""
from twisted.python import log
from ead2.backend.lib.action import *
#import utiles pour l'execution
from ead2.backend.actions import tools
from ead2.backend.config.filenames import simple_services as services_list
from ead2.backend.actions.lib import simple_services_tools as ss_tools

used_templates = ['main', 'form', 'listlist', 'checklist', 'formselect']

class SimpleServices(Action):
    """ page de redémarrage simplifiée de services """
    user_description = Dict(default={}, doc="Description de l'executant", keys=['name', 'role', 'ip'])
    name="asimple_services"
    libelle="Services (mode normal)"
    description="Redémarrage simplifié de services"
    category = "Système"
    request = Dict(default={}, doc="Arguments de la requête en cours côté frontend",
                        keys=['server', 'action', 'restart', 'stop'])

    def execute(self):
        params, self.server_nb = tools.get_request(self.request)
        if params.has_key('restart') or params.has_key('stop'):
            try:
                if params.has_key('restart'):
                    datas = self._restart_services(ss_tools.get_group_services(params['restart'][0]))
                else:
                    datas = self._stop_services(ss_tools.get_group_services(params['stop'][0]))
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name, confirm=True)
            except Exception, mess:
                datas = dict(message=mess.message)
                
            return self.send_frag(datas, template="simple_services", templates=used_templates)


        result={'titre':self.description}
        result.update(self._get_services_btns())
        return self.send_all(result=result, template="simple_services", templates=used_templates)

    def _get_services_btns(self):
        """ récupère la liste des services et renvoie les boutons de redémarrage """
        groups = ss_tools.get_groups()
        if groups == []:
            services = "Aucun groupe de service n'a été défini."
        else:
            services = []
            for group in groups:
                group_services = ss_tools.get_group_services(group)
                entete = "%s %s"%(group, ' '.join(group_services))
                reboot_btn = {'href':tools.make_js_link(self.server_nb, self.name, 
                                                        restart=group), 
                              'libelle':'Redémarrer',
                              'title':'Redémarrer %s'%entete,
                              'icone':'',
                              '_class':''}
                stop_btn = {'href':tools.make_js_link(self.server_nb, self.name, 
                                                        stop=group),
                            'libelle':'Arrêter',
                            'title':'Arrêter %s'%entete,
                            'icone':'',
                            '_class':''}
                services.append((group,reboot_btn, stop_btn))
        return dict(services=services)

    def _restart_services(self, services):
        """ redémarre des services """
        if services is None:
            raise Exception, "La liste des services à redémarrer est vide."
        message = "Redémarrage de services \\n"
        for service in services:
            a, b = tools.command_statusoutput("/etc/init.d/%s restart"%service)
            if a:
                b = ''.join(b.splitlines())
                message += "Erreur au redémarrage %s : %s\\n"%(service, b)
            else:
                message += "Redémarrage de %s réussi.\\n"%(service)
        return dict(message=message)

    def _stop_services(self, services):
        """ stoppe des services """
        if services is None:
            raise Exception, "La liste des services à arrêter est vide."
        message = "Arrêt de services \\n"
        for service in services:
            a, b = tools.command_statusoutput("/etc/init.d/%s stop"%service)
            if a:
                b = ''.join(b.splitlines())
                message += "Le service %s ne s'est pas arrêté, une erreur est survenue : %s \\n"%(service, b)
            else:
                message += "Le service %s a été arrêté.\\n"%(service)
        return dict(message=message)

class SimpleServicesEditor(Action):
    """ editeur de liste de services """
    user_description = Dict(default={}, doc="Description de l'executant", keys=['name', 'role', 'ip'])
    name="esimple_services_editor"
    libelle="Editeur de services"
    description="Gestion des services pour l'interface simplifiée"
    category = "Système"
    request = Dict(default={}, doc="Arguments de la requête en cours côté frontend",
                        keys=['server', 'action', 'edit', 'delete'])
    form_result = Dict(default={}, doc="Validation de formulaire",
                        keys=['group_name_form', 'available_services', 'group_services'])

    def execute(self):
        params, self.server_nb = tools.get_request(self.request)

        if self.form_result != {}:
            try:
                datas = self.edit_group()
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name, confirm=True)
            except Exception, mess:
                log.err("Erreur dans simple_services.py : %s"%mess)
                datas = {'message':mess.message}
            self.form_result = {}
            return self.send_frag(datas, template='simple_services_editor', templates=used_templates)
        
        if params.has_key('delete'):
            group_name = params['delete'][0]
            ss_tools.del_service_group(group_name)
            datas = dict(message="Le groupe %s a bien été supprimé"%group_name)
            datas['toexec'] = tools.make_js_link(self.server_nb, self.name, confirm=True)
            return self.send_frag(datas, template='simple_services_editor', templates=used_templates)

        if params.has_key('edit'):
            self.group_name = params['edit'][0]
            datas = self._get_form()
            return self.send_frag(datas, template='simple_services_editor', templates=used_templates)

        result = {'titre': 'Edition de groupes de services'}
        result.update(self._get_menu_btns())
        result.update(self._get_main_msg())
        return self.send_all(result, template='simple_services_editor', templates=used_templates)

    def _get_main_msg(self):
        """ renvoie le texte d'entrée """
        return dict(contenu="<b>OUTIL DE CRÉATION DE GROUPE DE SERVICES</b><br /> Cet outil vous permet de créer des groupes de services accessibles pour les utilisateurs ayant les droits sur l'action de redémarrage/arrêt des services (mode normal)")

    def _get_form(self):
        """ renvoie la description du formulaire de gestion des groupes de services """
        group_name = {'name':'group_name',
                            'libelle':"Intitulé du nouveau groupe de service",
                            'onblur':"testVoid('group_name', 'error_group_name');"
                       }
        if self.group_name != 'new':
            group_name['disabled'] = 'ok'
            group_name['default_value'] = self.group_name
            group_name.pop('onblur')

        if self.group_name == 'new':
            service_list = ss_tools.get_available_services()
        else:
            service_list = ss_tools.get_group_available_services(self.group_name)
        group_service_list = ss_tools.get_group_services(self.group_name)

        left = {'form':'available_services',
                'datas':[{'name':service}for service in service_list],
                'title':"Services disponibles"}
        right = {'form':'group_services', 
                'datas':[{'name':service}for service in group_service_list],
                 'title':'Services associés au groupe'}
        title = "Association de services au groupe."
        btns = self._get_transf_btns()
        return dict(group_name=group_name,
                        services=dict(left=left, 
                                        right=right, 
                                        title=title, 
                                        btns=btns,
                                        name='simple_services_editor_listlist'
                                        ),
                        validate=self._get_validate_btn()
                        )               

    def _get_transf_btns(self):
        """ renvoit les btns de transfert de donnees liste a liste pour les services"""
        href1 = "javascript:formTransferListElement('group_services','available_services_table');"
        href2 = "javascript:formTransferListElement('available_services','group_services_table');"
        icone1 = "/image/rightleft_red.png"
        icone2 = "/image/leftright.png"
        msg1 = "Retirer"
        msg2 = "Ajouter"
        btns = [{'href':href2, 'libelle':msg2, 'icone':icone2, 'title':msg2},
                {'href':href1, 'libelle':msg1, 'icone':icone1, 'title':msg1}]
        return btns

    def _get_menu_btns(self):
        """ renvoie la description des boutons de menu (liste des groupes et bouton nouveau groupe """
        new_group_btn = dict(href=tools.make_js_link(self.server_nb, self.name, balise='simple_services_editor_workspace', edit='new'),
                     title="Créer un nouveau groupe",
                     icone="/image/groupe_machine.png",
                     libelle="<b>Créer groupe</b>",
                     _class='btn_groupe_machine')
        
        btns = []
        for group in ss_tools.get_groups():
            btn = (dict(href=tools.make_js_link(self.server_nb, self.name, balise='simple_services_editor_workspace', edit=group),
                       title="Editer le groupe de services %s"%group,
                       icone="",
                       libelle="<b>%s</b><br />%s"%(group, '<br />'.join(ss_tools.get_group_services(group))),
                       _class="simple_services_mod"),
                   dict(href=tools.make_js_link(self.server_nb, self.name, balise='simple_services_editor_workspace', delete=group),
                       title="Supprimer le groupe de services %s"%group,
                       icone="/image/supprimer.gif",
                       libelle="",
                       _class="simple_services_btn_suppr")
                   )
            btns.append(btn)
        return dict(menu=btns, new_group_btn=new_group_btn)

    def _get_validate_btn(self):
        href = tools.make_form_link(self.server_nb, self.name, True, 
                                    ['group_name_form', 'group_services'], 
                                    'simple_services_editor_msg'
                                   )
        title = 'Créer/Modifier le groupe et associer les services'
        icone = '/image/ok.gif'
        if self.group_name == 'new':
            libelle = "Créer"
        else:
            libelle = "Modifier"
        return dict(href=href, title=title, icone=icone, libelle=libelle)

    def edit_group(self):
        """ edite le fichier de groupe de services """
        result = tools.format_form_result(self.form_result['group_name_form'])
        self.group_name = result.values()[0]
        group_services = [service for service in tools.format_form_result(self.form_result['group_services'], '2').keys()]
        if self.group_name not in ss_tools.get_groups():
            message = "Création du groupe de services %s\\n"%self.group_name
            ss_tools.create_service_group(self.group_name, group_services)
        else:
            message = "Modification du groupe de services %s\\n"%self.group_name
            ss_tools.mod_service_group(self.group_name, group_services)
        message += "Les services associés :\\n"
        for serv in group_services:
            message += "%s\\n"%serv
        return dict(message=message)
