# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# Action Status
#
# Page d'accueil d'un serveur de commande
#
###########################################################################

""" Actions affichant les différents statuts du serveur """

from ead2.backend.lib.action import *
from ead2.backend.actions.tools import get_request, make_js_link
import commands, xmlrpclib, socket
from os.path import isfile
from ead2.backend.config.config import AGENT_PORT, AGENT_PROTOCOL, module
from ead2.backend.config.filenames import bacula, maj, index, rapport_extraction, liste_comptes
from ead2.frontend.web.config import ACTION_SOCKET_TIMEOUT
from ead2.config.config import ip_locale
from zephir.monitor.agentmanager import config as config_agent #id_serveur
from creole import fonctionseole

save_report_file = bacula['rapport']
simple_save_report_file = bacula['rapport_simple']
update_report_file = maj['rapport']
blacklist_report_file = index['blacklist']
extraction_report_file = rapport_extraction
extraction_account_list = liste_comptes

as_save_module = ['horus', 'scribe']
as_blacklist_module = ['amon', 'amonecole']
as_extract_module = ['scribe']

# liste des services gérés par le module
list_services = fonctionseole.get_services_actifs()

class Status(Action):
    """ Action de presentation du statut du serveur
        page d'acceuil du serveur
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = 'main_status'
    libelle = 'Accueil'
    category = ""
    description = "Le statut actuel de votre serveur"
    request = Dict(default={}, doc="arguments de la requete en cours cote frontend",
                                keys=['server', 'action', 'aff_maj', 'aff_save', 'aff_blacklist', 'aff_extract',
                                     'aff_account_list', 'url'])


    def execute(self):
        """ Renvoie les données pour l'affichage
            1 - Renvoie les données pour la mise en page des résumés de rapports de sauvegarde,
                                            mise à jour, mise à jour de blacklist
            2 - Renvoie les données pour l'affichage complet de rapport
        """
        params, self.server_nb = get_request(self.request)
        print "STATUS PARAMS : %s"%params
        self.role = self.user_description['role']
        if 'admin' in self.role:
            if params.has_key('aff_maj'):
                ## 2 -
                if params['aff_maj'][0] == 'true':
                    datas = self.get_update_report(True)
                ## 1 -
                else:
                    datas = {'report':{'btn':self._get_update_btn(False), 'texte':''}}
                return self.frag(datas)
            if params.has_key('aff_extract'):
                ## 2 -
                if params['aff_extract'][0] == 'true':
                    datas = self.get_extract_report(True)
                ## 1 -
                else:
                    datas = {'report':{'btn':self._get_extract_btn(False), 'texte':''}}
                return self.frag(datas)
            if params.has_key('aff_account_list'):
                ## 2 -
                if params['aff_account_list'][0] == 'true':
                    datas = self.get_account_list(True)
                ## 1 -
                else:
                    datas = {'report':{'btn':self._get_account_list_btn(False), 'texte':''}}
                return self.frag(datas)
            if params.has_key('aff_save'):
                ## 2 -
                if params['aff_save'][0] == 'true':
                    datas = self.get_save_report(True)
                ## 1 -
                else:
                    datas = {'report':{'btn':self._get_save_btn(False), 'texte':''}}
                return self.frag(datas)
            if params.has_key('aff_blacklist'):
                ## 2 -
                if params['aff_blacklist'][0] == 'true':
                    datas = self.get_blacklist_report(True)
                ## 1 -
                else:
                    datas = {'report':{'btn':self._get_blacklist_btn(False), 'texte':''}}
                return self.frag(datas)
        ## 1 -
        result = {}
#        result['entete'] = "Vous êtes connecté comme %s."%self.user_description['role']
        if 'admin' in self.role:
            result.update(self.get_update_report())
            if module in as_extract_module:
                result.update(self.get_extract_report())
                result.update(self.get_account_list())
            if module in as_save_module:
                result.update(self.get_save_report())
            if module in as_blacklist_module:
                result.update(self.get_blacklist_report())
        result.update(self.get_services_status())
        return self.send_all(result, template='index', templates=['main'])

    def frag(self, datas):
        return self.send_frag(datas, template='index', templates=['main'])

    def _get_save_btn(self, aff=False):
        """ renvoie la description d'affichage/masquage des rapports complets de sauvegarde """
        if aff:
            href = make_js_link(self.server_nb, self.name, aff_save='false', balise="div_save_container")
            href += "window.scrollTo(0,0);"
            return {'href':href,
                    'icone':'/image/reduce.jpg', 'libelle':'Masquer le rapport',
                    'title':"Masquer le rapport de sauvegarde", '_class':'btn'}
        else:
            return {'href':make_js_link(self.server_nb, self.name, aff_save='true', balise="div_save_container"),
                    'icone':'/image/increase.jpg', 'libelle':'Afficher le rapport',
                    'title':"Afficher le rapport de sauvegarde", '_class':'btn'}

    def _get_update_btn(self, aff=False):
        """ renvoie la description d'affichage/masquage des rapports complets de mise à jour """
        if aff:
            href = make_js_link(self.server_nb, self.name, aff_maj='false', balise="div_maj_container")
            href += "window.scrollTo(0,0);"
            return {'href':href,
                    'icone':'/image/reduce.jpg', 'libelle':'Masquer le rapport',
                    'title':'Masquer le rapport de mise à jour', '_class':'btn'}
        else:
            return {'href':make_js_link(self.server_nb, self.name, aff_maj='true', balise="div_maj_container"),
                    'icone':'/image/increase.jpg', 'libelle':'Afficher le rapport',
                    'title':'Afficher le rapport de mise à jour', '_class':'btn'}

    def _get_blacklist_btn(self, aff=False):
        """ renvoie la description d'affichage/masquage des rapports complets de mise à jour de blacklists """
        if aff:
            href = make_js_link(self.server_nb, self.name, aff_blacklist='false', balise="div_blacklist_container")
            href += "window.scrollTo(0,0);"
            return {'href':href,
                    'icone':'/image/reduce.jpg', 'libelle':'Masquer le rapport',
                    'title':'Masquer le rapport', "_class":"btn"}
        else:
            return {'href':make_js_link(self.server_nb, self.name, aff_blacklist='true', balise="div_blacklist_container"),
                    'icone':'/image/increase.jpg', 'libelle':'Afficher le rapport',
                    'title':'Afficher le rapport', "_class":"btn"}

    def _get_extract_btn(self, aff=False):
        """ renvoie la description d'affichage/masquage des rapports complets d'extraction """
        if aff:
            href = make_js_link(self.server_nb, self.name, aff_extract='false', balise="div_extract_container")
            href += "window.scrollTo(0,0);"
            return {'href':href,
                    'icone':'/image/reduce.jpg', 'libelle':'Masquer le rapport', 'title':'Masquer le rapport',
                    '_class':'btn'}
        else:
            return {'href':make_js_link(self.server_nb, self.name, aff_extract='true', balise="div_extract_container"),
                    'icone':'/image/increase.jpg', 'libelle':'Afficher le rapport', 'title':'Masquer le rapport',
                    '_class':'btn'}

    def _get_account_list_btn(self, aff=False):
        """ renvoie la description d'affichage/masquage de la liste des comptes """
        if aff:
            return {'href':"javascript:OuvreFenetre('https://%s:8501/extraction/tmp/info.csv', 'liste des comptes')" % ip_locale,
                   'icone':'/image/reduce.jpg',
                   'libelle':'Masquer',
                   'title':'Lister les comptes utilisateurs extraits',
                   '_class':'btn'}
        else:
            return {'href':"javascript:OuvreFenetre('https://%s:8501/extraction/tmp/info.csv', 'liste des comptes')" % ip_locale,
                    'icone':'/image/increase.jpg',
                    'libelle':'Lister les comptes utilisateurs',
                    'title':'Lister les comptes utilisateurs extraits',
                    '_class':'btn'}

    def get_services_status(self):
        """ renvoie les données pour la mise en page des status des services
            - les statuts des services
            - la description des boutons de connexion
        """
        result = {}
        if AGENT_PORT is not None:
            serv = xmlrpclib.ServerProxy('%s://localhost:%s/xmlrpc/' % (AGENT_PROTOCOL, AGENT_PORT))
            try:
                socket.setdefaulttimeout(ACTION_SOCKET_TIMEOUT)
                result = serv.status_for_agents()
                _menu_services = serv.agents_menu()
            except:
                result = {}
                _menu_services = {}
            menu_services = {}
            for key, items in _menu_services.items():
                cle = key.encode('utf-8')
                options = [[item[0].encode('utf-8'), item[1].encode('utf-8')]for item in items]
                main_status = 'OK'
                for item in options:
                    if result.has_key(item[0]):
                        datas = {}
                        if result[item[0]].has_key('message'):
                            datas['message'] = result[item[0]]['message'].encode('utf-8')
                        datas['level'] = result[item[0]]['level'].encode('utf-8')
                        if datas['level'] == 'Warn':
                            main_status = 'Warn'
                        elif datas['level'] != 'OK' :
                            main_status = 'NO'
                        reload(config_agent)
#                        # on reconstruit l'url
#                        ip_client = self.url.split('/')[2]
#                        ip_client = ip_client.split(':')[0]
                        url = '%s://%s:%s/agents/%s/%s/' % (AGENT_PROTOCOL, ip_locale,
                                                          AGENT_PORT, config_agent.id_serveur,
                                                          item[0])
                        if 'admin' in self.role:
                            datas['href'] = make_js_link(self.server_nb, 'help', agent=url)
                        else:
                            datas['href'] = ''
                        item.append(datas)
                    else:
                        items.remove(item)
                menu_services[cle] = {'main_status':main_status, 'options':options}
            return dict(services=menu_services)
        else:
            for service in list_services:
                a = commands.getoutput('/etc/init.d/%s | grep -q status 2>/dev/null' % service)
                b = commands.getoutput('echo $?')
                if b == '0':
                    a, b = commands.getstatusoutput('/etc/init.d/%s status'%service)
                    if not a:
                        result[service] = {'level':'OK' }
                    else:
                        result[service] = {'level':'NO'}
            return dict(result)

    def get_save_report(self, aff=False):
        """ renvoie le rapport des dernières sauvegarde """
        if not isfile(save_report_file) or not isfile(simple_save_report_file):
            return dict(sauvegarde={'title':"Sauvegarde", 'texte': ["Dernière sauvegarde : ", 'Aucune sauvegarde'],
                          'btn':'', 'diode':''})
        else:
            if not aff:
                a = file(simple_save_report_file, 'r')
                report = a.readlines()
                if len(report)>=1:
                    report = report[0]
                report = ["Dernière sauvegarde : ", report]
                a.close()
                return dict(sauvegarde={'title':"Sauvegarde", 'texte':report, 'btn':self._get_save_btn(aff)})
            else:
                a = file(save_report_file, 'r')
                report = a.readlines()
                a.close()
                return dict(report={'btn':self._get_save_btn(aff), 'texte':report})

    def get_update_report(self, aff=False):
        """ renvoie le rapport des dernières mise à jour """
        if not isfile(update_report_file):
            return dict(maj={'title':"Mise à jour", 'texte': ["Dernière mise à jour : ", 'Aucune mise à jour'],
                          'btn':'', 'diode':''})
        else:
            a = file(update_report_file, 'r')
            if not aff:
                ## la mise a jour a-t-elle reussie?
                if 'Mise à jour OK' in a.read():
                    diode = 'OK'
                else:
                    diode = 'No'
                a.seek(0)
                report = a.readlines()
                if len(report)>=1:report = report[0]
                report = ["Dernière mise à jour : ", report]
                a.close()
                return dict(maj={'title':"Mise à jour", 'texte':report, 'btn':self._get_update_btn(aff), 'diode':diode})
            else:
                report = a.readlines()
                a.close()
                return dict(report={'btn':self._get_update_btn(aff), 'texte':report})

    def get_extract_report(self, aff=False):
        """ renvoie le rapport de la dernière extraction """
        if not isfile(extraction_report_file):
            return dict(extract={'title':"Extraction", 'texte': ["Dernière extraction : ", 'Aucune extraction'],
                          'btn':'', 'diode':''})
        else:
            a = file(extraction_report_file, 'r')
            if not aff:
                report = a.readlines()
                if len(report)>=1:
                    report = report[0]
                report = ["Dernière extraction : ", report]
                a.close()
                return dict(extract={'title':"Extraction", 'texte':report, 'btn':self._get_extract_btn(aff)})
            else:
                report = a.readlines()
                a.close()
                return dict(report={'btn':self._get_extract_btn(aff), 'texte':report})

    def get_account_list(self, aff=False):
        """ renvoie la liste des comptes créés suite à une extraction """
        if not isfile(extraction_account_list):
            return dict()
        else:
            if not aff:
                return dict(account_list={'btn':self._get_account_list_btn(aff), 'texte':''})
            else:
                a = file(extraction_account_list, 'r')
                counts = a.readlines()
                a.close()
                return dict(report={'btn':self._get_account_list_btn(aff), 'texte':counts})

    def get_blacklist_report(self, aff=False):
        """ renvoie le rapport des dernières mis a jour de la blacklist """
        report = ["Aucune mise à jour de la liste de sites interdits n'a été effectuée"]
        if not isfile(blacklist_report_file):
            return dict(blacklist={'title':"Liste de sites interdits",
                'texte': ["Dernière Mise à jour de la liste de sites interdits : ", 'Aucune mise à jour'],
                          'btn':'', 'diode':''})
        else:
            a = file(blacklist_report_file, 'r')
            if not aff:
                report = a.readlines()
                if len(report)>=1:
                    report = report[0]
                report = ["Dernière mise à jour : ", report]
                a.close()
                return dict(blacklist={'title':"Liste de sites interdits",
                    'texte':report, 'btn':self._get_blacklist_btn(aff)})
            else:
                report = a.readlines()
                a.close()
                return dict(report={'btn':self._get_blacklist_btn(aff), 'texte':report})
