#!/usr/bin/env python
# -*- coding: UTF-8 -*-

#imports python standard
from os.path import join, dirname, basename, isfile
import glob, re
import sha
import imp
import os, time, traceback
import xmlrpclib, socket, fcntl
import struct, array, glob
from sets import Set

# imports twisted
from twisted.web import server
from twisted.internet import reactor, defer
from twisted.web.xmlrpc import XMLRPC
from twisted.python import log

# imports ead2
from ead2.backend.lib.action import get_actions, get_action, get_full_actions,get_full_registered_actions
# from ead2.backend.lib import error
from ead2.lib.error import *
from ead2.lib.libbackend import PamAuth
from ead2.lib.libead import EadKeyParser
from ead2.backend.lib.perm.parser import parse_and_update, parse_and_update_local, RoleParser
from ead2.backend.lib.perm.registry import PermissionManager
from ead2.lib.libsecure import TransportEole, ServerContextFactory
#fichier de config specifique au backend
from ead2.backend.config.config import NOBODY_UID, NOBODY_GID, PASSWD_FILE, \
     FRONTEND_KEYS_FILE,CONFIG_DIR,ACTION_DIR, TEMPLATE_DIR
#fichier de config de l'EAD, contenant les infos importantes sur les tickets de session du sso
from ead2.config.config import AUTH_SERVER, MAGIC_NUMBER_TIMEOUT, EXPIRED_SESSION_TIMEOUT,BACKEND_LISTEN_PORT
# imports unicode ead
from ead2.lib.i18n_backend import _
from ead2.lib.libead import uni
from creole.fonctionseole import get_module_name

try:
    from creole.fonctionseole import get_module_name
    module_name = get_module_name()
except:
    module_name = ""

Fault = xmlrpclib.Fault

def load_actions():
    """charge les actions depuis le répertoire ACTION_DIR
    """
    # lecture des actions à charger
    fic_actions = ['actions.cfg']
    if isfile(join(CONFIG_DIR, 'actions_%s.cfg' % module_name)):
        fic_actions.append('actions_%s.cfg' % module_name)
    if isfile(join(CONFIG_DIR, 'actions_local.cfg')):
        fic_actions.append('actions_local.cfg')
    # liste des actions pour ce serveur
    actions_files = []
    for fic_action in fic_actions:
        list_f = file(join(CONFIG_DIR, fic_action))
        for action in list_f.readlines():
            if not action.startswith('#'):
                if isfile(join(ACTION_DIR,"%s.py" % action.strip())):
                    actions_files.append(action.strip())
    # chargement des actions de la liste
    for action_file in actions_files:
        try:
            log.msg("loading from module %s :" % action_file)
            filename, pathname, desc = imp.find_module(action_file,[ACTION_DIR])
            imp.load_module(action_file, filename, pathname, desc)
        except Exception, msg:
            log.msg("Error while loading the action file \"%s\": %s" % (action_file, msg))

def create_magicnb(cle1, cle2):
    """ génère une clé aléatoire
    """
    quote = '%s%s%s' % (int(time.time()), cle1, cle2)
    return sha.sha(quote).hexdigest()

class Amon(XMLRPC):
    """Serveur XMLRPC (backend Eole)
    """
    def __init__(self):

        #sauvegarde de notre UIG/GID courant
        self.UID = os.getuid()
        self.GID = os.getgid()
        # dictionnaire des clés de connexion autorisées
        self.frontend_key_dict = {}
        # dictionnaire des sessions actives
        self.magic_number_dict = {}
        # liste des sessions expirées
        self.expired_magic_number = []
        # chargement des actions
        load_actions()
        ACTION_LIST = get_full_actions()
        self.auth_server = xmlrpclib.ServerProxy(AUTH_SERVER, transport=TransportEole())
        self.perm_manager = perm_manager = PermissionManager(ACTION_LIST)
        # chargement des permissions sur les actions
        parse_and_update(perm_manager, join(CONFIG_DIR, 'perm.ini'))
        # self.trusted_clients = parser.get_trusted_clients()
        self.role_parser = RoleParser()
        self.key_parser = EadKeyParser()
        self._languages = {}
        # chargement des rôles par utilisateur
        self.role_parser.parse_file(join(CONFIG_DIR, 'roles.ini'))
        # configuration par module
        if module_name != "":
            if isfile(join(CONFIG_DIR, 'perm_%s.ini' % module_name)):
                parse_and_update(perm_manager, join(CONFIG_DIR, 'perm_%s.ini' % module_name))
            if isfile(join(CONFIG_DIR, 'roles_%s.ini' % module_name)):
                self.role_parser.parse_file(join(CONFIG_DIR, 'roles_%s.ini' % module_name))
        if isfile(join(CONFIG_DIR, 'perm_local.ini')):
           parse_and_update_local(perm_manager, join(CONFIG_DIR, 'perm_local.ini'), join(CONFIG_DIR, 'unavailable_actions.txt'))
        if isfile(join(CONFIG_DIR, 'roles_local.ini')):
            self.role_parser.parse_file(join(CONFIG_DIR, 'roles_local.ini'))
        self.allowed_users = self.role_parser.get_users()
        self.key_parser.parse_file(join(CONFIG_DIR, FRONTEND_KEYS_FILE))
        self.frontend_key_dict = self.key_parser.get_key_dict()
        #drop de privileges vers nobody/nobody
        self._drop_privileges()
        #initialisation de l'authentification locale (PAM)
        self.local_auth = PamAuth()
        XMLRPC.__init__(self)
        #liste des adresses IP de la machine
        self.ip_list = self._get_interface_list()

    def _drop_privileges(self):
        #drop de privileges vers nobody/nobody
        os.setegid(NOBODY_GID)
        os.seteuid(NOBODY_UID)

    def _gain_privileges(self):
        #on reprend nos privileges initiaux
        os.seteuid(self.UID)
        os.setegid(self.GID)

    def _get_ip_address(self, ifname):
        """
        retourne l'adresse IP associee a l'interface ifname
        tiré du python cookbook
        """
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        return socket.inet_ntoa(fcntl.ioctl(
            s.fileno(),
            0x8915,  # SIOCGIFADDR
            struct.pack('256s', ifname[:15])
        )[20:24])

    def _get_interface_list(self):
        """
        retourne les interfaces reseau de la machine et leur IP associee
        tiré du python cookbook, legerement modifie
        """
        result = []
        max_possible = 128  # arbitrary. raise if needed.
        bytes = max_possible * 32
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        names = array.array('B', '\0' * bytes)
        outbytes = struct.unpack('iL', fcntl.ioctl(
            s.fileno(),
            0x8912,  # SIOCGIFCONF
            struct.pack('iL', bytes, names.buffer_info()[0])
            ))[0]
        namestr = names.tostring()
        #liste des interfaces
        interface_list = [namestr[i:i+32].split('\0', 1)[0] for i in range(0, outbytes, 32)]
        #pour chaque interface, on obtient son adresse IP
        for inter in interface_list:
            result.append(self._get_ip_address(inter))
        return result


    def _check_local_ip(self,ip):
        """
        verifie si l'adresse IP de la requete est une adresse locale (ie une adresse
        d'une des carte reseau ou l'adresse de loopback) ou une autre adresse
        """
        return ip in self.ip_list

    def render(self, request):
        try:
            request.content.seek(0, 0)
            args, functionPath = xmlrpclib.loads(request.content.read())
            client_ip = request.getClientIP()
            try:
                function = self._getFunction(functionPath)
            except Fault, f:
                self._cbRender(f, request)
            else:
                request.setHeader("content-type", "text/xml")
                defer.maybeDeferred(function,client_ip, *args).addErrback(
                    self._ebRender
                ).addCallback(
                    self._cbRender, request
                )
        except:
            log.msg("----> Erreur dans la fonction render, la requête ne semble pas être de type xmlrpc")
            request.setHeader("content-length", str(len('Erreur')))
            request.write('Erreur')
            request.finish()
        return server.NOT_DONE_YET


    def _log_action_execution(self, action_name, magic_nb, ip):

        try:
            data = self.magic_number_dict[magic_nb]
            if data[1] == '':
                username = data[2]
            else:
                if data[1].has_key('uid'):
                    username = data[1]['uid'][0]
                elif data[1].has_key('cn'):
                    username = data[1]['cn'][0]
                else:
                    username=_("Unknown")
        except:
            username=_("Unknown")
        log.msg(("Execution of : <%s> by <%s> from <%s> (%s)")%(action_name,username,socket.getfqdn(ip),ip))

    def _get_user_description(self, ip, ldapentry, roles):
        """ renvoie un dico de description de l'utilisateur
            renvoie le type de l'utilisateur, son nom, l'ip de son frontend
        """
        try:
            if ldapentry.has_key('uid'):
                if hasattr(ldapentry['uid'], '__iter__'):
                    username = ldapentry['uid'][0]
                else:
                    username = ldapentry['uid']
            else:
                if hasattr(ldapentry['pam'], '__iter__'):
                    username = ldapentry['pam'][0]
                else:
                    username = ldapentry['pam']
            roles = list(roles)
        except:
            username="Inconnu"
            roles = list('eleve')
        return {'ip':ip, 'name':username, 'role':roles}

    def _renew_magic_number(self, magic_number):
        """
        remet a 0 le timer d'annulation du magic number
        """
        if magic_number in self.magic_number_dict.keys():
            defered = self.magic_number_dict[magic_number][0]
            defered.cancel()
            defered = reactor.callLater(MAGIC_NUMBER_TIMEOUT, self._remove_magic_number, magic_number)
            self.magic_number_dict[magic_number][0] = defered

    def store_session_info(self, magic_number, (data, identifier), language):
        #met en place le timeout d'expiration du magic number
        defered = reactor.callLater(MAGIC_NUMBER_TIMEOUT, self._remove_magic_number, magic_number)
        #stocke le defered pour pouvoir l'annuler au bon moment
        #si infos == '', c'est qu'on est en authentification locale
        self.magic_number_dict[magic_number] = [defered, data, identifier]
        self._languages[magic_number] = language

    def _remove_magic_number(self, magic_number):
        """
        enleve un magic number du cache interne
        """
        if magic_number in self.magic_number_dict.keys():
            #ajoute au cache des sessions expirees
            if magic_number not in self.expired_magic_number:
                self.expired_magic_number.append(magic_number)
                #definit un timeout avant que la session expiree ne soit enlevee du cache des sessions expirees
                reactor.callLater(EXPIRED_SESSION_TIMEOUT, self._remove_expired_magic_number, magic_number)
            del(self.magic_number_dict[magic_number])

    def _remove_expired_magic_number(self, magic_number):
        """
        enleve un magic number du cache des session expirees
        """
        if magic_number in self.expired_magic_number:
            self.expired_magic_number.remove(magic_number)
            del(self._languages[magic_number])

    def xmlrpc_unregister_frontend(self, client_ip, key, username, password):
        """
        supprime une clé permettant a un frontend de se connecter au backend
        on doit fournir key qui est la clé que le backend avait précédement fournit au frontend
        on verifiera qu'on a bien client_ip : key dans le dico avant de supprimer
        """
        if not self.frontend_key_dict.has_key(client_ip):
            return 5
        else:#if self.frontend_key_dict[client_ip] == key:
            self._gain_privileges()
            ok = self.local_auth.authenticate(username, password)
            self._drop_privileges()
            if ok:
                #on supprime la clé de notre dico
                del(self.frontend_key_dict[client_ip])
                # sauvegarde du fichier des clés (on reprend les droits root pour écrire le fichier)
                self._gain_privileges()
                if self.key_parser.write_file(join(CONFIG_DIR, FRONTEND_KEYS_FILE),self.frontend_key_dict) == False:
                    self._drop_privileges()
                    return 1
                else:
                    self._drop_privileges()
                    return 0
            else:
                return 2
    #    else:
    #        return 4


    def xmlrpc_register_frontend(self, client_ip, username, password, language):
        """
        crée une clé pour permettre à une adresse ip d'accéder au backend
        le parametre keep indique si on doit garder la clé
        on peut alors se servir de cette fonction pour verifier un couple login/mdp
        sans garder de nouvelle cle
        """
        log.msg("#### Enregistrement depuis un frontend")
        self._gain_privileges()
        log.msg("++++ client_ip ---> %s, username ---> %s"%(client_ip, username))
        a = self.local_auth.authenticate(username, password)
        self._drop_privileges()
        if a:
            #ok, authentification locale reussie
            #si on a deja une cle pour ce serveur on l'utilise
            if self.frontend_key_dict.has_key(client_ip):
                log.msg("====> Erreur ----> Ce frontend est déjà enregistré dans le backend")
                key = self.frontend_key_dict[client_ip]
                return 0, key
            #sinon on en genere une nouvelle, on la sauvegarde
            else:
                key = create_magicnb(password, username)
                self.frontend_key_dict[client_ip] = key
                # sauvegarde du fichier des clés (on reprend les droits root pour écrire le fichier)
                self._gain_privileges()
                if self.key_parser.write_file(join(CONFIG_DIR, FRONTEND_KEYS_FILE),self.frontend_key_dict) == False:
                    log.msg("====> Erreur ----> le backend n'a pas pu écrire la référence pour le frontend à enregistrer")
                    self._drop_privileges()
                    return 1, uni(_("error when saving the key", language))

                else:
                    log.msg("====>  L'enregistrement s'est bien passé")
                    self._drop_privileges()
                    return 0, key
        else:
            log.msg("====> Erreur ----> Authentification locale échouée")
            return 1, uni(_("authentification error", language))

    def xmlrpc_get_magic_number(self, client_ip, key, app_ticket, app_path, language):
        """
        fournit un magic number a un front end si son ticket d'application est valide
        """
        #on est paranoiaque, on verifie que le ticket d'application que l'on a envoye est
        #bien celui que l'on nous retourne. C'est une petite verification de securite supplementaire
        key_ok = False
        # vérification de la clé du frontend
        if self.frontend_key_dict.has_key(client_ip):
            if self.frontend_key_dict[client_ip] == key:
                key_ok = True
        if not key_ok:
            return ''
        try:
            result, infos = self.auth_server.get_user_details(app_ticket, app_path)
        except:
            traceback.print_exc()
            return ''
        #si ticket d'application valide
        if result:
            present = self._check_local_ip(client_ip)
            magic_number = create_magicnb(app_ticket, app_path)
            #met en place le timeout d'expiration du magic number
            self.store_session_info(magic_number, (infos, app_path), language)
            return magic_number
        else:
            return ''

    def xmlrpc_local_authentification(self, client_ip, key, username, password, keep, language):
        """
        fournit un magic number a un front end si username/password correspond a un couple name/password local
        cette fonction est appelée dans le cas ou le serveur d'authentification est injoignable
        """
        if username not in self.allowed_users:
            return ''
        if self.frontend_key_dict.get(client_ip) != key:
            return ''
        self._gain_privileges()
        ok = self.local_auth.authenticate(username, password)
        self._drop_privileges()
        if ok:
            #ok, authentification locale reussie
            magic_number = create_magicnb(password, username)
            #si on garde le magic number
            if keep:
                self.store_session_info(magic_number, ({'pam':username}, username), language)
            return magic_number
        else:
            return ''

    def xmlrpc_change_local_passwd(self, client_ip, username, old_password, new_password):
        """
        change le mot de passe local de username  : old_password-->new_password
        """
        res = self.xmlrpc_local_authentification(client_ip, username, old_password, False)
        # ancien login/mdp sont bon
        if res != '':
            crypted_pass = sha.sha(new_password).hexdigest()
            #on sauvegarde l'ancien mot de passe en cas d'erreur pour pouvoir le restaurer
            sauv_passwd = self.local_passwd[username]
            self.local_passwd[username] = crypted_pass
            #on reprend nos droits
            self._gain_privileges()
            #ecriture dans le fichier
            res = self.passwd_parser.write_file(PASSWD_FILE, self.local_passwd)
            #on rejette nos privileges
            self._drop_privileges()
            #erreur d'ecriture dans le fichier de mot de passe
            if res != True:
                #on restaure le mot de passe
                self.local_passwd[username] = sauv_passwd
                return 2, unicode(_("Error when writing password file, password unchanged"),'ISO-8859-1')
            else:
                #ok, tout s'est bien passé
                return 0,""
        else:
            return 1, unicode(_("Wrong login/password"),'ISO-8859-1')

    def _check_magic_number(self, magic_number):
        #verifie que le magic number est bon
        if magic_number in self.magic_number_dict.keys():
            #remet le timeout a 0
            self._renew_magic_number(magic_number)
            return True
        else:
            return False

    def _check_expired_magic_number(self, magic_number):
        """
        regarde si le magic number a deja ete utilise mais qu'il a expire
        """
        return not magic_number in self.expired_magic_number

    def xmlrpc_get_menu(self, client_ip, magic_number):
        """Renvoie la liste des actions que <username> peut effectuer"""
        ret_code, message = self._validate_session(magic_number)
        if ret_code != 1 :
            return ret_code, message
        language = self._languages[magic_number]
        present = self._check_local_ip(client_ip)
        #log de l'execution
        self._log_action_execution("get_menu",magic_number, client_ip)
        infos = self.magic_number_dict[magic_number][1]
        #authentification distante, on a des infos
        if infos != '':
            # calcul des roles
            roles = self.role_parser.get_roles(infos)
        else:
            #authentification locale, on 'construit' notre role
            roles = Set([self.magic_number_dict[magic_number][2]])
        try:
            actions_menu = {}
            actions = {}
            # dictionnaire catégorie/actions
            for act in self.perm_manager.authorized_actions_for(roles):
                act_infos = (act.name, act.description, act.get_params(), act.libelle)
          #     les actions sans categorie ne doivent pas apparaitre dans le menu
                if act.category is not None:
                    actions_menu.setdefault(act.category, []).append(act_infos)
                actions.setdefault(act.category, []).append(act_infos)
            if actions == {}:
                log.err("===> Erreur ---> get_menu le menu est vide")
        except Exception, e:
            log.msg("erreur :", e)
            actions = {}
        # FIXME ajouter le menu des agents
        menu={'menu':self._recurs_menu(actions_menu),
              'actions': self._format_action(actions, language),
              'role':self._get_user_description(client_ip, infos, roles),
             }
        return 0, menu

    def _format_action(self, dict, language):
        """
        pour formater le menu des actions
        """
        result = {}
        for key, value in dict.items():
            for script in value:
                #XXX gerer l'i18n ici pur description et pour description de param
                nom, description, param, libelle = script
                # on internationalise la description des parametres
                param_list = []
                #on internationalise la description de l'action
                desc = _(description, language)
                libe = _(libelle, language)
                if not isinstance(desc,unicode):
                    desc = uni(desc)
                if not isinstance(libe,unicode):
                    libe = uni(libe)
                for par in param:
                    nom_param, description_param, type_param , default_param, param_data = par
                    param_list.append((nom_param, uni(_(description_param,language)), type_param, default_param, param_data))
                result[str(nom)] = (desc, param_list, libe)
        return result

    def _recurs_menu(self, dict):
        """
        permet de construire la structure de donnes (range par catgories)
        @param dict: dictionaire d'entre (avec des clefs du types 'syst/general')
        @return: la structure de donne arborescence (cf doc/menu.html)
        """
        dico = {}
        tmp = {}
        for cle, value in dict.items():
            # parse la clef (il peut s'agir d'un chemin)
            chemin = cle.split('/')

            if not dico.has_key(chemin[0]):
                # si la catgorie n'existe pas dj
                dico[chemin[0]] = [[],{}]

            if len(chemin) == 1:
                # il n'y a pas de sous-catgorie
                for val in value:
                    # on construit la liste des noms d'actions pour cette catgorie
                    dico[chemin[0]][0].append(val[0])
            else:
                # il y a des sous-catgories
                if not tmp.has_key(chemin[0]):
                    # c'est une nouvelle sous-catgorie
                    tmp[chemin[0]] = {}
                reste = '/'.join(chemin[1:])

                if not tmp[chemin[0]].has_key(reste):
                    # c'est une nouvelle sous-catgorie
                    tmp[chemin[0]][reste] = []
                for val in value:
                    # construit la liste des actions pour cette sous-catgorie
                    tmp[chemin[0]][reste].append(val)

        for cle, value in tmp.items():
            dico[cle][1] = self._recurs_menu(value)

        return dico

    def _validate_session(self, magic_number):
        #si magic_number non enregistre en local
        if not self._check_magic_number(magic_number):
            return 4, uni(_("no session registered on this server, please connect first"))
        language = self._languages[magic_number]
        #si magic_number est une session expiree
        if not self._check_expired_magic_number(magic_number):
            return 3, uni(_("session expired, please reconnect",language))
        return 1, "OK"

    def xmlrpc_execute_action(self, client_ip, action_name, params, magic_number):
        """Exécute l'action"""
        log.msg("appel xmlrpc pour: %s"%action_name)
        ret_code, message = self._validate_session(magic_number)
        if ret_code != 1 :
            return ret_code, message
        language = self._languages[magic_number]
        try:
            # on récupère la classe de l'action (dans REGISTERED_ACTIONS)
            action_class = get_action(action_name)
            # on l'instancie
            action = action_class()
        except:
            try:
                # FIXME
                # insertion d'actions specifiques a un plugin donne
                action_class = get_full_registered_actions()[action_name]
                action = action_class()
            except:
                return 1,uni(_("unknown action",language))
        present = self._check_local_ip(client_ip)
        infos = self.magic_number_dict[magic_number][1]
        # FIXME : déporter le log dans la classe Action
        #log de l'execution
        self._log_action_execution(action_name,magic_number, client_ip)
        #authentification distante, on a des infos
        if infos != '':
            # calcul des roles
            roles = self.role_parser.get_roles(infos)
        else:
            #authentification locale, on 'construit' notre role
            roles = Set([self.magic_number_dict[magic_number][2]])
        if action_class not in self.perm_manager.authorized_actions_for(roles):
            return 1, uni(_("unauthorised action",language))
        else:
            #on reprend nos privileges
            self._gain_privileges()
            #on execute la fontion
            #return_code, result = action.execute(params)
        ## passage d'infos utilisateurs à l'action (permet la gestion de droits à l'intérieur de l'action)
            user = {'user_description':self._get_user_description(client_ip, infos, roles)}
            params.update(user)
            ## lancement de l'éxécution de l'action dans un callback
            ## pour éviter la perte de droit pour les actions différées
            _deferred = defer.Deferred()
            _deferred.callback(params)
            _deferred.addCallback(action.execute)
            _deferred.addCallback(self._deferred_drop_and_return)
            _deferred.addErrback(self._deferred_error)
            return _deferred

    def _deferred_error(self, err):
        self._drop_privileges()
        log.msg("Erreur d'execution : %s"%err)
        return 1, uni("Erreur d'éxécution")

    def _deferred_drop_and_return(self, ret):
        self._drop_privileges()
        return ret

    def xmlrpc_get_table(self, client_ip, magic_number):
        present = self._check_local_ip(client_ip)
        fic_perm = open(join(dirname(__file__), 'perm_amon.ini'))
        perm_string = fic_perm.read()
        fic_perm.close()
        # On renvoie des chaines unicode (les fichiers .ini sont en latin-1)
        return unicode(perm_string, 'utf-8')

    def xmlrpc_is_alive(self, client_ip):
        """
        pour dire a un client que l'on est vivant
        """
        return ''

    def xmlrpc_get_module_name(self, client_ip, magic_number):
        """ renvoit le nom du module du backend """
        ret_code, message = self._validate_session(magic_number)
        if ret_code != 1 :
            return ret_code, message
        return 1, get_module_name()

    def _get_template_from_disk(self, template_name):
        """:template_name: le nom (basename sans extenstion) du fichier de template
        :return: string de template ou error si template non trouve
        """
        templatefile = os.path.join(TEMPLATE_DIR, template_name + '.tmpl')
        if os.path.isfile(templatefile):
            return file(templatefile, 'r').read()
        raise AssertionError

    def xmlrpc_get_template(self, client_ip, template_name, magic_number):
        """regarde sur le disque si le template existe
        et retourne une string
        """
        ret_code, message = self._validate_session(magic_number)
        if ret_code != 1 :
            return ret_code, message
        try:
            tplstr = self._get_template_from_disk(template_name)
            # available_templates = {'exemple': "<p>%%variable</p>"}
            # return 0, available_templates[template_name]
            return 0, uni(str(tplstr))
        except AssertionError, e:
            # if not available_templates.has_key(template_name):
            return 1, "no available template"

def run():
    #on est pas root
    if os.getuid() != 0:
        import sys
        sys.exit(_("You have to launch it as root"))
    from twisted.internet import reactor
    reactor.listenSSL(BACKEND_LISTEN_PORT, server.Site(Amon()),ServerContextFactory())
    log.msg("\n"+_("launching server on port %s")%str(BACKEND_LISTEN_PORT))
    reactor.run()

if __name__ == '__main__':
    run()
